/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.samples {


using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.lw.grid;
using org.zaval.lw;
using org.zaval.port.cs;


public class LwPropertiesViewer:LwPanel
{
  public LwPropertiesViewer(Hashtable p)
  {
    TreeModel  treeModel = createTreeModel(p);
    LwTreeGrid treeGrid  = createTreeGrid (treeModel);
    createTreeGridModel(treeGrid);
    for (int i=0; i < treeGrid.getCols(); i++)
      treeGrid.setColWidth(i, 180);
    treeGrid.setViewProvider(new LwCellViewProvider());
    LwScrollPan sp = new LwScrollPan(treeGrid);
    add (LwBorderLayout.CENTER, sp);
  }

  public void createTreeGridModel(LwTreeGrid grid)
  {
    MatrixModel matrixModel = grid.getModel();
    TreeModel   treeModel   = grid.getTreeModel();

    int row = 2;
    Item man = treeModel.getChildAt(treeModel.getRoot(), 0);
    for (int i=0;i<treeModel.getChildrenCount(man); i++)
    {
      Item item = treeModel.getChildAt(man, i);
      object obj = LwManager.getStaticObject((string)item.getValue());
      matrixModel.put (row, 1, System.ComponentModel.TypeDescriptor.GetClassName(obj));
      row++;
    }

    Item sobj = treeModel.getChildAt(treeModel.getRoot(), 1);
    row++;
    for (int i=0;i<treeModel.getChildrenCount(sobj); i++)
    {
      Item item = treeModel.getChildAt(sobj, i);
      object obj = LwManager.getStaticObject((string)item.getValue());
      matrixModel.put (i + row, 1, System.ComponentModel.TypeDescriptor.GetClassName(obj));
      matrixModel.put (i + row, 2, obj);
    }
  }

  public LwTreeGrid createTreeGrid(TreeModel model)
  {
    LwTreeGrid treeGrid = new LwTreeGrid(model);
    LwGridCaption caption = new LwGridCaption(treeGrid);
    caption.putTitle (0, "Object Key");
    caption.putTitle (1, "Class Name");
    caption.putTitle (2, "View");
    treeGrid.add (LwGrid.TOP_CAPTION_EL, caption);
    return treeGrid;
  }

  public TreeModel createTreeModel(Hashtable p)
  {
    Item root = new Item("root");
    TreeModel treeModel = new Tree(root);
    Item man  = new Item("Managers");
    Item sobj = new Item("Static Objects");
    treeModel.add (root, man);
    treeModel.add (root, sobj);

    char[]   seps   = {','};
    string[] values = ((string)p.get("man")).Split(seps);

    for (int i=0; i<values.Length; i++)
      treeModel.add(man, new Item(values[i]));

    values = ((string)p.get("obj")).Split(seps);
    for (int i=0; i<values.Length; i++)
      treeModel.add(sobj, new Item(p.get("obj." + values[i] + ".key")));

    return treeModel;
  }

  public override LwLayout getDefaultLayout() {
    return new LwBorderLayout();
  }


  public static void Main (string[] args)
  {
    LwFrame frame = new LwFrame();
    frame.setSize(500, 500);
    LwContainer root = frame.getRoot();
    root.setLwLayout(new LwBorderLayout());
    LwPropertiesViewer pv = new LwPropertiesViewer(LwToolkit.getProperties("rs/lw.properties"));
    root.add (LwBorderLayout.CENTER, pv);
    frame.setVisible(true);
  }

}

class LwCellViewProvider:LwDefViews
{
  LwCanvas     target;
  LwCompRender render;

  public LwCellViewProvider() {
    target = new LwCanvas();
    render = new LwCompRender(target);
  }

  public override LwView getView(int row, int col, object obj)
  {
    if (col == 2)
    {
      if (obj is LwView)
      {
        LwView view = (LwView)obj;
        if (view.getType() != LwView.ORIGINAL)
        {
          target.getViewMan(true).setView(view);
          Dimension d = view.getPreferredSize();
          target.setPSSize(d.width, d.height);
          return render;
        }
        else return view;
      }
      else return base.getView(row, col, "---");
    }
    else
    if (col == 1) return base.getView(row, col, obj);
    return null;
  }
}



}
