/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.data {


using org.zaval.data.events;
using org.zaval.port.cs;
using org.zaval.util;


public class Matrix:MatrixModel
{
   private object[][] objs;
   private int        rows, cols, alloc_rows, alloc_cols;
   private Vector support;

   public Matrix(int rows, int cols) {
     setSize(rows, cols);
   }

   public int getRows () {
     return rows;
   }

   public int getCols () {
     return cols;
   }

   public void setRows (int rows) {
     setSize(rows, cols);
   }

   public void setCols (int cols) {
     setSize(rows, cols);
   }

   public void setSize (int rows, int cols)
   {
     if (rows != this.rows || cols != this.cols)
     {
       if (alloc_cols < cols||
           alloc_rows < rows  )
       {

          object[][] nobjs = new object[rows][];
          for (int i=0;i<rows;i++) nobjs[i] = new object[cols];

          alloc_cols = cols;
          alloc_rows = rows;
          if (objs != null) copy (objs, nobjs);
          objs = nobjs;
       }
       else
       {
         if (cols < this.cols)
         {
           for (int i=cols; i<this.cols; i++)
             for (int j=0; j<this.rows; j++)
               objs[j][i] = null;
         }

         if (rows < this.rows)
         {
           for (int i=0; i<cols; i++)
             for (int j=rows; j<this.rows; j++)
               objs[j][i] = null;
         }
       }

       int pc = this.cols;
       int pr = this.rows;
       this.cols = cols;
       this.rows = rows;

       if (pc != cols || pr != rows) perform (new MatrixEvent(this, pr, pc));
     }
   }

   public Dimension getSize () {
     return new Dimension (rows, cols);
   }

   public void put(int row, int col, object obj)
   {
      int nr = getRows();
      int nc = getCols();
      if (row >= nr) nr += (row - nr + 1);
      if (col >= nc) nc += (col - nc + 1);
      setSize(nr, nc);

      object old = objs[row][col];
      if ((obj == null && obj != objs[row][col]      )||
          (obj != null && !obj.Equals(objs[row][col]))  )
      {
        objs[row][col] = obj;
        perform(new MatrixEvent(this, row, col, old));
      }
   }

   public Point put(int index, object obj)
   {
      Point p = MathBox.index2point(index, getCols());
      put (p.x, p.y, obj);
      return p;
   }

   public object get(int row, int col) {
     return objs[row][col];
   }

   public object get(int index) {
     Point p = MathBox.index2point(index, getCols());
     return objs[p.x][p.y];
   }

   public void removeRows(int begrow, int count)
   {
     int from = begrow;
     for (int i=(begrow + count - 1); i<rows; i++, begrow++)
     {
       for (int j=0; j<cols; j++)
       {
         objs[begrow][j] = objs[i][j];
         objs[i][j] = null;
       }
     }
     rows -= count;
     perform(new MatrixEvent(this, rows + count, cols));
   }

   public void removeCols(int begcol, int count)
   {
     int from = begcol;
     for (int i=(begcol+count); i<cols; i++, begcol++)
     {
       for (int j=0; j<rows; j++)
       {
         objs[j][begcol] = objs[j][i];
         objs[j][i]      = null;
       }
     }
     cols -= count;
     perform(new MatrixEvent(this, rows, cols + count));
   }

   public void addMatrixListener (MatrixListener m) {
     if (support == null) support = new Vector(1);
     if (!support.contains(m)) support.addElement(m);
   }

   public void removeMatrixListener (MatrixListener m) {
     if (support != null) support.removeElement(m);
   }

   public void perform(MatrixEvent e)
   {
     if (support != null)
     {
       for (int i=0; i<support.size(); i++)
       {
         MatrixListener l = (MatrixListener)support.elementAt(i);
         if (e.getID() == MatrixEvent.MATRIX_RESIZED) l.matrixResized(e);
         else l.cellModified(e);
       }
     }
   }

   private static void copy (object[][] s, object[][] d)
   {
      Dimension sd = arraySize(s);
      Dimension dd = arraySize(d);

      int rows = System.Math.Min(sd.height, dd.height);
      int cols = System.Math.Min(sd.width,  dd.width );

      for (int i=0; i<rows; i++)
        for (int j=0; j<cols; j++)
          d[i][j] = s[i][j];
   }

   private static Dimension arraySize(object[][] s)
   {
     if (s == null) return new Dimension();
     Dimension d = new Dimension();
     d.height= s.Length;
     d.width = (s.Length==0)?0:((object[])s[0]).Length;
     return d;
   }
}

}
