/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.data {


using org.zaval.data.events;
using org.zaval.port.cs;


public class SingleLineTxt:TextModel
{
  private System.Text.StringBuilder buf;
  private int          maxLen, extra;

  private TextListenerSupport support;

  public SingleLineTxt(string s):this(s, -1) {
   
  }

  public SingleLineTxt(string s, int max) {
    maxLen = max;
    setText(s==null?"":s);
  }

  public string getLine (int line) {
    return buf.ToString();
  }

  public void setText (string text)
  {
    string old = getText();
    if (old == null || !old.Equals(text))
    {
      if (old != null)
      {
        buf = null;
        perform (TextEvent.REMOVED, 0, getTextLength());
      }

      buf = (maxLen > 0 && text.Length > maxLen)? new System.Text.StringBuilder(text.Substring(0, maxLen - 0))
                                                  : new System.Text.StringBuilder(text);
      perform(TextEvent.INSERTED, 0, getTextLength());
    }
  }

  public string getText () {
    return (buf == null)?null:buf.ToString();
  }

  public int getSize() {
    return (buf==null)?0:1;
  }

  public void write (string s, int offset)
  {
    if (maxLen <= 0 || getTextLength() + s.Length <= maxLen)
    {
      buf.Insert(offset, s);
      perform(TextEvent.INSERTED, offset, s.Length);
    }
  }

  public void write (char ch, int offset) {
    write (UtilBox.valueOf(ch), offset);
  }

  public void remove (int offset, int size)
  {
    string s = buf.ToString();
    s = s.Substring(0, offset - 0) + s.Substring(offset + size);
    buf = new System.Text.StringBuilder(s);
    perform(TextEvent.REMOVED, offset, size);
  }

  void perform (int id, int offset, int size)
  {
    if (support != null)
    {
      TextEvent e = new TextEvent(this, id, offset, size);
      e.setUpdatedLines(0, 1);
      support.perform(e);
    }
  }

  public void addTextListener(TextListener l) {
    if (support == null) support = new TextListenerSupport();
    support.addListener(l);
  }

  public void removeTextListener(TextListener l) {
    if (support != null) support.removeListener(l);
  }

  public int getTextLength() {
    return (buf == null)?-1:buf.Length;
  }

  public int getExtraChar (int i) {
    return extra;
  }

  public void setExtraChar (int i, int ch) {
    extra = ch;
  }
}


}
