/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.data {


using org.zaval.data.events;
using org.zaval.port.cs;


public class Text:TextModel
{
  private const  int EXTRA_SIZE = 1;

  private Vector lines;
  private int    textLength;
  private TextListenerSupport support;

  public Text(string s) {
    setText(s==null?"":s);
  }

  public string getLine (int line) {
    char[] buf = (char[])lines.elementAt(line);
    return new string(buf, 0, buf.Length - EXTRA_SIZE);
  }

  public void setText (string text)
  {
    string old = getText();
    if (old == null || !old.Equals(text))
    {
      if (old != null)
      {
        TextEvent e = new TextEvent(this, TextEvent.REMOVED, 0, getTextLength());
        e.setUpdatedLines(0, 0);
        lines.removeAllElements();
        lines = null;
        perform (e);
      }
      lines = parse(text.ToCharArray());
      textLength = text.Length;
      TextEvent ee = new TextEvent(this, TextEvent.INSERTED, 0, getTextLength());
      ee.setUpdatedLines(0, getSize());
      perform(ee);
    }
  }

  public string getText ()
  {
    if (lines == null || lines.size() == 0) return null;
    System.Text.StringBuilder buf = new System.Text.StringBuilder();
    for (int i=0; i<lines.size(); i++)
    {
      if (i > 0) buf.Append('\n');
      char[] ch = (char[])lines.elementAt(i);
      buf.Append(ch, 0, ch.Length - EXTRA_SIZE);
    }
    return buf.ToString();
  }

  public int getSize() {
    return (lines == null)?0:lines.size();
  }

  public void write (string s, int offset)
  {
    int    len     = s.Length;
    int[]  info    = getLnInfo(0, 0, offset);

    char[] line    = (char[])lines.elementAt(info[0]);
    int    length  =  line.Length - EXTRA_SIZE;

    char[] tmp      = new char[length + s.Length];
    int    lnOffset = offset - info[1];
    System.Array.Copy(line, 0, tmp, 0, lnOffset);
    System.Array.Copy(s.ToCharArray(), 0, tmp, lnOffset, len);
    System.Array.Copy(line, lnOffset, tmp, lnOffset + len, length - lnOffset);
    Vector v = parse(tmp);
    lines.removeElementAt(info[0]);
    for (int i=0; i<v.size(); i++)
      lines.insertElementAt(v.elementAt(i), info[0] + i);

    textLength += len;
    TextEvent e = new TextEvent(this, TextEvent.INSERTED, offset, s.Length);
    e.setUpdatedLines(info[0], v.size());
    perform(e);
  }

  public void write (char ch, int offset) {
    write (UtilBox.valueOf(ch), offset);
  }

  public void remove (int offset, int size)
  {
    int[]  i1 = getLnInfo (0, 0, offset);
    int[]  i2 = getLnInfo (i1[0], i1[1], offset + size);
    char[] line1 = (char[])lines.elementAt(i1[0]);
    System.Text.StringBuilder buf = new System.Text.StringBuilder();
    buf.Append(line1, 0, line1.Length - EXTRA_SIZE);
    for (int i=i1[0] + 1; i<i2[0]+1; i++)
    {
      buf.Append('\n');
      char[] bb =( char[])lines.elementAt(i);
      buf.Append(bb, 0, bb.Length - EXTRA_SIZE);
    }

    char[] tmp      = new char[buf.Length - size];
    int    lnOffset = offset - i1[1];

    UtilBox.getChars(0, lnOffset, tmp, 0, buf);
    if (lnOffset + size < buf.Length)
      UtilBox.getChars(lnOffset + size, buf.Length, tmp, lnOffset, buf);

    for (int i=i1[0]; i<i2[0]+1; i++)
      lines.removeElementAt(i1[0]);

    Vector v = parse(tmp);
    for (int i=0; i<v.size(); i++)
      lines.insertElementAt(v.elementAt(i), i1[0] + i);

    textLength -= size;
    TextEvent e = new TextEvent(this, TextEvent.REMOVED, offset, size);
    e.setUpdatedLines(i1[0], (i1[0]!=i2[0] && (i1[1]-offset)==0 && (i2[1]-offset-size)==0)?0:1);
    perform(e);
  }

  public Vector parse(char[] buffer)
  {
    int    size = 0, offset = 0;
    Vector v = new Vector(3);

    while (offset < buffer.Length)
    {
      if (buffer[offset] == '\n')
      {
        char[] line = new char[size + EXTRA_SIZE];
        System.Array.Copy(buffer, offset - size, line, 0, size);
        v.addElement(line);
        size = 0;
      }
      else size++;
      offset++;
    }

    if (offset >= buffer.Length)
    {
      char[] line = new char[size + EXTRA_SIZE];
      System.Array.Copy(buffer, offset - size, line, 0, size);
      v.addElement(line);
    }

    return v;
  }

  void perform (TextEvent e) {
    if (support != null) support.perform(e);
  }

  public void addTextListener(TextListener l) {
    if (support == null) support = new TextListenerSupport();
    support.addListener(l);
  }

  public void removeTextListener(TextListener l) {
    if (support != null) support.removeListener(l);
  }

  public int getTextLength() {
    return textLength;
  }

  public int getExtraChar (int i) {
    char[] line = (char[])lines.elementAt(i);
    return (int) line[line.Length-1];
  }

  public void setExtraChar (int i, int ch) {
    char[] line = (char[])lines.elementAt(i);
    line[line.Length-1] = (char)ch;
  }

  private int[] getLnInfo (int start, int startOffset, int o)
  {
    int offset = startOffset;
    for (int i=start; i<lines.size(); i++)
    {
      char[] buf = (char[])lines.elementAt(i);
      if (o >= offset && o <= offset + buf.Length - EXTRA_SIZE) return new int[] { i, offset };
      offset += (buf.Length + 1 - EXTRA_SIZE);
    }
    return null;
  }
}


}
