/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.misc;
using org.zaval.port.cs;



public class LwActContainer:LwPanel,LwComposite
{
  private Color        rectColor = Color.gray;
  private LwComponent  focusComponent;

  public LwActContainer(LwComponent t)
  {
    if (t != null)
    {
      setInsets(4,4,4,4);
      add (t);
      setAsFocusComponent(0);
    }
  }

  public override bool canHaveFocus() {
    return true;
  }

  public LwComponent getFocusComponent() {
    return focusComponent;
  }

  public void setAsFocusComponent(int index) {
    focusComponent = (index >= 0)?(LwComponent)get(index):null;
  }

  public void setRectColor(Color c)
  {
    if (c != null && rectColor.Equals(c)) return;
    rectColor = c;
    repaint();
  }

  public Color getRectColor() {
    return rectColor;
  }

  public virtual bool catchInput(LwComponent child) {
    return true;
  }

  public override void paintOnTop (Graphics g)
  {
    if (focusComponent != null && hasFocus())
    {
      g.setColor(rectColor);
      g.drawRect(focusComponent.getX(), focusComponent.getY(),
                 focusComponent.getWidth()-1, focusComponent.getHeight()-1);
    }
  }

  public override void remove(int i) {
    if (get(i) == focusComponent) focusComponent = null;
    base.remove(i);
  }

  public override void removeAll() {
    focusComponent = null;
    base.removeAll();
  }

  public override LwLayout getDefaultLayout ()
  {
    LwFlowLayout l = new LwFlowLayout(Alignment.CENTER, Alignment.CENTER);
    l.setGaps (2, 2);
    return l;
  }
}

}
