/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.port.cs;


public class LwBorder:LwView
{
    public const  int RAISED = 1;

    public const  int SUNKEN = 2;

    public const  int ETCHED = 3;

    public const  int PLAIN = 4;

    public const  int DOT = 5;

    public const  int SUNKEN2 = 6;


    private int borderType;

    public LwBorder(int type):base(STRETCH) {
     
      setBorderType(type);
    }

    public int getBorderType() {
      return borderType;
    }

    public virtual void leftLine(Graphics g, int x1, int y1, int x2, int y2)
    {
      g.setColor(Color.gray);
      switch(borderType)
      {
        case DOT:
        {
          LwToolkit.drawDotVLine(g, y1, y2, x1);
        } break;
        case PLAIN:
        {
          g.setColor(Color.black);
          g.drawLine(x1, y1, x1, y2);
        } break;
        case ETCHED:
        {
          g.drawLine(x1, y1, x1, y2-1);
          g.setColor(Color.white);
          g.drawLine(x1 + 1, y1 + 1, x1 + 1, y2 - 2);
        } break;
        case RAISED:
        {
          g.setColor(Color.white);
          g.drawLine(x1, y1, x1, y2 - 1);
        } break;
        case SUNKEN2:
        case SUNKEN:
        {
          g.drawLine(x1, y1, x1, y2 - 1);
          if (borderType == SUNKEN)
          {
            g.setColor(Color.black);
            g.drawLine(x1 + 1, y1 + 1, x1 + 1, y2 - 2);
          }
        } break;
      }
    }

    public virtual void rightLine(Graphics g, int x1, int y1, int x2, int y2)
    {
      g.setColor(Color.gray);
      switch(borderType)
      {
        case DOT: LwToolkit.drawDotVLine(g, y1, y2, x2); break;
        case PLAIN:
        {
          g.setColor(Color.black);
          g.drawLine(x2, y1, x2, y2);
        } break;
        case ETCHED:
        {
          g.drawLine(x2 - 1, y1, x2 - 1, y2 - 1);
          g.setColor(Color.white);
          g.drawLine(x2, y1 + 1, x2, y2);
        } break;
        case RAISED:
        {
          g.drawLine(x2, y1, x2, y2);
          //g.drawLine(x2 - 1, y1 + 1, x2 - 1, y2 - 1);
          //g.setColor(Color.black);
          //g.drawLine(x2, y1, x2, y2);
        } break;
        case SUNKEN2:
        case SUNKEN:
        {
          g.setColor(Color.white);
          g.drawLine(x2, y1, x2, y2);
        } break;
      }
    }

    public virtual void topLine(Graphics g, int x1, int y1, int x2, int y2)
    {
      g.setColor(Color.gray);
      switch(borderType)
      {
        case DOT:
        {
          LwToolkit.drawDotHLine(g, x1, x2, y1);
        } break;
        case PLAIN:
        {
          g.setColor(Color.black);
          g.drawLine(x1, y1, x2, y1);
        } break;
        case ETCHED:
        {
          g.drawLine(x1, y1, x2 - 1, y1);
          g.setColor(Color.white);
          g.drawLine(x1 + 1, y1 + 1, x2 - 2, y1 + 1);
        } break;
        case RAISED:
        {
          g.setColor(Color.white);
          g.drawLine(x1, y1, x2 - 1, y1);
        } break;
        case SUNKEN2:
        case SUNKEN:
        {
          g.drawLine(x1, y1, x2 - 1, y1);
          if (borderType == SUNKEN)
          {
            g.setColor(Color.black);
            g.drawLine(x1 + 1, y1 + 1, x2 - 2, y1 + 1);
          }
        } break;
      }
    }

    public virtual void bottomLine(Graphics g, int x1, int y1, int x2, int y2)
    {
      g.setColor(Color.gray);
      switch(borderType)
      {
        case DOT:
        {
          LwToolkit.drawDotHLine(g, x1, x2, y2);
        } break;
        case PLAIN:
        {
          g.setColor(Color.black);
          g.drawLine(x1, y2, x2, y2);
        } break;
        case ETCHED:
        {
          g.drawLine(x1, y2 - 1, x2 - 1, y2 - 1);
          g.setColor(Color.white);
          g.drawLine(x1 + 1, y2, x2, y2);
        } break;
        case RAISED:
        {

           g.drawLine(x1, y2, x2, y2);
          //g.drawLine(x1 + 1, y2 - 1, x2 - 1, y2 - 1);
          //g.setColor(Color.black);
          //g.drawLine(x1, y2, x2, y2);
        } break;
        case SUNKEN2:
        case SUNKEN:
        {
          g.setColor(Color.white);
          g.drawLine(x1, y2, x2, y2);
        } break;
      }
    }

    public override void paint(Graphics g, int x, int y, int w, int h, Drawable d)
    {
      int xx = x + w - 1;
      int yy = y + h - 1;
      leftLine  (g, x, y, xx, yy);
      rightLine (g, x, y, xx, yy);
      topLine   (g, x, y, xx, yy);
      bottomLine(g, x, y, xx, yy);
    }

    public override Insets getInsets() {
      return new Insets(2, 2, 2, 2);
    }

    public override Dimension calcPreferredSize() {
      return new Dimension (2, 2);
    }

    private void setBorderType(int t)
    {
      if(t != RAISED && t != SUNKEN && t != ETCHED && t != PLAIN && t != DOT && t != SUNKEN2)
        throw new org.zaval.port.cs.IllegalArgumentException();
      borderType = t;
    }
}

}
