/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.port.cs;


public class LwBorderLayout:LwLayout
{
    public static readonly  object NORTH  = 1;

    public static readonly  object SOUTH  = 2;

    public static readonly  object EAST   = 3;

    public static readonly  object WEST   = 4;

    public static readonly  object CENTER = 5;

    private int        hgap, vgap;
    private Layoutable north, west, east, south, center;

    public LwBorderLayout():this(0, 0) {
     
    }

    public LwBorderLayout(int hgap, int vgap) {
      this.hgap = hgap;
      this.vgap = vgap;
    }

    public int getHgap() {
      return hgap;
    }

    public void setHgap(int hgap) {
      this.hgap = hgap;
    }

    public int getVgap() {
      return vgap;
    }

    public void setVgap(int vgap) {
      this.vgap = vgap;
    }

    public void componentAdded(object id, Layoutable comp, int index)
    {
      if (CENTER.Equals(id)) center = comp;
      else
      if (NORTH.Equals(id)) north = comp;
      else
      if (SOUTH.Equals(id)) south = comp;
      else
      if (EAST.Equals(id)) east = comp;
      else
      if (WEST.Equals(id)) west = comp;
      else                 throw new org.zaval.port.cs.IllegalArgumentException();
    }

    public void componentRemoved (Layoutable lw, int index)
    {
      if (lw == center) center = null;
      else
      if (lw == north)  north = null;
      else
      if (lw == south) south = null;
      else
      if (lw == east)  east = null;
      else
      if (lw == west)  west = null;
    }

    public Dimension calcPreferredSize(LayoutContainer target)
    {
      Dimension dim = new Dimension();

      if (isVisible(east))
      {
        Dimension d = east.getPreferredSize();
        dim.width += d.width + hgap;
        dim.height = System.Math.Max(d.height, dim.height);
      }

      if (isVisible(west))
      {
        Dimension d = west.getPreferredSize();
        dim.width += d.width + hgap;
        dim.height = System.Math.Max(d.height, dim.height);
      }

      if (isVisible(center))
      {
        Dimension d = center.getPreferredSize();
        dim.width += d.width;
        dim.height = System.Math.Max(d.height, dim.height);
      }

      if (isVisible(north))
      {
        Dimension d = north.getPreferredSize();
        dim.width = System.Math.Max(d.width, dim.width);
        dim.height += d.height + vgap;
      }

      if (isVisible(south))
      {
        Dimension d = south.getPreferredSize();
        dim.width = System.Math.Max(d.width, dim.width);
        dim.height += d.height + vgap;
      }
      return dim;
    }

    public void layout(LayoutContainer target)
    {
      Insets  insets = target.getInsets();
      Point   offs   = target.getLayoutOffset();
      int top = insets.top + offs.y;
      int bottom = target.getHeight() - insets.bottom;
      int left = insets.left + offs.x;
      int right = target.getWidth() - insets.right;

      if (isVisible(north))
      {
          Dimension d = north.getPreferredSize();
          north.setLocation(left, top);
          north.setSize(right - left, d.height);
          top += d.height + vgap;
      }

      if (isVisible(south))
      {
          Dimension d = south.getPreferredSize();
          south.setLocation(left, bottom - d.height);
          south.setSize(right - left, d.height);
          bottom -= d.height + vgap;
      }

      if (isVisible(east))
      {
          Dimension d = east.getPreferredSize();
          east.setLocation(right - d.width, top);
          east.setSize(d.width, bottom - top);
          right -= d.width + hgap;
      }

      if (isVisible(west))
      {
          Dimension d = west.getPreferredSize();
          west.setLocation(left, top);
          west.setSize(d.width, bottom - top);
          left += d.width + hgap;
      }

      if (isVisible(center))
      {
          center.setLocation(left, top);
          center.setSize    (right - left, bottom - top);
      }
    }

    public static bool isVisible (Layoutable l) {
      return l != null && l.isVisible();
    }
}





}
