/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.misc;
using org.zaval.port.cs;


public class LwBorderPan:LwPanel,LwLayout, LwTitleInfo
{
  public static readonly  object CENTER = 1;

  public static readonly  object TITLE  = 2;

  private Layoutable label;
  private Layoutable center;
  private int titleAlignment, xAlignment, indent = 4;

  public LwBorderPan():this(null, null) {
   
  }

  public LwBorderPan(LwComponent title, LwComponent center)
  {
    getViewMan(true).setBorder(new LwTitledBorder(LwBorder.ETCHED, Alignment.CENTER));
    setTitleAlignment(Alignment.TOP);
    setXAlignment(Alignment.LEFT);
    if (title  != null) add (TITLE, title);
    if (center != null) add (CENTER, center);
  }

  public void componentAdded (object id, Layoutable lw, int index)
  {
    if (id.Equals(TITLE)) label = lw;
    else
    if (id.Equals(CENTER)) center = lw;
    else throw new org.zaval.port.cs.IllegalArgumentException();
  }

  public void  componentRemoved (Layoutable lw, int index) {
    if (lw == label) label = null;
    else if (lw == center) center = null;
  }

  public Dimension calcPreferredSize(LayoutContainer target)
  {
     Dimension ps = new Dimension();
     if (center != null && center.isVisible()) ps = center.getPreferredSize();

     if (label != null && label.isVisible())
     {
       Dimension lps = label.getPreferredSize();
       lps.width += (2*indent);
       ps.height += lps.height;
       ps.width   = System.Math.Max(ps.width, lps.width);
     }
     return ps;
  }

  public void layout(LayoutContainer target)
  {
    Insets targetInsets = getInsets();
    int    y = 0, x = 0, w = 0, h = 0;
    if (label != null && label.isVisible())
    {
      Dimension ps = label.getPreferredSize();
      if (getTitleAlignment() == Alignment.BOTTOM)
        y = height - targetInsets.bottom - ps.height;
      else
        y = targetInsets.top;

      if (xAlignment == Alignment.LEFT ) x = targetInsets.left + indent;
      else
      if (xAlignment == Alignment.RIGHT) x = width - targetInsets.right - ps.width - indent;
      else
        x = width/2 - ps.width/2;

      w = ps.width;
      h = ps.height;
      label.setSize(w, h);
      label.setLocation(x, y);
    }

    if (center != null && center.isVisible())
    {
      if (getTitleAlignment() == Alignment.BOTTOM)
      {
        center.setSize(width - targetInsets.right - targetInsets.left, height - targetInsets.top - System.Math.Max(height - y, targetInsets.bottom));
        center.setLocation(targetInsets.left, targetInsets.bottom);
      }
      else
      {
        center.setSize(width - targetInsets.right - targetInsets.left, height - targetInsets.bottom - System.Math.Max(y + h, targetInsets.top));
        center.setLocation(targetInsets.left, System.Math.Max(y + h, targetInsets.top));
      }
    }
  }

  public override LwLayout getDefaultLayout() {
    return this;
  }

  public Rectangle getTitleBounds() {
    if (label != null) return label.getBounds();
    else return getBounds();
  }

  public void setTitleAlignment(int a)
  {
    if (a != Alignment.TOP && a != Alignment.BOTTOM) throw new org.zaval.port.cs.IllegalArgumentException();
    if (a != titleAlignment)
    {
      titleAlignment = a;
      vrp();
    }
  }

  public void setXAlignment(int a)
  {
    if (a != Alignment.LEFT && a != Alignment.RIGHT && a != Alignment.CENTER) throw new org.zaval.port.cs.IllegalArgumentException();
    if (a != xAlignment)
    {
      xAlignment = a;
      vrp();
    }
  }

  public int getTitleAlignment() {
    return titleAlignment;
  }
}



}
