/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;


public class LwButton:LwActContainer,LwMouseListener, LwKeyListener, LwMouseMotionListener
{
  private bool pressed;
  private LwActionSupport support;

  public LwButton():this((LwComponent)null) {
   
  }

  public LwButton(string lab):this(new LwLabel(lab)) {
   
  }

  public LwButton(LwComponent t):base(t) {
   
    setPressed(false);
  }

  public void addActionListener(LwActionListener a) {
    if (support == null) support = new LwActionSupport();
    support.addListener(a);
  }

  public void removeActionListener(LwActionListener a) {
    if (support != null) support.removeListener(a);
  }

  public virtual void perform() {
    if (support != null) support.perform(new LwActionEvent(this));
  }

  public void mouseClicked(LwMouseEvent e) {}

  public void mousePressed(LwMouseEvent e)  {
    if (LwToolkit.isActionMask(e.getMask())) setPressed(true);
  }

  public void mouseReleased(LwMouseEvent e) {
    if (isPressed()) perform();
    setPressed(false);
  }

  public void keyPressed (LwKeyEvent e) {
    if (!isPressed() && (e.getKeyChar() == ' ' || e.getKeyCode() == LwConstants.VK_ENTER)) setPressed(true);
  }

  public void keyReleased(LwKeyEvent e)
  {
    if (isPressed())
    {
      setPressed(false);
      perform();
    }
  }

  public void focusLost(LwFocusEvent e) {
    setPressed(false);
  }

  public void mouseEntered(LwMouseEvent e) {}

  public void mouseExited(LwMouseEvent e) {
    setPressed(false);
  }

  public void startDragged(LwMouseMotionEvent e) {}
  public void endDragged  (LwMouseMotionEvent e) {}
  public void mouseMoved  (LwMouseMotionEvent e) {}

  public void mouseDragged(LwMouseMotionEvent e)
  {
    int x = e.getX();
    int y = e.getY();
    setPressed(x>=0&&x<width&&y>=0&&y<=height);
  }

  public void keyTyped(LwKeyEvent e) {}

  public override void viewManChanged()
  {
    LwViewMan s = getViewMan(true);
    if (isPressed ()) s.setView("button.on");
    else              s.setView("button.off");
  }

  public virtual void setPressed (bool b)
  {
    if (pressed == b && getViewMan(true).getView() != null) return;
    pressed = b;
    LwViewMan s = getViewMan(true);
    if (b) s.setView("button.on");
    else   s.setView("button.off");
    repaint();
  }

  public bool isPressed () {
    return pressed;
  }
}

}
