/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;
using org.zaval.util;


public class LwCanvas:org.zaval.util.ValidationObject,LwComponent
{
   private Color back;
   public long insets;
   public short bits = DEFAULT_BITS;

   public int x, y, width, height, psWidth = -1, psHeight = -1;

   public LwComponent parent;

   public LwViewMan skins;

   public virtual Insets getInsets()  {
     return getInsets(this);
   }

   public int getX() {
     return x;
   }

   public int getY() {
     return y;
   }

   public int getWidth() {
     return width;
   }

   public int getHeight() {
     return height;
   }

   public void setInsets (int top, int left, int bottom, int right)
   {
     Insets i1 = getInsets();
     if (!(new Insets (top, left, bottom, right)).Equals(i1))
     {
       insets = (top & 0xFFFF) + ((left & 0xFFFF) << 16);
       long l = (bottom & 0xFFFF) + ((right & 0xFFFF) << 16);
       insets += (l << 32);
       vrp();
     }
   }

   public Dimension getSize() {
     return new Dimension(width, height);
   }

   public virtual Rectangle getVisiblePart()
   {
     if (isVisible())
     {
       //???
       if (parent == null) return new Rectangle (0, 0, width, height);
       Rectangle pr  = parent.getVisiblePart();

       if (pr != null) pr = MathBox.intersection(pr.x, pr.y, pr.width, pr.height,
                                                 x, y, width, height);
       if (pr != null && pr.width > 0 && pr.height > 0)
       {
         pr.x -= x;
         pr.y -= y;
         return pr;
       }
     }
     return null;
   }

   public void setLwParent(LwComponent o)
   {
     if (o != parent)
     {
       parent = o;
       invalidate();
     }
   }

   public Point getLocation() {
     return new Point(x, y);
   }

   public Color getBackground() {
     return back == null?LwToolkit.BACK_COLOR:back;
   }

   public virtual bool isVisible() {
     return MathBox.checkBit(bits, VISIBLE_BIT);
   }

   public virtual void setVisible(bool b)
   {
     if (MathBox.checkBit(bits, VISIBLE_BIT) != b)
     {
       bits = MathBox.getBits(bits, VISIBLE_BIT, b);
       invalidate();
       if (b)
       {
         LwEventManager.manager.perform(new LwComponentEvent(this, LwComponentEvent.COMP_SHOWN));
         repaint();
       }
       else
       {
         LwEventManager.manager.perform(new LwComponentEvent(this, LwComponentEvent.COMP_HIDDEN));
         if (parent != null) parent.repaint(x, y, width, height);
       }
     }
   }

   public virtual bool isEnabled() {
     return MathBox.checkBit(bits, ENABLE_BIT) && (parent == null || parent.isEnabled());
   }

   public virtual void setEnabled(bool b)
   {
     if (MathBox.checkBit(bits, ENABLE_BIT) != b)
     {
       bits = MathBox.getBits(bits, ENABLE_BIT, b);
       if (b) LwEventManager.manager.perform(new LwComponentEvent(this, LwComponentEvent.COMP_ENABLED ));
       else   LwEventManager.manager.perform(new LwComponentEvent(this, LwComponentEvent.COMP_DISABLED));
       repaint();
     }
   }

   public virtual void setBackground(Color c)
   {
     if (c != back && (c == null || !c.Equals(back)))
     {
       back = c;
       repaint();
     }
   }

   public override void invalidate()  {
     if (parent != null) parent.invalidate();
     base.invalidate();
   }

   public override void validate()
   {
     if (isVisible())
     {
       if (skins != null) skins.validate();
       base.validate();
     }
   }

   public void setLocation (int xx, int yy)
   {
     if (xx != x || y != yy)
     {
       int ox = x;
       int oy = y;
       x = xx;
       y = yy;
       if (parent != null) parent.repaint(System.Math.Min(x, ox), System.Math.Min(y, oy), width + System.Math.Abs(x - ox), height + System.Math.Abs(y - oy));
     }
   }

   public void setSize (int w, int h)
   {
     if (w != width || h != height)
     {
       int mw = System.Math.Max(w, width);
       int mh = System.Math.Max(h, height);
       width  = w;
       height = h;
       invalidate();
       repaint(0, 0, mw, mh);
     }
   }

   public Rectangle getBounds() {
     return new Rectangle (x, y, width, height);
   }

   public virtual LwComponent getLwComponentAt(int xx, int yy) {
     Rectangle r = getVisiblePart();
     return r != null && r.contains(xx, yy)?this:null;
   }

   public virtual LwComponent getLwParent() {
     return parent;
   }

   public override void recalc() {}

   public virtual void repaint() {
     LwPaintManager.manager.repaint(this);
   }

   public virtual void repaint(int x, int y, int w, int h) {
     LwPaintManager.manager.repaint(this, x, y, w, h);
   }

   public virtual void update(Graphics g){}

   public virtual void paint (Graphics g){}

   public virtual Dimension getPreferredSize () {
     validate();
     return getPreferredSize(this);
   }

   public LwViewMan getViewMan (bool autoCreate)
   {
     if (skins == null && autoCreate)
     {
       skins = new LwViewMan();
       skins.setParent(this);
     }
     return skins;
   }

   public void setViewMan(LwViewMan man)
   {
     if (man != skins)
     {
       skins = man;
       if (skins != null) skins.setParent(this);
       viewManChanged();
       vrp();
     }
   }

   public string toString () {
     return "[(" + x + "," + y + ") " + width + " " + height + "] valid=" + isValid() + ",op=" + isOpaque() + ",vis=" + isVisible();
   }

   public virtual Dimension calcPreferredSize() {
     return new Dimension(LwToolkit.PS_SIZE);
   }

   public virtual bool canHaveFocus() {
     return false;
   }

   public bool hasFocus() {
     return LwFocusManager.manager.hasFocus(this);
   }

   public void requestFocus() {
     LwFocusManager.manager.requestFocus(this);
   }

   public virtual void setOpaque(bool b)
   {
     if (b != MathBox.checkBit(bits, OPAQUE_BIT))
     {
       bits = MathBox.getBits(bits, OPAQUE_BIT, b);
       repaint();
     }
   }

   public bool isOpaque() {
     return MathBox.checkBit(bits, OPAQUE_BIT);
   }

   public void setPSSize (int w, int h)
   {
     if (w != psWidth || h != psHeight)
     {
       psWidth  = w;
       psHeight = h;
       vrp();
     }
   }

   public virtual Point getOrigin () {
     return null;
   }

   public virtual void viewManChanged() {}

   public void vrp() {
     invalidate();
     repaint();
   }

   private static Dimension getPreferredSize (LwCanvas c)
   {
     Dimension d = null;
     if (c.psWidth >= 0 && c.psHeight >= 0) d = new Dimension(c.psWidth, c.psHeight);
     else
     {
       d = c.calcPreferredSize();
       if (c.skins != null) d = MathBox.max (d, c.skins.getPreferredSize());
       if (c.psWidth  >= 0) d.width  = c.psWidth;
       if (c.psHeight >= 0) d.height = c.psHeight;
     }

     Insets i = c.getInsets();
     d.width  += (i.left + i.right);
     d.height += (i.top  + i.bottom);
     return d;
   }

   private static Insets getInsets(LwCanvas c)
   {
     Insets i = new Insets((int)(c.insets & 0xFFFF),
                           (int)((c.insets >> 16) & 0xFFFF),
                           (int)((c.insets >> 32) & 0xFFFF),
                           (int)((c.insets >> 48) & 0xFFFF));
     return (c.skins != null)?MathBox.max(c.skins.getInsets(), i):i;
   }

   private static short VISIBLE_BIT   = 1;
   private static short ENABLE_BIT    = 2;
   private static short OPAQUE_BIT    = 4;
   private static short DEFAULT_BITS  = 7;
}

}
