/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;


public class LwCheckbox:LwActContainer,LwMouseListener, LwKeyListener, Switchable
{
  public const  int RADIO = 1;

  public const  int CHECK = 2;

  private LwSwitchManager manager;
  private LwComponent box;
  private int type;

  public LwCheckbox():this(null) {

  }

  public LwCheckbox(string lab):this(lab, CHECK) {

  }

  public LwCheckbox(string lab, int type):this(lab != null?new LwLabel(lab):(LwComponent)null, type) {

  }

  public LwCheckbox(LwComponent c, int type):base(c)
  {

    setBox(new LwCanvas());
    setSwitchManager(new LwSwitchManImpl());
    setBoxType (type);
  }

  public void setBoxType (int t)
  {
    if (t != RADIO && t != CHECK) throw new org.zaval.port.cs.IllegalArgumentException();
    if (t != type)
    {
      type = t;
      vrp();
    }
  }

  public int getBoxType () {
    return type;
  }

  public void setBox(int i) {
    setBox((LwComponent)get(i));
  }

  public void setBox(LwComponent b)
  {
    if (box != b)
    {
      int i = 0;
      if (box != null)
      {
        i = indexOf(box);
        remove(i);
      }
      box = b;
      if (box != null) insert(i, box);
    }
  }

  public LwComponent getBox() {
    return box;
  }

  public void setSwitchManager(LwSwitchManager m)
  {
    if (m == null) throw new org.zaval.port.cs.IllegalArgumentException();
    if (manager != m)
    {
      if (manager != null) manager.uninstall(this);
      manager = m;
      manager.install(this);
    }
  }

  public LwSwitchManager getSwitchManager() {
    return manager;
  }

  public void setState(bool b) {
    manager.setState(this, b);
  }

  public bool getState() {
    return manager.getState(this);
  }

  public void switchedOn() {
    sync ();
  }

  public void switchedOff() {
    sync ();
  }

  public void sync ()
  {
    string prefix = (type == CHECK)?"check.":"radio.";
    if (isEnabled())
    {
      if (getState()) box.getViewMan(true).setView(prefix + "on");
      else            box.getViewMan(true).setView(prefix + "off");
    }
    else
    {
      if (getState()) box.getViewMan(true).setView(prefix + "dison");
      else            box.getViewMan(true).setView(prefix + "disoff");
    }
    repaint();
  }

  public override void setEnabled (bool b) {
    base.setEnabled(b);
    sync();
  }

  public void mouseClicked (LwMouseEvent e) {}
  public void mouseReleased(LwMouseEvent e) {}

  public void mousePressed (LwMouseEvent e) {
    if (LwToolkit.isActionMask(e.getMask())) setState(!getState());
  }

  public void keyPressed (LwKeyEvent e) {
    if (e.getKeyChar() == ' ' || e.getKeyCode() == LwConstants.VK_ENTER) setState(!getState());
  }

  public override void remove(int i) {
    if (get(i) == box)  box = null;
    base.remove(i);
  }

  public override void removeAll() {
    box = null;
    base.removeAll();
  }

  public void mouseEntered (LwMouseEvent e) {}
  public void mouseExited  (LwMouseEvent e) {}
  public void keyReleased  (LwKeyEvent e) {}
  public void keyTyped     (LwKeyEvent e) {}

  public override void recalc ()
  {
    sync();
    base.recalc();
  }
}


}
