/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.misc;
using org.zaval.port.cs;


public class LwFlowLayout:LwLayout
{
  private int ax, ay, direction, vgap, hgap;

  public LwFlowLayout():this(Alignment.LEFT, Alignment.TOP, LwToolkit.HORIZONTAL) {
   
  }

  public LwFlowLayout (int ax, int ay):this(ax, ay, LwToolkit.HORIZONTAL) {
   
  }

  public LwFlowLayout (int ax, int ay, int dir)
  {
    if (dir != LwToolkit.HORIZONTAL && dir != LwToolkit.VERTICAL) throw new org.zaval.port.cs.IllegalArgumentException();
    this.ax = ax;
    this.ay = ay;
    direction = dir;
  }

  public Dimension calcPreferredSize (LayoutContainer target) {
    return calcPreferredSize (target, direction, hgap, vgap);
  }

  public void layout(LayoutContainer c)
  {
    Dimension psSize = calcPreferredSize(c);
    Insets    ins    = c.getInsets();
    psSize.width  += (ins.left + ins.right);
    psSize.height += (ins.top  + ins.bottom);
    Point p = Alignment.getLocation(psSize, ax, ay, c.getWidth(), c.getHeight());

    int x = p.x, y = p.y;
    for (int i=0; i<c.count(); i++)
    {
      Layoutable a = c.get(i);
      if (a.isVisible())
      {
        Dimension d  = a.getPreferredSize();
        if (direction == LwToolkit.HORIZONTAL)
        {
          a.setLocation(x + ins.left, (psSize.height - d.height)/2 + y);
          x += (d.width + hgap);
        }
        else
        {
          a.setLocation(x + (psSize.width - d.width)/2, y + ins.top);
          y += d.height + vgap;
        }
        a.setSize(d.width, d.height);
      }
    }
  }

  public int getVGap () {
    return vgap;
  }

  public int getHGap () {
    return hgap;
  }

  public void setGaps (int vg, int hg) {
    vgap = vg;
    hgap = hg;
  }

  public void  componentAdded  (object id, Layoutable lw, int index) {}

  public void  componentRemoved(Layoutable lw, int index) {}

  private static Dimension calcPreferredSize (LayoutContainer c, int direction, int hgap, int vgap)
  {
    Dimension m  = new Dimension();
    int       cc = 0;
    for (int i = 0; i<c.count(); i++)
    {
      Layoutable a = c.get(i);
      if (a.isVisible())
      {
        Dimension d = a.getPreferredSize();
        if (direction == LwToolkit.HORIZONTAL)
        {
          m.width  += d.width;
          m.height  = System.Math.Max(d.height, m.height);
        }
        else
        {
          m.width   = System.Math.Max(d.width, m.width);
          m.height += d.height;
        }
        cc++;
      }
    }

    if (direction == LwToolkit.HORIZONTAL) m.width  += (hgap * (cc - 1));
    else                                   m.height += (vgap * (cc - 1));

    return m;
  }
}




}
