/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;


public class LwFocusManager:LwMouseListener, LwComponentListener, LwContainerListener, LwKeyListener
{
  public static LwFocusManager manager;

  private LwComponent focusOwner;

  public void requestFocus(LwComponent c)
  {
    if (c != focusOwner && (c == null || isFocusable(c)))
    {
      LwComponent oldFocusOwner = focusOwner;
      if  (c != null)
      {
        LwComponent nf = LwEventManager.manager.getEventDestination(c);
        if (nf == null || oldFocusOwner == nf) return;
        focusOwner = nf;
      }
      else focusOwner = c;

      if (oldFocusOwner != null)
      {
        LwEventManager.manager.perform(new LwFocusEvent(oldFocusOwner, LwFocusEvent.FOCUS_LOST));
        oldFocusOwner.repaint();
      }

      if (focusOwner != null)
      {
        LwEventManager.manager.perform(new LwFocusEvent(focusOwner, LwFocusEvent.FOCUS_GAINED));
        focusOwner.repaint();
      }
    }
  }

  public LwComponent findFocusable(LwComponent c) {
    return (isFocusable(c)?c:fd(c, 0));
  }

  public bool hasFocus(LwComponent c) {
    return focusOwner == c;
  }

  public LwComponent getFocusOwner() {
    return focusOwner;
  }

  public void mousePressed(LwMouseEvent e) {
    if (LwToolkit.isActionMask(e.getMask()))
      requestFocus(e.getLwComponent());
  }

  public void mouseClicked(LwMouseEvent e) {}

  public void mouseEntered(LwMouseEvent e) {}

  public void mouseExited(LwMouseEvent e) {}

  public void mouseReleased(LwMouseEvent e) {}

  public void compDisabled (LwComponentEvent e) { freeFocus(e.getLwComponent()); }

  public void compEnabled(LwComponentEvent e) { }

  public void compShown(LwComponentEvent e) { }

  public void compHidden(LwComponentEvent e) { freeFocus(e.getLwComponent()); }

  public void compAdded(LwContainerEvent e) {}

  public void compRemoved(LwContainerEvent e) { freeFocus(e.getLwComponent()); }

  public void keyPressed (LwKeyEvent e)
  {
    if (e.getKeyCode() == LwConstants.VK_TAB && e.getMask() == 0)
    {
      LwComponent cc = ff (e.getLwComponent());
      if (cc != null) requestFocus(cc);
    }
  }

  public void keyReleased(LwKeyEvent e) {}
  public void keyTyped   (LwKeyEvent e) {}

  public virtual bool isFocusable (LwComponent c) {
    return c.isEnabled() && c.canHaveFocus();
  }

  private void freeFocus(LwComponent target) {
    if (target == getFocusOwner()) requestFocus(null);
  }

  private static LwComponent fd(LwComponent target, int index)
  {
    if (target is LwContainer)
    {
      LwContainer container   = (LwContainer)target;
      bool     isComposite = target is LwComposite;
      for (int i=index; i<container.count(); i++)
      {
        LwComponent cc = (LwComponent)container.get(i);
        if (cc.isEnabled() && cc.isVisible() && (!isComposite || !((LwComposite)container).catchInput(cc)))
        {
          if (cc.canHaveFocus()) return cc;
          cc = fd(cc, 0);
          if (cc != null) return cc;
        }
      }
    }
    return null;
  }

  private static LwComponent ff(LwComponent c)
  {
    LwComponent top = c;
    while (!(top is LwLayer)) top = top.getLwParent();
    top = ((LwLayer)top).getFocusRoot();

    for (int index = 0;c != top.getLwParent();)
    {
      LwComponent cc = fd(c, index);
      if (cc != null) return cc;
      cc = c;
      c  = c.getLwParent();
      if (c != null) index = 1 + ((LwContainer)c).indexOf(cc);
    }
    return fd(top, 0);
  }
}


}
