/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.misc;
using org.zaval.port.cs;
using org.zaval.util;


public class LwGridLayout:LwLayout, PosInfo
{
  private int       cols, rows;
  private Hashtable args;

  public LwGridLayout(int r, int c) {
    rows = r;
    cols = c;
  }

  public Point[] getRowSizes(LayoutContainer c)
  {
    Point[] res = new Point[rows];
    for (int i = 0; i<rows; i++)
    {
      res[i] = new Point();
      res[i].x = getRowSize(i, c, args, cols);
      if (i > 0) res[i].y += res[i-1].y + res[i-1].x;
    }
    return res;
  }

  public Point[] getColSizes(LayoutContainer c)
  {
    Point[] res = new Point[cols];
    for (int i = 0; i<cols; i++)
    {
      res[i] = new Point();
      res[i].x = getColSize(i, c, args, cols);
      if (i > 0) res[i].y += res[i-1].y + res[i-1].x;
    }
    return res;
  }

  public Dimension calcPreferredSize (LayoutContainer c)
  {
    Point[] colSizes = getColSizes(c);
    Point[] rowSizes = getRowSizes(c);
    Dimension max = new Dimension();
    for (int i = 0; i < rowSizes.Length; i++) max.height += rowSizes[i].x;
    for (int i = 0; i < colSizes.Length; i++) max.width  += colSizes[i].x;
    return max;
  }

  public void layout(LayoutContainer c)
  {
    Point[] colSizes = getColSizes(c);
    Point[] rowSizes = getRowSizes(c);
    Insets  insets   = c.getInsets();
    Point   offset   = c.getLayoutOffset();

    for (int i = 0; i<c.count(); i++)
    {
      Layoutable l = c.get(i);
      if (l.isVisible())
      {
        Point rc = MathBox.index2point(i, cols);
        LwConstraints arg = getConstraints(l, args);
        Insets        ins = (arg.insets == null)?new Insets(0, 0, 0, 0):arg.insets;
        Dimension     d   = l.getPreferredSize();

        int px    = ins.left + insets.left + colSizes[rc.y].y;
        int py    = ins.top  + insets.top  + rowSizes[rc.x].y;
        int cellW = colSizes[rc.y].x - ins.left - ins.right;
        int cellH = rowSizes[rc.x].x - ins.top - ins.bottom;

        if ((LwToolkit.HORIZONTAL & arg.fill) > 0) d.width  = cellW;
        if ((LwToolkit.VERTICAL   & arg.fill) > 0) d.height = cellH;

        Point p2 = Alignment.getLocation(d, arg.ax, arg.ay, cellW, cellH);
        py += p2.y;
        px += p2.x;

        l.setSize    (d.width, d.height);
        l.setLocation(px + offset.x, py + offset.y);
      }
    }
  }

  public void componentAdded(object id, Layoutable b, int index)
  {
    if (id != null)
    {
      if (args == null) args = new Hashtable();
      args.put(b, id);
    }
  }

  public void componentRemoved(Layoutable lw, int index)
  {
    if (args != null) {
      args.remove(lw);
      if (args.size() == 0) args = null;
    }
  }

  public int getLines   () {
    return rows;
  }

  public int getLineSize(int line) {
    return cols;
  }

  public int getMaxOffset() {
    return rows*cols - 1;
  }

  private static int getRowSize(int row, LayoutContainer c, Hashtable args, int cols)
  {
    int max = 0;
    for (int i = 0; i<c.count(); i++)
    {
      Layoutable a = c.get(i);
      Point      p = MathBox.index2point(i, cols);
      if (a.isVisible() && p.x == row)
      {
        LwConstraints arg = getConstraints(a, args);
        Dimension d = a.getPreferredSize();
        if (arg.insets != null) d.height += (arg.insets.top + arg.insets.bottom);
        max = System.Math.Max(d.height, max);
      }
    }
    return max;
  }

  private static int getColSize(int col, LayoutContainer c, Hashtable args, int cols)
  {
    int max = 0;
    for (int i = 0; i<c.count(); i++)
    {
      Layoutable a = c.get(i);
      Point      p = MathBox.index2point(i, cols);
      if (a.isVisible() && p.y == col)
      {
        LwConstraints arg = getConstraints(a, args);
        Dimension d = a.getPreferredSize();
        if (arg.insets != null) d.width += (arg.insets.left + arg.insets.right);
        max = System.Math.Max(d.width, max);
      }
    }
    return max;
  }

  private static LwConstraints getConstraints (Layoutable l, Hashtable args)
  {
    if (args == null) return DEF_CONSTR;
    LwConstraints arg = (LwConstraints)args.get(l);
    return arg != null?arg:DEF_CONSTR;
  }

  private static readonly  LwConstraints DEF_CONSTR = new LwConstraints();
}



}
