/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.port.cs;


public class LwLink:LwStButton,Cursorable
{
   private static readonly  Font linkFont = new Font("HELVETICA", Font.ITALIC | Font.BOLD, 12);

   private Color highlightColor = Color.blue;
   private Color pressingColor  = LwToolkit.darkBlue;
   private Color nonActiveColor = Color.black;

   public LwLink(string s)
   {
     LwTextRender tr = new LwTextRender(s);
     tr.setFont (linkFont);
     getViewMan(true).setView(tr);
     setOpaque (false);
     sync();
   }


   public void setHighlightColor(Color c)
   {
     if (!highlightColor.Equals(c))
     {
       highlightColor = c;
       sync();
     }
   }

   public void setPressingColor(Color c)
   {
     if (!pressingColor.Equals(c))
     {
       pressingColor = c;
       sync();
     }
   }

   public void setNonActiveColor(Color c)
   {
     if (!nonActiveColor.Equals(c))
     {
       nonActiveColor = c;
       sync();
     }
   }

   public int getCursorType(int x, int y) {
     return Cursor.HAND_CURSOR;
   }

   public override void stateChanged(int s, int ps)
   {
     switch (s)
     {
       case ST_ONSURFACE : setForeground(highlightColor); break;
       case ST_PRESSED   : setForeground(pressingColor ); break;
       case ST_OUTSURFACE: setForeground(nonActiveColor); break;
     }
     repaint();
   }

   private void setForeground(Color c)
   {
     if (skins != null)
     {
       LwTextRender r = (LwTextRender)skins.getView();
       if (!r.getForeground().Equals(c))
       {
         r.setForeground(c);
         repaint();
       }
     }
   }
}



}
