/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.misc.events;
using org.zaval.misc;
using org.zaval.port.cs;
using org.zaval.util.events;


public class LwList:LwPanel,LwKeyListener, LwMouseListener, LwFocusListener, LwComposite, PosListener, ScrollObj
{
    private int selectedIndex = -1;
    private PosController controller;
    private PosInfo       posInfo;
    private LwActionSupport support;
    private LwComponent input;
    private int  dx, dy;
    private Color selectColor = LwToolkit.darkBlue, posRectColor = Color.yellow;
    private ScrollMan man;

    public LwList() {
      setPosController(new PosController());
    }

    public override bool canHaveFocus() {
      return true;
    }

    public void setPosController(PosController c)
    {
      if(c == null) throw new org.zaval.port.cs.IllegalArgumentException();
      if(c != controller)
      {
        if (controller != null) controller.removePosListener(this);
        controller = c;
        controller.addPosListener(this);
        if (posInfo != null) controller.setPosInfo(posInfo);
        repaint();
      }
    }

    public override void setLwLayout(LwLayout l)
    {
      if (l != getLwLayout())
      {
        base.setLwLayout(l);
        if (l is PosInfo) posInfo = (PosInfo)l;
        if (controller != null) controller.setPosInfo(posInfo);
      }
    }

    public PosController getPosController() {
      return controller;
    }

    public Color getSelectColor() {
      return selectColor;
    }

    public void setSelectColor(Color c)
    {
      if (!c.Equals(selectColor)) {
        selectColor = c;
        repaint();
      }
    }

    public Color getPosRectColor() {
      return posRectColor;
    }

    public void setPosRectColor(Color c)
    {
      if (!c.Equals(posRectColor)) {
        posRectColor = c;
        repaint();
      }
    }

    public void add (string s)  {
      add (new LwLabel(s));
    }

    public void select(int index)
    {
      if (selectedIndex != index)
      {
        bool b = clearSelection();
        if (index >= 0)
        {
          selectedIndex = index;
          b = !notifyScrollMan(index);
        }
        if (b) repaint();

        if (selectedIndex >= 0) perform(selectedIndex);
      }
    }

    public int getSelectedIndex() {
      return selectedIndex;
    }

    public bool isSelected(int i)  {
      return i == selectedIndex;
    }

    public void addSelectionListener(LwActionListener l)  {
      if (support == null) support = new LwActionSupport();
      support.addListener(l);
    }

    public void removeSelectionListener(LwActionListener l)  {
      if (support != null) support.removeListener(l);
    }

    public override void paintOnTop(Graphics g) {
      if (!hasInputFocus()) drawSelMarker(g);
      if (hasFocus()) drawPosMarker(g);
    }

    public void drawSelMarker(Graphics g)
    {
       if (selectedIndex >= 0)
       {
         LwComponent c = getSelected();
         LwToolkit.drawMarker(g, c.getX(), c.getY(), c.getWidth(), c.getHeight(), getBackground(), selectColor);
       }
    }

    public void drawPosMarker(Graphics g)
    {
       int offset = controller.getOffset();
       if (offset >= 0)
       {
         LwComponent c = (LwComponent)get(offset);
         g.setColor(posRectColor);
         g.drawRect(c.getX(), c.getY(), c.getWidth()-1, c.getHeight()-1);
       }
    }

    public virtual void mouseClicked(LwMouseEvent e) {}
    public virtual void mouseEntered(LwMouseEvent e) {}
    public virtual void mouseExited(LwMouseEvent e)  {}

    public virtual void mousePressed(LwMouseEvent e)
    {
      if (LwToolkit.isActionMask(e.getMask()))
      {
        int index = LwToolkit.getDirectCompAt(e.getX(), e.getY(), this);
        if (index >= 0)
        {
          controller.setOffset(index);
          select(index);
        }
      }
    }

    public virtual void mouseReleased(LwMouseEvent e) {}

    public virtual void keyPressed(LwKeyEvent e)
    {
      if (posInfo.getMaxOffset() > 0)
      {
        bool isCtrl = (e.getMask() &  LwConstants.CTRL_MASK) > 0;
        switch(e.getKeyCode())
        {
          case LwConstants.VK_END :  if (isCtrl) controller.setOffset(controller.getMaxOffset());
                                  else        controller.seekLineTo(PosController.END);
                                  break;
          case LwConstants.VK_HOME:  if (isCtrl) controller.setOffset(0);
                                  else        controller.seekLineTo(PosController.BEG);
                                  break;
          case LwConstants.VK_RIGHT: controller.seek(1); break;
          case LwConstants.VK_DOWN:  controller.seekLineTo(PosController.DOWN); break;
          case LwConstants.VK_LEFT:  controller.seek(-1); break;
          case LwConstants.VK_UP:    controller.seekLineTo(PosController.UP); break;
          case LwConstants.VK_PAGE_UP  : controller.seek(pageSize(-1));  break;
          case LwConstants.VK_PAGE_DOWN: controller.seek(pageSize(1)); break;
        }
      }
    }

    public virtual void keyReleased(LwKeyEvent e){}
    public virtual void keyTyped(LwKeyEvent e)   {}

    public virtual void focusGained(LwFocusEvent e)
    {
      int o = controller.getOffset();
      if (o >= 0 && o == selectedIndex) input = (LwComponent)get(controller.getOffset());
      else input=null;
    }

    public virtual void focusLost(LwFocusEvent e) {}

    public virtual bool catchInput (LwComponent child)
    {
      bool b = input != null && LwToolkit.isAncestorOf(input, child);
      if (b && !hasInputFocus() && !hasFocus()) input = null;
      return (input == null || !b);
    }

    public virtual void posChanged(PosEvent e)
    {
      int off = controller.getOffset();
      select(off);
      repaint();
      input = (controller.getOffset() >= 0)?(LwComponent)get(off):null;
    }

    public override void insert(int i, object s, LwComponent d)
    {
      int offset = controller.getOffset();
      base.insert(i, s, d);

      if (offset >= 0 && offset >= i)
      {
        clearSelection();
        controller.clearPos();
        offset = offset == i?offset:offset+1;
        controller.setOffset(offset);
      }
      //if (csupport != null) csupport.perform(new LwContainerEvent(d, this, LwContainerEvent.COMP_ADDED));
    }

    public override void remove(int i)
    {
      int offset = controller.getOffset();
      base.remove(i);

      if (offset >= 0)
      {
        if (count() == 0 || offset >= i)
        {
          clearSelection();
          controller.clearPos();
        }

        if (count() > 0 && offset >= i)
        {
          offset = offset == i?offset:offset-1;
          controller.setOffset(offset);
        }
      }
    }

    public override void removeAll()
    {
      clearSelection();
      controller.clearPos();
      base.removeAll();
    }

    public Point getSOLocation () {
      return getLayoutOffset();
    }

    public void setSOLocation (int x, int y)
    {
      if (x != dx || y != dy)
      {
        dx = x;
        dy = y;
        vrp();
      }
    }

    public Dimension getSOSize() {
      return getPreferredSize();
    }

    public void setScrollMan (ScrollMan m) {
      man = m;
    }

    public bool moveContent() {
      return true;
    }

    public override Point getLayoutOffset()  {
      return new Point(dx, dy);
    }

    public LwComponent getSelected() {
      return selectedIndex < 0?null:(LwComponent)get(selectedIndex);
    }

    public virtual void perform(int from) {
      if (support != null) support.perform(new LwActionEvent(this, from));
    }

    public virtual bool notifyScrollMan(int index)
    {
      if (index >= 0)
      {
        LwComponent c = (LwComponent)get(index);
        Point p = LwToolkit.calcOrigin (c.getX() - dx, c.getY()- dy, c.getWidth(), c.getHeight(), dx, dy, this);
        if (p.x != dx || p.y != dy)
        {
          if (man != null) man.scrollObjMoved(p.x, p.y);
          else             setSOLocation(p.x, p.y);
          return true;
        }
      }
      return false;
    }

    public virtual int pageSize(int d)
    {
      int offset = controller.getOffset();
      if (offset >= 0)
      {
        Rectangle vp = getVisiblePart();
        int sum = 0, i = 0;
        for (i=offset; i>=0 && i<=controller.getMaxOffset() && sum<vp.height; i+=d)
          sum += get(i).getHeight();
        return i - offset - d;
      }
      return 0;
    }

    public override LwLayout getDefaultLayout() {
      return new LwListLayout();
    }

    private bool clearSelection()
    {
      if (selectedIndex >= 0)
      {
        int prev = selectedIndex;
        selectedIndex = -1;
        return true;
      }
      return false;
    }

   private bool hasInputFocus()  {
     return input != null && LwToolkit.isAncestorOf(input, LwFocusManager.manager.getFocusOwner());
   }
}

}
