/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.port.cs;


public class LwListLayout:LwLayout, org.zaval.misc.PosInfo
{
  private int count = 0, gap = 0;

   public LwListLayout():this(0) {
   
  }

  public LwListLayout(int gap) {
    this.gap = gap;
  }

  public void componentAdded   (object id, Layoutable lw, int index) { count++; }

  public void componentRemoved (Layoutable lw, int index) { count--; }

  public Dimension calcPreferredSize(LayoutContainer lw)
  {
    int w = 0, h = 0, c = 0;
    for (int i=0; i<lw.count(); i++)
    {
      Layoutable cc = lw.get(i);
      if (cc.isVisible())
      {
        Dimension d = cc.getPreferredSize();
        h += (d.height + (c > 0?gap:0));
        if (w < d.width) w = d.width;
        c ++;
      }
    }
    return new Dimension(w, h);
  }

  public void layout(LayoutContainer lw)
  {
    Insets ins = lw.getInsets();
    Point  offset = lw.getLayoutOffset();
    int x = ins.left + offset.x, y = ins.top + offset.y, psw = lw.getWidth() - ins.left - ins.right;

    for (int i=0; i<lw.count(); i++)
    {
      Layoutable cc = lw.get(i);
      if (cc.isVisible())
      {
        Dimension d = cc.getPreferredSize();
        cc.setSize(d.width > psw?d.width:psw, d.height);
        cc.setLocation(x, y);
        y += (d.height + gap);
      }
    }
  }

  public int getLineSize(int line) {
    return 1;
  }

  public int getLines() {
    return count;
  }

  public int getMaxOffset() {
    return count - 1;
  }
}


}
