/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;
using org.zaval.util;


public abstract class LwPaintManager
{
  public static LwPaintManager manager;

  public virtual void repaint(LwComponent c) {
    if (c.isVisible())
      repaint(c, 0, 0, c.getWidth(), c.getHeight());
  }

  public virtual void repaint(LwComponent c, int x, int y, int w, int h)
  {
    if (c.isVisible() && w > 0 && h > 0)
    {
      LwDesktop desktop = LwToolkit.getDesktop(c);
      if (desktop != null)
      {
        Point     p = LwToolkit.getAbsLocation(x, y, c);
        Rectangle r = MathBox.intersection(0, 0, desktop.getWidth(), desktop.getHeight(),
                                           p.x, p.y, w, h);
        if (r.width > 0 && r.height > 0) desktop.repaint (r.x, r.y, r.width, r.height);
      }
    }
  }

  public virtual void rootPaint(Graphics g, LwComponent c) {
    paint(g, c);
  }

  public virtual void paint(Graphics g, LwComponent c)
  {
    //!!! This path for IE.
    int dw = c.getWidth(), dh = c.getHeight();
    if (dw == 0 || dh == 0) return;
    c.validate();

    if (c.isOpaque()) updateComponent (g, c);
    paintComponent (g, c);
    if (c is LwContainer)
    {
      LwContainer container = (LwContainer)c;

      g.clipRect(0, 0, dw, dh);
      Rectangle clipRect = g.getClipBounds();
      if (clipRect == null) return;

      for (int i = 0; i< container.count(); i++)
      {
        LwComponent kid = (LwComponent)container.get(i);
        if (kid.isVisible())
        {
          int kidX = kid.getX(), kidY = kid.getY();
          Rectangle intersect = MathBox.intersection(kidX, kidY, kid.getWidth(), kid.getHeight(), clipRect.x, clipRect.y, clipRect.width, clipRect.height);
          if (intersect.width > 0 && intersect.height > 0)
          {
            g.setClip(intersect.x, intersect.y, intersect.width, intersect.height);
            g.translate(kidX, kidY);
            paint(g, kid);
            g.translate(-kidX, -kidY);
          }
        }
      }
      g.setClip(clipRect);
      paintOnTop(g, container);
    }
  }

  abstract public void updateComponent (Graphics g, LwComponent c);

  abstract public void paintComponent(Graphics g, LwComponent c);

  abstract public void paintOnTop (Graphics g, LwContainer c);
}

}
