/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;



public class LwPanel:LwCanvas,LwContainer
{
  public Vector children;
  private LwLayout layout;
  private int cpsWidth = -1, cpsHeight;

  public LwPanel() {
    setLwLayout(getDefaultLayout());
  }

  public virtual void paintOnTop(Graphics g) {}

  public void add(object s, LwComponent c) {
    insert(count(), s, c);
  }

  public void add(LwComponent c) {
    insert(count(), null, c);
  }

  public void insert(int i, LwComponent d) {
    insert(i, null, d);
  }

  public virtual void insert(int i, object s, LwComponent d) {
    insert(this, i, s, d);
  }

  public void remove(LwComponent c) {
    remove(indexOf(c));
  }

  public virtual void remove(int i) {
    remove(this, i);
  }

  public virtual void removeAll() {
    removeAll(this);
  }

  public int indexOf(LwComponent c) {
    return children == null?-1:children.IndexOf(c);
  }

  public Layoutable get(int i) {
    return (Layoutable)children.elementAt(i);
  }

  public int count() {
    return children == null?0:children.size();
  }

  public virtual void setLwLayout(LwLayout m)
  {
    if (m == null) throw new org.zaval.port.cs.IllegalArgumentException();
    if (layout != m)
    {
      LwLayout prev = layout;
      layout = m;
      invalidate();
    }
  }

  public LwLayout getLwLayout() {
    return layout;
  }

  public override void setBackground(Color c)
  {
    base.setBackground(c);
    if (children != null)
      for (int i=0; i<children.size(); i++)
        ((LwComponent)children.elementAt(i)).setBackground(c);
  }

  public override void setOpaque(bool b)
  {
    base.setOpaque(b);
    if (children != null)
      for (int i=0; i<children.size(); i++)
        ((LwComponent)children.elementAt(i)).setOpaque(b);
  }

  public override LwComponent getLwComponentAt(int xx, int yy)
  {
    LwComponent c = base.getLwComponentAt(xx, yy);
    if (children != null && c != null)
    {
      for (int i=children.size()-1;i >= 0;i--)
      {
        LwComponent d = (LwComponent)children.elementAt(i);
        LwComponent r = d.getLwComponentAt(xx - d.getX(), yy - d.getY());
        if (r != null) return r;
      }
    }
    return c;
  }

  public void toFront(LwComponent c)
  {
    int index = indexOf (c);
    if (index < 0) throw new org.zaval.port.cs.IllegalArgumentException();
    if (index < (count()-1))
    {
      children.removeElementAt(index);
      children.addElement(c);
      repaint(c.getX(), c.getY(), c.getWidth(), c.getHeight());
    }
  }

/*  public void validate () {
    if (isVisible() && width > 0 && height > 0) base.validate();
  }*/

  public override void recalc ()
  {
    layout.layout(this);
    if (children != null)
      for (int i=0; i<children.size(); i++)
        ((LwComponent)children.elementAt(i)).validate();
    isValidFlag = true;
  }

  public virtual Point getLayoutOffset() {
    return new Point();
  }

  public override void invalidate() {
    cpsWidth = -1;
    base.invalidate();
  }

  public override Dimension calcPreferredSize()
  {
    if (cpsWidth < 0)
    {
      Dimension d = layout.calcPreferredSize(this);
      cpsWidth  = d.width;
      cpsHeight = d.height;
      return d;
    }
    else return new Dimension(cpsWidth, cpsHeight);
  }

  public virtual LwLayout getDefaultLayout() {
    return LwManager.getLwLayout("layout.raster");
  }

  public virtual void updateCashedPs(int w, int h) {
    if (w >= 0) cpsWidth  = w;
    if (h >= 0) cpsHeight = h;
  }

  private static void insert(LwPanel c, int i, object s, LwComponent d)
  {
    d.setLwParent(c);
    if (c.children == null) c.children = new Vector(5);
    c.children.insertElementAt(d, i);
    c.layout.componentAdded(s, d, i);
    LwEventManager.manager.perform(new LwContainerEvent(d, c, LwContainerEvent.COMP_ADDED));
    c.invalidate();
  }

  private static void remove(LwPanel c, int i)
  {
    if (c.children != null)
    {
      LwComponent obj = (LwComponent)c.children.elementAt(i);
      obj.setLwParent(null);
      c.children.removeElementAt(i);
      c.layout.componentRemoved(obj, i);
      LwEventManager.manager.perform(new LwContainerEvent(obj, c, LwContainerEvent.COMP_REMOVED));
      c.vrp();
    }
  }

  private static void removeAll(LwPanel c)
  {
    if (c.count() > 0)
    {
      int size = c.children.size();
      for (;size>0;size--)
      {
        LwComponent child = (LwComponent)c.children.elementAt(size-1);
        c.children.removeElementAt(size-1);
        c.layout.componentRemoved(child, size-1);
        LwEventManager.manager.perform(new LwContainerEvent(child, c, LwContainerEvent.COMP_REMOVED));
      }
      c.vrp();
    }
  }
}


}
