/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.port.cs;


public class LwRasterLayout:LwLayout
{
  public const  int W_BY_PARENT = 1;

  public const  int H_BY_PARENT = 2;

  public const  int USE_PS_SIZE = 4;

  private int flag;

  public LwRasterLayout():this(0) {
   
  }

  public LwRasterLayout(int f) {
    flag = f;
  }

  public void  componentAdded(object id, Layoutable lw, int index) {}

  public void  componentRemoved(Layoutable lw, int index) {}

  public Dimension calcPreferredSize(LayoutContainer c)
  {
    Dimension m = new Dimension();
    bool   b = (flag & USE_PS_SIZE) > 0;
    for (int i=0; i<c.count(); i++)
    {
      Layoutable el = c.get(i);
      if (el.isVisible())
      {
        Point p = el.getLocation();
        Dimension ps = b?el.getPreferredSize():el.getSize();
        p.x += ps.width;
        p.y += ps.height;
        if (p.x > m.width ) m.width  = p.x;
        if (p.y > m.height) m.height = p.y;
      }
    }
    return m;
  }

  public void layout(LayoutContainer c)
  {
    Point     offs = c.getLayoutOffset();
    Insets    ins  = c.getInsets();
    int       r = c.getWidth() - ins.right, b = c.getHeight() - ins.bottom;
    bool   usePsSize = (flag & USE_PS_SIZE) > 0;
    for (int i=0; i<c.count(); i++)
    {
      Layoutable el = c.get(i);
      if (el.isVisible())
      {
        Point p = el.getLocation();
        p.x += offs.x;
        p.y += offs.y;
        el.setLocation(p.x, p.y);

        Dimension ps = usePsSize?el.getPreferredSize():el.getSize();
        if ((flag & W_BY_PARENT)>0) ps.width  = r - p.x;
        if ((flag & H_BY_PARENT)>0) ps.height = b - p.y;
        el.setSize(ps.width, ps.height);
      }
    }
  }
}




}
