/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;
using org.zaval.util;


public class LwSlider:LwCanvas,LwKeyListener, LwMouseListener, LwMouseMotionListener, LwViewProvider
{
  public const  int BUNDLE_VIEW  = 0;

  public const  int GAUGE_VIEW   = 1;

  private int             max, min, value, roughStep, exactStep;
  private int             netSize = 3, gap = 3, orient, scaleStep, psW, psH;
  private int[]           intervals, pl;
  private LwView[]        views = new LwView[2];
  private LwActionSupport support;
  private Color           scaleColor = LwToolkit.darkBlue;
  private LwViewProvider  provider;
  private LwTextRender    render = new LwTextRender("");

  public LwSlider():this(LwToolkit.HORIZONTAL) {
   
  }

  public LwSlider(int o)
  {
    provider = this;
    setOrientation(o);
    setView (GAUGE_VIEW, LwManager.getView("sl.gauge"));
    setValues (0, 20, new int[] { 0, 5, 10 }, 2, 1);
    render = new LwTextRender("");
    render.setFont (LwToolkit.BFONT);
    render.setForeground (scaleColor);
    setScaleStep(1);
    showScale (true);
    showTitle (true);
  }


  public override bool canHaveFocus() {
    return true;
  }


  public int getScaleGap (){
    return gap;
  }

  public void setScaleGap (int g)
  {
    if (g != gap) {
      gap = g;
      vrp();
    }
  }

  public Color getScaleColor () {
    return scaleColor;
  }

  public void setScaleColor (Color c)
  {
    if (!c.Equals(scaleColor)) {
      scaleColor = c;
      repaint();
    }
  }

  public void setScaleStep (int s)
  {
    if (s != scaleStep)
    {
      scaleStep = s;
      repaint();
    }
  }

  public void showScale (bool b)
  {
    if (MathBox.checkBit(bits, SHOW_SCALE) != b) {
      bits = MathBox.getBits(bits, SHOW_SCALE, b);
      vrp();
    }
  }

  public bool isJumpOnPress () {
    return MathBox.checkBit(bits, JUMP_ONPRESS);
  }

  public void jumpOnPress (bool b) {
    bits = MathBox.getBits(bits, JUMP_ONPRESS, b);
  }

  public void showTitle (bool b)
  {
    if (MathBox.checkBit(bits, SHOW_TITLE) != b) {
      bits = MathBox.getBits(bits, SHOW_TITLE, b);
      vrp();
    }
  }

  public void setViewProvider(LwViewProvider p)
  {
    if (p != provider)
    {
      provider = p;
      vrp();
    }
  }

  public LwView getView(Drawable d, object o) {
    render.getTextModel().setText(o!=null?o.ToString():"");
    return render;
  }

  public void setOrientation(int o)
  {
    if (o != LwToolkit.HORIZONTAL && o != LwToolkit.VERTICAL)
      throw new org.zaval.port.cs.IllegalArgumentException();

    if (o != orient)
    {
      orient = o;
      setView (BUNDLE_VIEW, o == LwToolkit.HORIZONTAL?LwManager.getView("sl.hbundle")
                                                     :LwManager.getView("sl.vbundle"));
    }
  }

  public int getOrientation() {
    return orient;
  }

  public LwView getView (int id) {
    return views[id];
  }

  public void setView (int id, LwView v)
  {
    if (v != views[id])
    {
      views[id] = v;
      vrp();
    }
  }

  public bool isIntervalModel() {
    return MathBox.checkBit(bits, INTERVAL_MODEL_BIT);
  }

  public void useIntervalModel(bool b) {
    if (b != isIntervalModel())
      bits = MathBox.getBits (bits, INTERVAL_MODEL_BIT, b);
  }

  public int getValue() {
    return value;
  }

  public void setValue(int v)
  {
    if (v < min || v > max) throw new org.zaval.port.cs.IllegalArgumentException();

    int prev = value;
    if (value != v)
    {
      value = v;
      if (support != null) support.perform(new LwActionEvent(this, prev));
      repaint();
    }
  }

  public int getMax () {
    return max;
  }

  public int getMin() {
    return min;
  }

  public int getIntervals () {
    return intervals.Length;
  }

  public int getIntervalSize (int i) {
    return intervals[i];
  }

  public void setValues (int min, int max, int[] intervals, int roughStep, int exactStep)
  {
    if (roughStep <= 0 || exactStep < 0 || min >= max ||
        min + roughStep > max || min + exactStep > max   ) throw new org.zaval.port.cs.IllegalArgumentException();

    for (int i=0, start = min; i<intervals.Length; i++)
    {
      start += intervals[i];
      if (start > max || intervals[i] < 0) throw new org.zaval.port.cs.IllegalArgumentException();
    }

    this.min  = min;
    this.max  = max;
    this.roughStep = roughStep;
    this.exactStep = exactStep;
    this.intervals = new int[intervals.Length];
    System.Array.Copy (intervals, 0, this.intervals, 0, intervals.Length);
    if (value < min || value > max) setValue(isIntervalModel()?min + intervals[0]:min);
    vrp();
  }

  public int getPointValue(int i)
  {
    int v = min + getIntervalSize(0);
    for (int j=0; j < i; j++, v+=getIntervalSize(j));
    return v;
  }

  public void addActionListener(LwActionListener l) {
    if (support == null) support = new LwActionSupport();
    support.addListener(l);
  }

  public void removeActionListener(LwActionListener l) {
    if (support != null) support.removeListener(l);
  }

  public void keyPressed (LwKeyEvent e)
  {
    bool b = isIntervalModel();
    switch (e.getKeyCode())
    {
      case LwConstants.VK_UP:
      case LwConstants.VK_LEFT:
      {
        int v = nextValue(value, exactStep, -1);
        if (v >= min) setValue(v);
      } break;

      case LwConstants.VK_DOWN:
      case LwConstants.VK_RIGHT:
      {
        int v = nextValue(value, exactStep, 1);
        if (v <= max) setValue(v);
      } break;
      case LwConstants.VK_HOME: setValue(b?getPointValue(0):min); break;
      case LwConstants.VK_END : setValue(b?getPointValue(getIntervals()-1):max); break;
    }
  }

  public void mousePressed (LwMouseEvent e)
  {
    if (LwToolkit.isActionMask(e.getMask()))
    {
      int x = e.getX(), y = e.getY();
      if (!getBundleBounds(value).contains(x, y))
      {
        int l = ((orient == LwToolkit.HORIZONTAL)?x:y);
        int v = loc2value(l);
        if (value != v) setValue(isJumpOnPress() ? v : nextValue(value, roughStep, v<value?-1:1));
      }
    }
  }

  private int correctDt;

  public void startDragged(LwMouseMotionEvent e)
  {
    Rectangle r = getBundleBounds(getValue());
    if (r.contains(e.getX(), e.getY()))
    {
      bits = MathBox.getBits(bits, DRAGGED_BIT, true);
      correctDt = orient == LwToolkit.HORIZONTAL?r.x + r.width/2 - e.getX():r.y + r.height/2 - e.getY();
    }
  }

  public void endDragged  (LwMouseMotionEvent e) {
    bits = MathBox.getBits(bits, DRAGGED_BIT, false);
  }

  public void mouseDragged(LwMouseMotionEvent e)
  {
    if (MathBox.checkBit(bits, DRAGGED_BIT))
      setValue (findNearest(e.getX() + (orient == LwToolkit.HORIZONTAL?correctDt:0),
                            e.getY() + (orient == LwToolkit.HORIZONTAL?0:correctDt)));
  }

  public void keyReleased  (LwKeyEvent e)    {}
  public void keyTyped     (LwKeyEvent e)    {}
  public void mouseClicked (LwMouseEvent e)  {}
  public void mouseEntered (LwMouseEvent e)  {}
  public void mouseExited  (LwMouseEvent e)  {}
  public void mouseReleased(LwMouseEvent e)  {}
  public void mouseMoved   (LwMouseMotionEvent e) {}

  public override void paint(Graphics g)
  {
    if (pl == null)
    {
      pl = new int[getIntervals()];
      for (int i=0, l=min; i<pl.Length; i++)
      {
        l += getIntervalSize(i);
        pl[i] = value2loc(l);
      }
    }

    Insets    ins = getInsets();
    Dimension bs = views[BUNDLE_VIEW].getPreferredSize();
    Dimension gs = views[GAUGE_VIEW].getPreferredSize();
    int       w = width - ins.left - ins.right - 2, h = height - ins.top - ins.bottom - 2;

    if (orient == LwToolkit.HORIZONTAL)
    {
      int topY = ins.top + (h - psH)/2 + 1;
      if (isEnabled())
        views[GAUGE_VIEW].paint(g, ins.left + 1, topY + (bs.height - gs.height)/2, w, gs.height, this);
      else
      {
        g.setColor(Color.gray);
        g.drawRect(ins.left + 1, topY + (bs.height - gs.height)/2, w, gs.height);
      }

      int by = topY;
      topY += bs.height;
      if (MathBox.checkBit(bits, SHOW_SCALE))
      {
        topY += gap;
        g.setColor(isEnabled ()?scaleColor:Color.gray);

        for (int i=min; i<=max; i+=scaleStep)
        {
          int xx = value2loc(i);
          g.drawLine(xx, topY, xx, topY + netSize);
        }

        for (int i=0; i<pl.Length; i++)
          g.drawLine(pl[i], topY, pl[i], topY + 2*netSize);

        topY += (2*netSize);
      }

      paintNums(g, topY);
      views[BUNDLE_VIEW].paint(g, getBundleLoc(value), by, this);
    }
    else
    {
      int leftX = ins.left + (w - psW)/2 + 1;
      if (isEnabled())
        views[GAUGE_VIEW].paint(g, leftX + (bs.width - gs.width)/2, ins.top + 1, gs.width, h, this);
      else
      {
        g.setColor(Color.gray);
        g.drawRect(leftX + (bs.width - gs.width)/2, ins.top + 1, gs.width, h);
      }

      int bx = leftX;
      leftX += bs.width;
      if (MathBox.checkBit(bits, SHOW_SCALE))
      {
        leftX += gap;
        g.setColor(scaleColor);

        for (int i=min; i<=max; i+=scaleStep)
        {
          int yy = value2loc(i);
          g.drawLine(leftX, yy, leftX + netSize, yy);
        }
        for (int i=0; i<pl.Length; i++)
          g.drawLine(leftX, pl[i], leftX + 2*netSize, pl[i]);

        leftX += (2*netSize);
      }
      paintNums(g, leftX);
      views[BUNDLE_VIEW].paint (g, bx, getBundleLoc (value), this);
    }

    if (hasFocus())
      LwManager.getView("br.dot").paint(g, ins.left, ins.top, w + 2, h + 2, this);
  }

  public int findNearest(int x, int y)
  {
    int v = loc2value(orient == LwToolkit.HORIZONTAL?x:y);
    if (isIntervalModel())
    {
      int nearest = System.Int32.MaxValue, res = 0;
      for (int i=0; i<intervals.Length; i++)
      {
        int pv = getPointValue(i);
        int dt = System.Math.Abs(pv - v);
        if (dt < nearest)
        {
          nearest = dt;
          res = pv;
        }
      }
      return res;
    }
    else
    {
      v = exactStep*((v + v%exactStep)/exactStep);
      if (v > max) v = max;
      else
      if (v < min) v = min;
      return v;
    }
  }

  public int value2loc (int v) {
    return (getScaleSize() * (v - min))/(max - min) + getScaleLocation();
  }

  public int loc2value (int xy) {
    return min + ((max - min)*(xy - getScaleLocation()))/getScaleSize();
  }

  public override void invalidate() {
    pl = null;
    base.invalidate();
  }

  public override Dimension calcPreferredSize() {
    return new Dimension (psW + 2, psH + 2);
  }

  public override void recalc ()
  {
    Dimension ps = views[BUNDLE_VIEW].getPreferredSize();
    int       ns = MathBox.checkBit(bits, SHOW_SCALE)?(gap + 2*netSize):0;
    int       dt = getMax() - getMin();

    int hMax = 0, wMax = 0;
    if (MathBox.checkBit(bits, SHOW_TITLE) && getIntervals()>0)
    {
      for (int i=0; i<getIntervals(); i++)
      {
        LwView    v = provider.getView(this, getPointValue(i));
        Dimension d = v.getPreferredSize();
        if (d.height > hMax) hMax = d.height;
        if (d.width  > wMax) wMax = d.width;
      }
    }

    if (orient == LwToolkit.HORIZONTAL)
    {
      psW = dt*2 + ps.width;
      psH = ps.height + ns + hMax;
    }
    else
    {
      psW = ps.width + ns + wMax;
      psH = dt*2 + ps.height;
    }
  }

  private void paintNums (Graphics g, int loc)
  {
    if (MathBox.checkBit(bits, SHOW_TITLE))
      for (int i=0; i<pl.Length; i++)
      {
        LwView render = provider.getView(this, getPointValue(i));
        Dimension d = render.getPreferredSize();
        if (orient == LwToolkit.HORIZONTAL)
          render.paint (g, pl[i] - d.width/2, loc, this);
        else
          render.paint (g, loc, pl[i] - d.height/2, this);
      }
  }

  private int getScaleSize()
  {
    Insets    ins = getInsets();
    Dimension bs = views[BUNDLE_VIEW].getPreferredSize();
    return (orient == LwToolkit.HORIZONTAL?width - ins.left - ins.right - bs.width :height - ins.top - ins.bottom - bs.height)-2;
  }

  private int getScaleLocation()
  {
    Dimension bs = views[BUNDLE_VIEW].getPreferredSize();
    return (orient == LwToolkit.HORIZONTAL?(int)((insets >> 16) & 0xFFFF) + bs.width/2
                                          :(int)(insets & 0xFFFF) + bs.height/2) + 1;
  }

  private int nextValue (int value, int s, int d)
  {
    if (isIntervalModel()) return getNeighborPoint(value, d);
    else
    {
      int v = value + (d * s);
      if (v > max) v = max;
      else
      if (v < min) v = min;
      return v;
    }
  }

  private int getBundleLoc (int v) {
    Dimension bs  = views[BUNDLE_VIEW].getPreferredSize();
    return value2loc(v) - (orient == LwToolkit.HORIZONTAL?bs.width/2:bs.height/2);
  }

  private Rectangle getBundleBounds(int v)
  {
    Insets    ins = getInsets();
    Dimension bs  = views[BUNDLE_VIEW].getPreferredSize();

    if (orient == LwToolkit.HORIZONTAL)
      return new Rectangle (getBundleLoc(v),
                            ins.top + (height - ins.top - ins.bottom - psH)/2 + 1,
                            bs.width, bs.height);
    else
      return new Rectangle (ins.left + (width - ins.left - ins.right - psW)/2 + 1,
                            getBundleLoc(v),
                            bs.width, bs.height);
  }

  private int getNeighborPoint(int v, int d)
  {
    int left = min + getIntervalSize(0), right = getPointValue(getIntervals()-1);
    if (v < left) return left;
    else
    if (v > right) return right;

    if (d > 0)
    {
      int start = min;
      for (int i=0; i<getIntervals(); i++)
      {
        start += getIntervalSize(i);
        if (start > v) return start;
      }
      return right;
    }
    else
    {
      int start = right;
      for (int i=getIntervals()-1; i>=0; i--)
      {
        if (start < v) return start;
        start -= getIntervalSize(i);
      }
      return left;
    }
  }

  private const  short INTERVAL_MODEL_BIT = 128;
  private const  short DRAGGED_BIT        = 256;
  private const  short SHOW_SCALE         = 512;
  private const  short SHOW_TITLE         = 1024;
  private const  short JUMP_ONPRESS       = 2048;
}


}
