/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.data.events;
using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.lw.mask;
using org.zaval.port.cs;
using org.zaval.util;


public class LwSpin:LwPanel,LwLayout, LwActionListener, TextListener
{
  private LwMaskTextField editor;
  private LwStButton  incButton, decButton;
  private int gapx = 1, prevValue, step = 1;
  private LwActionSupport support;

  public LwSpin()
  {
    LwAdvViewMan m1 = new LwAdvViewMan();
    m1.put("st.out", LwManager.getView("sp.b1.out"));
    m1.put("st.over", LwManager.getView("sp.b1.over"));
    m1.put("st.pressed", LwManager.getView("sp.b1.pressed"));
    m1.setBorder (LwManager.getView("br.etched"));
    incButton = new LwSpinButton(this);
    incButton.fireByPress(true, 150);
    incButton.setViewMan(m1);
    incButton.addActionListener(this);

    LwAdvViewMan m2 = new LwAdvViewMan();
    m2.put("st.out", LwManager.getView("sp.b2.out"));
    m2.put("st.over", LwManager.getView("sp.b2.over"));
    m2.put("st.pressed", LwManager.getView("sp.b2.pressed"));
    m2.setBorder (LwManager.getView("br.etched"));
    decButton = new LwSpinButton(this);
    decButton.fireByPress(true, 150);
    decButton.setViewMan(m2);
    decButton.addActionListener(this);

    editor = new LwMaskTextField();
    editor.setValidator(null);
    add (decButton);
    add (editor);
    add (incButton);
    setSpinValidator(new LwSpinValidator());
    getTextModel().addTextListener(this);
  }

  public TextModel getTextModel () {
    return editor.getTextModel();
  }

  public void enableLoop(bool b) {
    bits = MathBox.getBits(bits, LOOP_BIT, b);
  }

  public bool isLoopEnabled() {
    return MathBox.checkBit(bits, LOOP_BIT);
  }

  public void setStep(int s) {
    step = s;
  }

  public int getStep() {
    return step;
  }

  public void addActionListener(LwActionListener l) {
    if (support == null) support = new LwActionSupport();
    support.addListener(l);
  }

  public void removeActionListener(LwActionListener l) {
    if (support != null) support.removeListener(l);
  }

  public void componentAdded(object id, Layoutable lw, int index) { }
  public void componentRemoved(Layoutable lw, int index) {}

  public Dimension calcPreferredSize(LayoutContainer target)
  {
    Dimension ps1 = editor.getPreferredSize();
    Dimension ps2 = incButton.getPreferredSize();
    Dimension ps3 = decButton.getPreferredSize();
    return new Dimension (ps1.width + System.Math.Max(ps2.width, ps3.width) + gapx,
                          System.Math.Max(ps1.height, ps2.height + ps3.height));
  }

  public void layout(LayoutContainer target)
  {
    Insets    ins = target.getInsets();
    Dimension ps1 = editor.getPreferredSize();
    Dimension ps2 = incButton.getPreferredSize();
    Dimension ps3 = decButton.getPreferredSize();
    int mBtWidth = System.Math.Max(ps2.width, ps3.width);

    editor.setLocation(ins.left, ins.top);
    editor.setSize    (width  - mBtWidth - ins.left - ins.right - gapx,
                       height - ins.top - ins.bottom);

    incButton.setLocation (width - mBtWidth - ins.right, ins.top);
    incButton.setSize     (mBtWidth, ps2.height);
    decButton.setLocation (width - mBtWidth - ins.right,  height - ps3.height - ins.bottom);
    decButton.setSize     (mBtWidth, ps3.height);
  }

  public void actionPerformed (LwActionEvent e)
  {
    if (e.getSource() == incButton) setValue(getValue() + step);
    else
    if (e.getSource() == decButton) setValue(getValue() - step);
  }

  public int getValue () {
    return getSpinValidator().fetchValue(editor.getText().Trim());
  }

  public void setValue (int v)
  {
    bool b = isLoopEnabled();
    int min = getMinValue(), max = getMaxValue();
    if (v < min) v = b?max:min;
    if (v > max) v = b?min:max;
    int prev = getValue ();
    if (prev != v)
    {
      prevValue = prev;
      editor.setText(getSpinValidator().createValue(v, ((MaskedText)getTextModel()).getMask()));
      repaint();
    }
  }

  public int getMinValue () {
    return getSpinValidator().getMinValue();
  }

  public int getMaxValue () {
    return getSpinValidator().getMaxValue();
  }

  public LwSpinValidator getSpinValidator() {
    return (LwSpinValidator)editor.getValidator();
  }

  public void setSpinValidator(LwSpinValidator v)
  {
    LwSpinValidator old = getSpinValidator();
    if (old != v)
    {
      int value = (old == null)?0:getValue();
      editor.setValidator(v);
      editor.setMask(v.createMask());
      setValue(value);
      repaint();
    }
  }

  public void setBound (int min, int max)
  {
    if (min >= max) throw new org.zaval.port.cs.IllegalArgumentException();

    if (getMaxValue() != max || getMinValue() != min)
    {
      getSpinValidator().setBound(min, max);
      editor.setMask(getSpinValidator().createMask());
      setValue (getValue());
      repaint();
    }
  }

  public void textRemoved  (TextEvent e) { valueChanged(); }
  public void textInserted (TextEvent e) { valueChanged(); }
  public void textUpdated  (TextEvent e) { valueChanged(); }

  public override LwLayout getDefaultLayout () {
    return this;
  }

  public virtual void valueChanged()
  {
    if (editor.getMask().Length > 0)
    {
      int value = getValue();
      if (prevValue != value)
      {
        if (support != null) support.perform (new LwActionEvent(this, prevValue));
        prevValue = value;
      }
    }
  }

  private const  short LOOP_BIT = 128;
}

class LwSpinButton:LwSpeedButton
{
  private int saveY = 0;
  private LwSpin target;

  public LwSpinButton(LwSpin s) {
    target = s;
  }

  public override void startDragged(LwMouseMotionEvent e)
  {
    base.startDragged(e);
    saveY = e.getY();
  }

  public override void mouseDragged(LwMouseMotionEvent e)
  {
    base.mouseDragged(e);
    int y = e.getY();
    target.setValue(target.getValue() + target.getStep()*(saveY - y));
    saveY = y;
  }
}




}
