/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.mask;
using org.zaval.port.cs;


public class LwSpinValidator:BasicMaskValidator
{
  private int max = 10, min = -10;

  public override bool isValidValue(MaskElement e, string value)
  {
    if (e.getType () == NUMERIC_TYPE)
    {
      string s = value;
      if (value.Length > 0 && value[0] == '+') s = value.Substring(1);
      try {
        int iv = System.Convert.ToInt32(s);
        if (iv > max || iv < min) return false;
      }
      catch (System.FormatException) {
        return false;
      }
    }
    return base.isValidValue(e, value);
  }

  public int getMinValue () {
    return min;
  }

  public int getMaxValue () {
    return max;
  }

  public virtual string createMask()
  {
    System.Text.StringBuilder mask = new System.Text.StringBuilder();
    if  (max < 0 || min < 0) mask.Append('#');
    string s1 = System.Convert.ToString(System.Math.Abs(min));
    string s2 = System.Convert.ToString(System.Math.Abs(max));
    for (int i=0; i< System.Math.Max(s1.Length, s2.Length); i++) mask.Append('n');
    return mask.ToString();
  }

  public virtual string createValue(int value, string mask)
  {
    bool  hasSign = mask.IndexOf('#') == 0;
    string res = System.Convert.ToString(value);
    if (value < 0) res = res.Substring(1);
    System.Text.StringBuilder buf = new System.Text.StringBuilder(res);
    for (int i=0; i < (mask.Length - res.Length - (hasSign?1:0)); i++) buf.Insert(0, '0');
    if (value < 0) buf.Insert(0, '-');
    else if (hasSign) buf.Insert(0, '+');
    return buf.ToString();
  }

  public virtual int fetchValue(string maskValue) {
    if (maskValue.IndexOf('+') == 0) maskValue = maskValue.Substring(1);
    return System.Convert.ToInt32(maskValue);
  }

  public virtual void setBound (int min, int max)
  {
    if (this.max != max || this.min != min)
    {
      this.max = max;
      this.min = min;
    }
  }
}

}
