/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;
using org.zaval.util;


public class LwStButton:LwCanvas,LwMouseListener, LwMouseMotionListener, Runnable
{
   public const  int ST_ONSURFACE = 1;

   public const  int ST_PRESSED = 2;

   public const  int ST_OUTSURFACE = 3;

   public const  int ST_DISABLED = 4;

   private int state, time = 20;
   private LwActionSupport support;

   public LwStButton() {
     setState(ST_OUTSURFACE);
   }

   public bool isFireByPress() {
     return MathBox.checkBit(bits, EVENT_BY_PRESS);
   }

   public void fireByPress(bool b, int time) {
     bits      = MathBox.getBits(bits, EVENT_BY_PRESS, b);
     this.time = time;
   }

   public virtual void mouseClicked (LwMouseEvent e) {}

   public virtual void mouseEntered (LwMouseEvent e) {
     setState(ST_ONSURFACE);
   }

   public virtual void mouseExited  (LwMouseEvent e) {
     if (isEnabled()) setState(ST_OUTSURFACE);
   }

   public virtual void mousePressed (LwMouseEvent e)
   {
     if (LwToolkit.isActionMask(e.getMask()))
     {
       setState(ST_PRESSED);
       if (isFireByPress())
       {
         perform (new LwActionEvent(this));
         if (time > 0) Timer.getTimer(false).add (this, 400, time);
       }
     }
   }

   public virtual void mouseReleased(LwMouseEvent e)
   {
     if (LwToolkit.isActionMask(e.getMask()) && getState() != ST_OUTSURFACE)
     {
       setState(ST_ONSURFACE);
       if (!isFireByPress()) perform (new LwActionEvent(this));
     }
     Timer.getTimer(false).remove (this);
   }

   public virtual void startDragged(LwMouseMotionEvent e) {}
   public virtual void endDragged  (LwMouseMotionEvent e) {}
   public virtual void mouseMoved  (LwMouseMotionEvent e) {}

   public virtual void mouseDragged(LwMouseMotionEvent e)
   {
     int x = e.getX();
     int y = e.getY();
     setState(x>=0 && x<width && y>=0 && y<=height?ST_PRESSED:ST_OUTSURFACE);
   }


   public int getState () {
     return state;
   }

   public void addActionListener(LwActionListener a) {
     if (support == null) support = new LwActionSupport();
     support.addListener(a);
   }

   public void removeActionListener(LwActionListener a) {
     if (support != null) support.removeListener(a);
   }

   public override void setEnabled (bool b)
   {
     base.setEnabled(b);
     if (b) setState(ST_OUTSURFACE);
     else   setState(ST_DISABLED);
   }

   public virtual void sync() {
     stateChanged(getState(), getState());
   }

   public virtual void run () {
     if (getState() == ST_PRESSED) perform(new LwActionEvent (this));
   }

   public override void viewManChanged() {
     sync();
   }

   public virtual void setState (int s)
   {
     if (s != state)
     {
       int p = state;
       state = s;
       stateChanged(s, p);
     }
   }

   public virtual void stateChanged(int state, int prevState)
   {
     if (skins != null)
     {
       LwView n = null;
       switch (state)
       {
         case ST_ONSURFACE  :  n = skins.getView("st.over");     break;
         case ST_PRESSED    :  n = skins.getView("st.pressed");  break;
         case ST_OUTSURFACE :  n = skins.getView("st.out");      break;
         case ST_DISABLED   :  n = skins.getView("st.disabled"); break;
       }

       if (skins.getView() != n)
       {
         skins.setView(n);
         repaint();
       }
     }
   }

   public virtual void perform (LwActionEvent e) {
     if (support != null) support.perform (e);
   }

   private const  short EVENT_BY_PRESS = 64;
}



}
