/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.misc.events;
using org.zaval.misc;
using org.zaval.port.cs;


public class LwTextField:LwLabel,LwKeyListener, LwMouseListener, LwMouseMotionListener, LwFocusListener, Cursorable, PosInfo, PosListener, ScrollObj, TxtSelectionInfo
{
  private Color         selectColor = LwToolkit.darkBlue;
  private LwView        curView;
  private PosController cur;
  private ScrollMan     man;
  private int           cx = -1, cy = -1, dx = 0, dy = 0;
  private int           startLine = -1, startCol, startOff, endLine, endCol, endOff;
  private bool       isEditableVal = true;

  public LwTextField():this("") {
   
  }

  public LwTextField(string s):this(new Text(s)) {
   
  }

  public LwTextField(string s, int maxCol):this(new SingleLineTxt(s, maxCol)) {
   
    if (maxCol > 0) setPSByRowsCols(-1, maxCol);
  }

  public LwTextField(TextModel model):base(model)
  {
   
    setPosController(new PosController(this));
    setCursorView(LwManager.getView("txt.cur"));
    getViewMan(true).setBorder(LwManager.getView("br.sunken"));
  }

  public override bool canHaveFocus() {
    return true;
  }

  public PosController getPosController() {
    return cur;
  }

  public void setPosController(PosController p)
  {
    if (cur != p)
    {
      if (cur != null) cur.removePosListener(this);
      cur = p;
      cur.setPosInfo(this);
      cur.addPosListener(this);
      invalidate();
    }
  }

  public LwView  getCursorView() {
    return curView;
  }

  public void setCursorView(LwView v)
  {
    if (v != curView) {
      curView = v;
      repaint();
    }
  }

  public void setPSByRowsCols(int r, int c)
  {
    LwTextRender tr = getTextRender();
    int w = (c > 0)? (tr.stringWidth("W") * c):psWidth;
    int h = (r > 0)? (r * tr.getLineHeight() + (r - 1) * tr.getLineIndent()):psHeight;
    setPSSize(w, h);
  }

  public override void paint(Graphics g) {
    base.paint(g);
    drawCursor(g);
  }

  public void setEditable (bool b) {
    if (b != isEditableVal) isEditableVal = b;
  }

  public bool isEditable() {
    return isEditableVal;
  }

  public void keyPressed(LwKeyEvent e) {
    if (!isFiltered(e)) handleKey(e);
  }

  public void keyTyped(LwKeyEvent e)
  {
    if ((e.getMask()&LwConstants.CTRL_MASK) == 0)
    {
      char ch = (char)e.getKeyChar();
      switch (e.getKeyCode())
      {
        case LwConstants.VK_ENTER       : if (getTextModel() is SingleLineTxt) return;
                                       else ch = '\n'; break;
        case LwConstants.VK_ESCAPE      :
        case LwConstants.VK_TAB         :
        case LwConstants.VK_BACK_SPACE  : return;
      }

      if (hasSelection())
      {
        Point p = getSelectionOffsets();
        remove(p.x, p.y - p.x);
        stopSelection();
      }
      write(cur.getOffset(), UtilBox.valueOf(ch));
    }
  }

  public void focusGained  (LwFocusEvent e) {
    if (cur.getOffset() < 0) cur.setOffset(0);
  }

  public void focusLost    (LwFocusEvent e) {}
  public void keyReleased  (LwKeyEvent e)   {}
  public void mouseClicked (LwMouseEvent e) {}
  public void mouseEntered (LwMouseEvent e) {}
  public void mouseExited  (LwMouseEvent e) {}
  public void startDragged (LwMouseMotionEvent e) { startSelection(); }
  public void endDragged   (LwMouseMotionEvent e) {}
  public void mouseReleased(LwMouseEvent e) {}
  public void mouseMoved   (LwMouseMotionEvent e) {}

  public void mouseDragged (LwMouseMotionEvent e) {
    Point p = getTextRowColAt(getTextRender(), e.getX() - dx, e.getY() - dy);
    if (p != null) cur.setRowCol(p.x, p.y);
  }

  public void mousePressed (LwMouseEvent e)
  {
    if (LwToolkit.isActionMask(e.getMask()))
    {
      if ((e.getMask() & LwConstants.SHIFT_MASK) > 0) startSelection();
      else stopSelection();
      Insets i = getInsets();
      Point p = getTextRowColAt(getTextRender(), e.getX() - dx - i.left, e.getY() - dy - i.top);
      if (p != null) cur.setRowCol(p.x, p.y);
    }
  }

  public int getCursorType (int x, int y) {
    return Cursor.TEXT_CURSOR;
  }

  public int getLineSize(int i) {
    return getTextModel().getLine(i).Length + 1;
  }

  public int getLines() {
    return getTextModel().getSize();
  }

  public int getMaxOffset() {
    return getTextModel().getTextLength();
  }

  public override void setText(string s)
  {
    cur.setOffset(0);
    if (man != null) man.scrollObjMoved(0, 0);
    else             setSOLocation(0, 0);
    base.setText(s);
  }

  public virtual void posChanged(PosEvent e)
  {
    cur.validate();
    if (cur.getOffset() >=0)
    {
      LwTextRender r = getTextRender();
      Point  p = getTextLocationAt(r, cur);
      Insets i = getInsets();

      cx = p.x + i.left;
      cy = p.y + i.top;
      Point o = LwToolkit.calcOrigin(cx, cy, curView.getPreferredSize().width, r.getLineHeight(), this);
      if (man != null) man.scrollObjMoved(o.x, o.y);
      else             setSOLocation(o.x, o.y);

      bool isSelStarted = isSelectionStarted();
      if (e.getPrevLine() >= 0 && !isSelStarted)
      {
        int minUpdatedLine = System.Math.Min(e.getPrevLine(), cur.getCurrentLine());
        int maxUpdatedLine = System.Math.Max(e.getPrevLine(), cur.getCurrentLine());

        int lh = r.getLineHeight(), li = r.getLineIndent();
        int y1 = lh * minUpdatedLine + minUpdatedLine * li + i.top + dy;
        int y2 = lh * maxUpdatedLine + maxUpdatedLine * li + i.top + dy;
        repaint (i.left, y1, width, y2 - y1 + r.getLineHeight()) ;
      }
      else
      {
        if (isSelStarted)
        {
          endLine = cur.getCurrentLine();
          endCol  = cur.getCurrentCol();
          endOff  = cur.getOffset();
        }
        repaint();
      }
    }
  }

  public override Point getOrigin() {
    return new Point(dx, dy);
  }

  public override Dimension getPreferredSize()
  {
    Dimension d1 = base.getPreferredSize();
    Dimension d2 = curView.getPreferredSize();
    d1.width += d2.width;
    return d1;
  }

  public Point getSOLocation () {
    return getOrigin();
  }

  public void setSOLocation (int x, int y)
  {
    if (x != dx || y != dy)
    {
      dx = x;
      dy = y;
      repaint();
    }
  }

  public Dimension getSOSize() {
    return getPreferredSize();
  }

  public void setScrollMan (ScrollMan m) {
    man = m;
  }

  public bool moveContent   () {
    return true;
  }

  public Point getStartSelection()
  {
    if (isSelectionStarted())
      return (startOff < endOff)?new Point(startLine, startCol)
                                :new Point(endLine, endCol);
    else return new Point(-1, -1);
  }

  public Point getEndSelection()
  {
    if (isSelectionStarted())
      return (startOff < endOff)?new Point(endLine, endCol)
                                :new Point(startLine, startCol);
    else return new Point(-1, -1);
  }

  public bool hasSelection() {
    return isSelectionStarted() && startOff != cur.getOffset();
  }

  public Point getSelectionOffsets()
  {
    if (isSelectionStarted())
      return (startOff < endOff)?new Point(startOff, endOff)
                                :new Point(endOff, startOff);
    else return new Point(-1, -1);
  }

  public string getSelectedText() {
    return hasSelection()?getSubString(getTextModel(), getStartSelection(), getEndSelection()):null;
  }

  public Color getSelectColor () {
    return selectColor;
  }

  public void setSelectColor (Color c)
  {
    if (!c.Equals(selectColor)) {
      selectColor = c;
      if (hasSelection()) repaint();
    }
  }

  public override void setEnabled (bool b) {
    stopSelection();
    base.setEnabled(b);
  }

  public virtual void select (int startOffset, int endOffset)
  {
    if (endOffset   <= startOffset ||
        startOffset < 0            ||
        endOffset   > cur.getMaxOffset()) throw new org.zaval.port.cs.IllegalArgumentException();

    stopSelection();
    if (startOff != startOffset || endOffset != endOff)
    {
      startOff = startOffset;
      Point  p1 = PosController.getPointByOffset(startOffset, cur);
      startLine =  p1.x;
      startCol  =  p1.y;

      endOff = endOffset;
      Point  p2 = PosController.getPointByOffset(endOffset, cur);
      endLine = p2.x;
      endCol  = p2.y;

      repaint();
    }
  }

  public virtual void drawCursor(Graphics g)  {
    if (isEditableVal && hasFocus() && cur.getOffset() >= 0)
      curView.paint(g, cx, cy, curView.getPreferredSize().width, getTextRender().getLineHeight() - 1, this);
  }

  public override void recalc()
  {
    base.recalc();
    stopSelection();
    if (cur.getOffset() >= 0)
    {
      cur.validate();
      int row = cur.getCurrentLine();
      int col = cur.getCurrentCol ();
      TextModel text = getTextModel();
      if (row >= text.getSize() || (row > 0 && col > text.getLine(row).Length)) cur.setOffset(0);
    }
  }

  public virtual bool handleKey(LwKeyEvent e)
  {
    int     mask          = e.getMask();
    bool isControlDown = (mask & LwConstants.CTRL_MASK ) > 0;
    bool isShiftDown   = (mask & LwConstants.SHIFT_MASK) > 0;
    if (isShiftDown) startSelection();
    switch (e.getKeyCode())
    {
      case LwConstants.VK_DOWN  : cur.seekLineTo(PosController.DOWN); break;
      case LwConstants.VK_UP    : cur.seekLineTo(PosController.UP); break;
      case LwConstants.VK_RIGHT :
      {
        if (isControlDown)
        {
          Point p = findNextWord(getTextModel(), cur.getCurrentLine(), cur.getCurrentCol(), 1);
          if (p != null) cur.setRowCol(p.x, p.y);
        }
        else cur.seek(1);
      } break;
      case LwConstants.VK_LEFT  :
      {
        if (isControlDown)
        {
          Point p = findNextWord(getTextModel(), cur.getCurrentLine(), cur.getCurrentCol(), -1);
          if (p != null) cur.setRowCol(p.x, p.y);
        }
        else  cur.seek(-1);
      } break;
      case LwConstants.VK_END   :
      {
        if (isControlDown) cur.seekLineTo(PosController.DOWN, cur.getPosInfo().getLines() - cur.getCurrentLine() - 1);
        else               cur.seekLineTo(PosController.END);
      } break;
      case LwConstants.VK_HOME  :
      {
        if (isControlDown) cur.seekLineTo(PosController.UP, cur.getCurrentLine());
        else               cur.seekLineTo(PosController.BEG);
      } break;
      case LwConstants.VK_PAGE_DOWN : cur.seekLineTo(PosController.DOWN, pageSize()); break;
      case LwConstants.VK_PAGE_UP   : cur.seekLineTo(PosController.UP, pageSize()); break;
      case LwConstants.VK_DELETE    :
      {
        if (hasSelection())
        {
          if (isShiftDown) LwClipboardMan.manager.put(getSelectedText());
          Point p = getSelectionOffsets();
          remove(p.x, p.y - p.x);
        }
        else remove(cur.getOffset(), 1);
      } break;
      case LwConstants.VK_BACK_SPACE: if (!hasSelection()) remove(cur.getOffset()-1, 1);
                                   else { Point p = getSelectionOffsets(); remove(p.x, p.y - p.x); }
                                   break;
      case LwConstants.VK_INSERT    :
      {
        if (hasSelection() && isControlDown)
        {
          LwClipboardMan.manager.put(getSelectedText());
          return true;
        }
        else
        if (isShiftDown)
        {
          string s = (string)LwClipboardMan.manager.get();
          if (s != null)
          {
            if (hasSelection())
            {
              Point p = getSelectionOffsets();
              remove(p.x, p.y - p.x);
            }
            write(cur.getOffset(), s);
          }
        }
      } break;
      default: return false;
    }

    if (!isShiftDown) stopSelection();
    return true;
  }

  public virtual bool isFiltered(LwKeyEvent e)
  {
    int code = e.getKeyCode();
    return code == LwConstants.VK_SHIFT || code == LwConstants.VK_CONTROL ||
           code == LwConstants.VK_TAB   || code == LwConstants.VK_ALT ||
           (e.getMask() & LwConstants.ALT_MASK) > 0;
  }

  public override LwTextRender makeTextRender(TextModel t) {
    return new LwAdvTextRender(t);
  }

  public virtual void remove(int pos, int size)
  {
    if (isEditableVal)
    {
      int max = cur.getMaxOffset();
      int pl  = getLines();
      if (pos >= 0 && (pos + size) <= max)
      {
        int old = cur.getOffset();
        cur.setOffset(pos);
        getTextModel().remove(pos, size);
        if (man != null) {
          Dimension d = getSOSize();
          man.scrollObjResized(d.width, d.height);
        }
        if (getLines() != pl || old == pos) repaint();
      }
    }
  }

  public virtual void write(int pos, string s)
  {
    if (isEditableVal)
    {
      int old = cur.getOffset();
      int pl  = getLines();
      getTextModel().write(s, pos);
      if (man != null)  {
        Dimension d = getSOSize();
        man.scrollObjResized(d.width, d.height);
      }
      cur.seek (s.Length);
      if (getLines() != pl || cur.getOffset() == old) repaint();
    }
  }

  public virtual int pageSize()
  {
    Dimension d   = getSize();
    Insets    ins = getInsets();
    d.height -= (ins.top + ins.bottom);
    LwTextRender render = getTextRender();
    int indent = render.getLineIndent();
    int height = render.getLineHeight();
    return (d.height + indent)/(height + indent) +
           (((d.height + indent)%(height + indent)>indent)?1:0);
  }

  public void stopSelection()
  {
    if (startLine >= 0) {
      startLine = -1;
      repaint();
    }
  }

  private bool isSelectionStarted() {
    return startLine != -1;
  }

  private void startSelection()
  {
    if (!isSelectionStarted())
    {
      startLine = cur.getCurrentLine();
      startCol  = cur.getCurrentCol();
      startOff  = cur.getOffset();
    }
  }


   public static Point getTextLocationAt(LwTextRender render, PosController pos)
   {
     if (pos.getOffset() < 0) return null;
     int cl = pos.getCurrentLine();
     return new Point(render.substrWidth(render.getLine(cl), 0, pos.getCurrentCol()),
                      cl * (render.getLineHeight() + render.getLineIndent()));
   }

   public static Point getTextRowColAt(LwTextRender render, int x, int y)
   {
     int size = render.getTextModel().getSize();
     if (size == 0) return null;

     int lineHeight = render.getLineHeight();
     int lineIndent = render.getLineIndent();
     int lineNumber = (y<0)?0:(y  + lineIndent)/(lineHeight + lineIndent) +
                      ((y  + lineIndent)%(lineHeight + lineIndent)>lineIndent?1:0) - 1;

     if (lineNumber >= size) return new Point (size - 1, render.getLine(size - 1).Length);
     else
     if (lineNumber < 0) return new Point();

     if (x < 0) return new Point(lineNumber, 0);

     int x1 = 0, x2 = 0;
     string s = render.getLine(lineNumber);
     for(int c = 0; c < s.Length; c++)
     {
       x1 = x2;
       x2 = render.substrWidth(s, 0, c + 1);
       if (x >= x1 && x < x2) return new Point(lineNumber, c);
     }
     return new Point (lineNumber, s.Length);
   }

   public static Point findNextWord(TextModel t, int line, int col, int d)
   {
     if (line < 0 || line >= t.getSize()) return null;

     string ln = t.getLine(line);
     col += d;

     if (col < 0 && line > 0) return new Point (line - 1, t.getLine(line - 1).Length);
     else
     if (col > ln.Length && line < t.getSize() - 1) return new Point (line + 1, 0);

     char[]  buf = ln.ToCharArray();
     bool b   = false;
     int     j   = col;
     for (; j >= 0 && j < buf.Length; j += d)
     {
       if (b)
       {
         if (d > 0)
         {
           if (System.Char.IsLetter(buf[j])) return new Point (line, j);
         }
         else
           if (!System.Char.IsLetter(buf[j])) return new Point (line, j + 1);
       }
       else
       {
         if (d > 0) b = !System.Char.IsLetter(buf[j]);
         else       b = System.Char.IsLetter(buf[j]);
       }
     }
     return (d > 0?new Point(line, buf.Length):new Point (line, 0));
   }

   public static string getSubString(TextModel t, Point start, Point end)
   {
     System.Text.StringBuilder res = new System.Text.StringBuilder();
     for (int i=start.x; i<end.x + 1; i++)
     {
       string ln = t.getLine(i);
       if (i != start.x) res.Append('\n');
       if (i == start.x) ln = ln.Substring(start.y);
       if (i == end.x  ) ln = ln.Substring(0, end.y - ((start.x == end.x)?start.y:0));
       res.Append(ln);
     }
     return res.ToString();
   }
}


}
