/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.misc;
using org.zaval.port.cs;


public class LwTitledBorder:LwBorder
{
    private int lineAlignment;
    public LwTitledBorder(int type):this(type, Alignment.BOTTOM) {
     
    }

    public LwTitledBorder(int type, int a):base(type)
    {
     
      if (a != Alignment.BOTTOM && a != Alignment.TOP && a != Alignment.CENTER)
        throw new org.zaval.port.cs.IllegalArgumentException();
      lineAlignment = a;
    }

    public override void paint(Graphics g, int x, int y, int w, int h, Drawable d)
    {
      if (d is LwTitleInfo)
      {
        Rectangle r = ((LwTitleInfo)d).getTitleBounds();
        int       o = ((LwTitleInfo)d).getTitleAlignment();
        int       borderType = getBorderType();

        int xx = w - 1;
        int yy = h - 1;
        switch (o)
        {
          case Alignment.TOP:
          {
            if (lineAlignment == Alignment.TOP)    y = r.y;
            else
            if (lineAlignment == Alignment.BOTTOM) y = r.y + r.height - 1;
            else
            if (lineAlignment == Alignment.CENTER) y = r.y + r.height/2;

            leftLine(g, x, y, x, yy);
            rightLine(g, xx, y, xx, yy);
            topLine(g, x, y, r.x, y);
            topLine(g, r.x + r.width, y, xx, y);
            bottomLine(g, x, yy, xx, yy);
          } break;
          case Alignment.LEFT:
          {
            if (lineAlignment == Alignment.TOP)    x = r.x;
            else
            if (lineAlignment == Alignment.BOTTOM) x = r.x + r.width - 1;
            else
            if (lineAlignment == Alignment.CENTER) x = r.x + r.width/2;

            leftLine(g, x, y, x, r.y);
            leftLine(g, x, r.y + r.height, x, yy);
            rightLine(g, xx, y, xx, yy);
            topLine(g, x, y, xx, y);
            bottomLine(g, x, yy, xx, yy);
          } break;
          case Alignment.BOTTOM:
          {
            if (lineAlignment == Alignment.TOP)    yy = r.y + r.height - 1;
            else
            if (lineAlignment == Alignment.BOTTOM) yy = r.y;
            else
            if (lineAlignment == Alignment.CENTER) yy = r.y + r.height/2;

            leftLine(g, x, y, x, yy);
            rightLine(g, xx, y, xx, yy);
            topLine(g, x, y, xx, y);
            bottomLine(g, x, yy, r.x, yy);
            bottomLine(g, r.x + r.width, yy, xx, yy);
          } break;
          case Alignment.RIGHT :
          {
            if (lineAlignment == Alignment.TOP)    xx = r.x + r.width - 1;
            else
            if (lineAlignment == Alignment.BOTTOM) xx = r.x;
            else
            if (lineAlignment == Alignment.CENTER) xx = r.x + r.width/2;

            leftLine(g, x, y, x, yy);
            rightLine(g, xx, y, xx, r.y);
            rightLine(g, xx, r.y + r.height, xx, yy);
            topLine(g, x, y, r.x, y);
            bottomLine(g, x, yy, xx, yy);
          } break;
        }
      }
      else
        base.paint(g, x, y, w, h, d);
    }
}

}
