/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.port.cs;
using org.zaval.util;


public abstract class LwView:ValidationObject
{
  public const  int STRETCH  = 1;

  public const  int MOSAIC   = 2;

  public const  int ORIGINAL = 3;

  private int type = ORIGINAL;

  public LwView():this(MOSAIC) {
   
  }

  public LwView(int type) {
    setType(type);
  }

  public int getType() {
    return type;
  }

  public void setType(int t)
  {
    if (t != STRETCH && t != MOSAIC && t != ORIGINAL) throw new org.zaval.port.cs.IllegalArgumentException();
    if (type != t)
    {
      type = t;
      invalidate();
    }
  }

  public virtual Insets  getInsets() {
    return new Insets(0, 0, 0, 0);
  }

  public virtual Dimension getPreferredSize () {
    validate();
    return calcPreferredSize ();
  }

  public virtual Dimension calcPreferredSize () {
    return new Dimension();
  }

  public override void recalc() {}

  public virtual void paint(Graphics g, int x, int y, Drawable d)
  {
    //???
    validate();
    Dimension size = calcPreferredSize ();
    paint(g, x, y, size.width, size.height, d);
  }

  public abstract void paint(Graphics g, int x, int y, int w, int h, Drawable d);

  public virtual void ownerChanged(Validationable v) {}
}


}
