/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.port.cs;
using org.zaval.util.events;
using org.zaval.util;


public class LwViewMan:ValidationObject
{
  private Drawable parent;
  private LwView  bg, border, view;
  private Insets insets;
  private int width, height;

  public void setParent(Drawable p)
  {
    if (p != parent)
    {
      parent = p;
      invalidate();
    }
  }

  public Drawable getParent () {
    return parent;
  }

  public Insets getInsets() {
    validate();
    return insets;
  }

  public LwView getBg() {
    return bg;
  }

  public LwView getBorder() {
    return border;
  }

  public LwView getView() {
    return view;
  }

  public void setBg(string s) {
    setBg(getView(s));
  }

  public void setBorder(string s) {
    setBorder(getView(s));
  }

  public void setView (string s) {
    setView(getView(s));
  }

  public void setBg(LwView s)
  {
    if (s != bg)
    {
      bg = s;
      invalidate();
    }
  }

  public void setBorder(LwView s)
  {
    if (s != border)
    {
      LwView old = border;
      border = s;
      if (!isValid(old, view)) invalidate();
    }
  }

  public void setView(LwView s)
  {
    if (s != view)
    {
      LwView old = view;
      view = s;
      if (view != null) view.ownerChanged(this);
      if (!isValid(old, view)) invalidate();
    }
  }

  public override void invalidate() {
    if (parent != null) parent.invalidate();
    base.invalidate();
  }

  public override void recalc ()
  {
    LwView[] s = getAllViews();
    for (int i=0; i<s.Length; i++) if (s[i] != null) s[i].validate();
    insets = calcInsets(this);

    Dimension ps = new Dimension();
    for (int i=0; i<s.Length; i++)
      if (s[i] != null && (s[i].getType() == LwView.ORIGINAL || s[i].getType() == LwView.STRETCH))
        ps = MathBox.max(s[i].getPreferredSize(), ps);

    width  = ps.width;
    height = ps.height;
  }

  public virtual Dimension getPreferredSize() {
    validate();
    return new Dimension (width, height);
  }

  public virtual LwView getView(string key) {
    return LwManager.getView(key);
  }

  public virtual LwView[] getAllViews() {
    return  new LwView[] { view, border, bg };
  }

  private static Insets calcInsets(LwViewMan view)
  {
    LwView[] skins = view.getAllViews();
    Insets   res   = new Insets(0, 0, 0, 0);
    for (int i=0; i<skins.Length; i++)
      if (skins[i] != null) res = MathBox.max(res, skins[i].getInsets());
    return res;
  }

  private static bool isValid(LwView v1, LwView v2)
  {
    return ( v1 != null && v2 != null &&
             v1.getPreferredSize().Equals(v2.getPreferredSize()) &&
             v1.getInsets().Equals(v2.getInsets()));
  }
}

}
