/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.port.cs;



public class LwWinLayer:LwLayer,LwLayout, LwChildrenListener
{
  public static readonly  object ID = "win";

  public static readonly  int MODAL_WIN = 1;

  public static readonly  int MDI_WIN = 2;

  public static readonly  int INFO_WIN = 3;

  private Hashtable   winsInfo  = new Hashtable();
  private Vector      winsStack = new Vector();
  private int         topModalIndex = -1;
  private LwComponent activeWin;

  public LwWinLayer():base(ID) {
   
    setOpaque (false);
  }

  public void childPerformed(LwAWTEvent e)
  {
    if (e.getID() == LwFocusEvent.FOCUS_GAINED)
    {
      LwComponent child = LwToolkit.getDirectChild(this, e.getLwComponent());
      activate(child);
    }
  }

  public void add (LwComponent c, LwWinListener l, int type) {
    add (new object[] {type, l},  c);
  }

  public void componentAdded(object id, Layoutable lw, int index)
  {
    if (id == null) throw new org.zaval.port.cs.IllegalArgumentException();

    object[] info = new object[3];
    info[0] = activeWin;
    if (id is int) info[1] = id;
    else
    if (id is LwWinListener) info[2] = id;
    else
    {
      info[1] = ((object[])id)[0];
      info[2] = ((object[])id)[1];
    }

    if (info[1] == null) info[1] = MDI_WIN;
    if (!info[1].Equals(MDI_WIN) && !info[1].Equals(MODAL_WIN) && !info[1].Equals(INFO_WIN))
      throw new org.zaval.port.cs.IllegalArgumentException();

    winsInfo.put(lw, info);
    winsStack.addElement(lw);
    pw((LwWinListener)info[2], new LwWinEvent((LwComponent)lw, LwWinEvent.WIN_OPENED));

    if (info[1].Equals(MODAL_WIN))
    {
      topModalIndex = winsStack.size()-1;
      activate((LwComponent)lw);
    }

    repaint();
  }

  public void componentRemoved(Layoutable lw, int index)
  {
    //
    // Test if the window is active
    //
    if (activeWin == lw)
    {
      activeWin = null;
      LwFocusManager.manager.requestFocus(null);
    }

    //
    // Validate topModalIndex
    //
    int ci = winsStack.IndexOf(lw);

    LwWinListener l = getWinListener((LwComponent)lw);
    winsInfo.remove(lw);
    winsStack.removeElement(lw);

    if (ci < topModalIndex) topModalIndex--;
    else
    if (topModalIndex == ci)
    {
      for (topModalIndex=count()-1; topModalIndex>=0; topModalIndex--)
        if (getWinType((LwComponent)winsStack.elementAt(topModalIndex)).Equals(MODAL_WIN))
          break;
    }

    pw(l, new LwWinEvent((LwComponent)lw, LwWinEvent.WIN_CLOSED));

    if (topModalIndex >= 0)
    {
      int aindex = winsStack.size() - 1;
      while (getWinType((LwComponent)winsStack.elementAt(aindex)).Equals(INFO_WIN)) aindex--;
      activate((LwComponent)winsStack.elementAt(aindex));
    }
  }

  public override LwComponent getLwComponentAt (int x, int y) {
    return (activeWin == null)?null:activeWin.getLwComponentAt(x - activeWin.getX(), y - activeWin.getY());
  }

  public Dimension calcPreferredSize(LayoutContainer target) {
    return LwToolkit.getMaxPreferredSize(target);
  }

  public override void invalidate() {
    isValidFlag = false;
  }

  public override void mousePressed (int x, int y, int mask)
  {
    int cnt = count();
    if (cnt > 0)
    {
      if (activeWin != null && indexOf (activeWin) == cnt-1)
      {
        int x1 = activeWin.getX(), y1 = activeWin.getY();
        int x2 = x1 + activeWin.getWidth(), y2 = y1 + activeWin.getHeight();
        if (x >= x1 && y >= y1 && x < x2 && y < y2) return;
      }

     for (int i=cnt-1;i>=0 && i>=topModalIndex;i--)
      {
        LwCanvas d = (LwCanvas)children.elementAt(i);
        if (d.isVisible() && d.isEnabled() && !getWinType(d).Equals(INFO_WIN) &&
            x >= d.x && y >= d.y && x < d.x + d.width && y < d.y + d.height)
        {
          activate(d);
          return;
        }
      }

      if (topModalIndex < 0 && activeWin != null)  activate (null);
    }
  }

  public void layout(LayoutContainer target)
  {
    Dimension d = target.getSize();
    for (int i=0; i<target.count(); i++)
    {
      Layoutable l = target.get(i);
      if (l.isVisible())
      {
        int x = l.getX(), y = l.getY(), w = l.getWidth(), h = l.getHeight();
        int minH = System.Math.Min(VIS_PART_SIZE, h), minW = System.Math.Min(VIS_PART_SIZE, w);
        if (x > target.getWidth() - minW) x = target.getWidth() - minW;
        else
        if (x + w < minW) x = minW - w;

        if (y > target.getHeight() - minH) y = target.getHeight() - minH;
        else
        if (y < 0) y = 0;

        l.setLocation (x, y);
      }
    }
  }

  public void activate (LwComponent c)
  {
    if (c != null && (winsInfo.get(c) == null || getWinType(c).Equals(INFO_WIN)))
      throw new org.zaval.port.cs.IllegalArgumentException();

    if (c != activeWin)
    {
       if (c == null)
       {
         if (getWinType(activeWin).Equals(MODAL_WIN)) throw new org.zaval.port.cs.IllegalArgumentException();

         LwComponent old = activeWin;
         activeWin = null;
         pw(getWinListener(old), new LwWinEvent(old, LwWinEvent.WIN_DEACTIVATED));
         LwFocusManager.manager.requestFocus(null);
       }
       else
       {
         if (winsStack.IndexOf(c) < topModalIndex) throw new org.zaval.port.cs.IllegalArgumentException();

         LwComponent old = activeWin;
         activeWin = c;
         toFront(activeWin);

         if (old != null)
           pw(getWinListener(old), new LwWinEvent(old, LwWinEvent.WIN_DEACTIVATED));

         pw(getWinListener(activeWin), new LwWinEvent(activeWin, LwWinEvent.WIN_ACTIVATED));
         LwFocusManager.manager.requestFocus(LwFocusManager.manager.findFocusable(activeWin));
       }
    }
  }

  public override void keyPressed  (int keyCode, int mask)
  {
    if (count() > 0 && keyCode == LwConstants.VK_TAB && (mask & LwConstants.SHIFT_MASK) > 0)
    {
      if (activeWin == null) activate((LwComponent)get(count()-1));
      else
      {
        int winIndex = winsStack.IndexOf(activeWin) - 1;
        if (winIndex < topModalIndex || winIndex < 0) winIndex = winsStack.size() - 1;
        activate ((LwComponent)winsStack.elementAt(winIndex));
      }
    }
  }

  public LwComponent getActive () {
    return activeWin;
  }

  public override bool isActive () {
    return activeWin != null;
  }

  public override LwComponent getFocusRoot() {
    return activeWin;
  }

  public override LwLayout getDefaultLayout() {
    return this;
  }

  private int getWinType (LwComponent w) {
    return (int)((object[])winsInfo.get(w))[1];
  }

  private LwComponent getWinOwner (LwComponent w) {
    return (LwComponent)((object[])winsInfo.get(w))[0];
  }

  private LwWinListener getWinListener(LwComponent c) {
    return (LwWinListener)((object[])winsInfo.get(c))[2];
  }

  private void pw (LwWinListener l, LwWinEvent e) {
    LwEventManager.manager.perform (e);
    if (l != null) LwWinSupport.process(l, e);
  }

  private const  int VIS_PART_SIZE = 30;
}


}
