/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.misc;
using org.zaval.port.cs;
using org.zaval.util;


public class LwWindow:LwPanel,LwMouseMotionListener, LwWinListener, LwComposite, LwActionListener, Cursorable
{
  private LwPanel root, icons;
  private LwLabel title = new LwLabel();
  private LwComponent corner, icon;
  private Color activeColor = LwToolkit.darkBlue, nonactiveColor = Color.lightGray;
  private LwStButton exit;
  private int cursor = -1, px, py, dx = 0, dy = 0, action = -1;

  public LwWindow():this("") {
   
  }

  public LwWindow(string s)
  {
    LwPanel titlePanel = new LwPanel();
    titlePanel.setBackground(nonactiveColor);
    titlePanel.setLwLayout(new LwBorderLayout (2, 2));
    titlePanel.setInsets(1,1,1,1);

    add (LwBorderLayout.NORTH, titlePanel);
    root = new LwPanel();
    add (LwBorderLayout.CENTER, root);
    getViewMan(true).setBorder("br.raised");

    exit = new LwStButton();
    LwAdvViewMan man = new LwAdvViewMan();
    man.put ("st.over", LwManager.getView("win.b.over"));
    man.put ("st.out",  LwManager.getView("win.b.out"));
    man.put ("st.pressed", LwManager.getView("win.b.pressed"));
    exit.setViewMan(man);
    exit.addActionListener(this);
    title.setOpaque(false);
    title.setForeground (Color.white);
    title.setText(s);
    LwPanel buttons = new LwPanel();
    buttons.setLwLayout(new LwFlowLayout(Alignment.CENTER, Alignment.CENTER));
    buttons.add (exit);
    buttons.setOpaque (false);

    icons = new LwPanel();
    icons.setLwLayout(new LwFlowLayout(Alignment.CENTER, Alignment.CENTER));
    icon = new LwImage((LwImgRender)LwManager.getView("win.icon"));
    icons.add (icon);

    titlePanel.add (LwBorderLayout.EAST, buttons);
    titlePanel.add (LwBorderLayout.WEST, icons);
    titlePanel.add (LwBorderLayout.CENTER, title);

    LwPanel status = new LwPanel();
    status.setLwLayout(new LwFlowLayout(Alignment.RIGHT, Alignment.CENTER));
    corner = new LwImage((LwImgRender)LwManager.getView("win.corner"));
    status.add (corner);
    add (LwBorderLayout.SOUTH, status);

    setSizeable (true);
    hideOnClose (true);
  }

  public void setTitle (string s) {
    title.setText(s);
  }

  public string getTitle () {
    return title.getText();
  }

  public void setIcon (LwComponent i)
  {
    if (i != icon)
    {
      icons.remove(icon);
      icon = i;
      if (icon != null) icons.add (icon);
      repaint();
    }
  }

  public void setCaptionColor(bool state, Color c)
  {
    Color old = state?activeColor:nonactiveColor;
    if (!old.Equals(c))
    {
      if (state) activeColor = c;
      else       nonactiveColor = c;
      repaint();
    }
  }

  public bool isHideOnClose () {
    return MathBox.checkBit(bits, ONCLOSE_BIT);
  }

  public void hideOnClose (bool b){
    if (isHideOnClose() != b) bits = MathBox.getBits(bits, ONCLOSE_BIT, b);
  }

  public bool isSizeable() {
    return MathBox.checkBit(bits, SIZABLE_BIT);
  }

  public void setSizeable(bool b) {
    if (isSizeable() != b) bits = MathBox.getBits(bits, SIZABLE_BIT, b);
  }

  public LwContainer getRoot () {
    return root;
  }

  public void startDragged(LwMouseMotionEvent e)
  {
    if (cursor == -1 || isSizeable())
    {
      action = cursor==-1?MOVE_ACTION:SIZE_ACTION;
      px = e.getX();
      py = e.getY();
      dx = 0;
      dy = 0;
      LwToolkit.drawXORRect(this, 0, 0, width, height);
    }
  }

  public void mouseDragged(LwMouseMotionEvent e)
  {
    if (action > 0)
    {
      if (action == MOVE_ACTION) LwToolkit.drawXORRect(this, dx, dy, width, height);
      else                       LwToolkit.drawXORRect(this, 0, 0, width + dx, height + dy);
      dx = (e.getX() - px);
      dy = (e.getY() - py);
      if (action == MOVE_ACTION) LwToolkit.drawXORRect(this, dx, dy, width, height);
      else                       LwToolkit.drawXORRect(this, 0, 0, width + dx, height + dy);
    }
  }

  public void endDragged(LwMouseMotionEvent e)
  {
    if (action > 0)
    {
      if (action == MOVE_ACTION)
      {
        LwToolkit.drawXORRect(this, dx, dy, width, height);
        int xx = x + dx;
        int yy = y + dy;
        // don't touch the invalidate
        invalidate();
        setLocation (xx, yy);
      }
      else
      {
        LwToolkit.drawXORRect(this, 0, 0, width + dx, height + dy);
        int nw = width + dx, nh = height + dy;
        if (nw > MIN_SIZE && nh > MIN_SIZE) setSize (nw, nh);
      }
      action = -1;
    }
  }

  public void mouseMoved  (LwMouseMotionEvent e) {}

  public void winOpened (LwWinEvent e)  {}

  public void winClosed (LwWinEvent e) {
    getTitlePanel().setBackground(nonactiveColor);
  }

  public void winActivated (LwWinEvent e) {
    getTitlePanel().setBackground(activeColor);
  }

  public void winDeactivated (LwWinEvent e) {
    getTitlePanel().setBackground(nonactiveColor);
  }

  public void actionPerformed (LwActionEvent e) {
    LwLayer l = LwToolkit.getDesktop(this).getLayer(LwWinLayer.ID);
    if (e.getSource() == exit && isHideOnClose()) l.remove(this);
  }

  public int getCursorType (int x, int y) {
    cursor = (getLwComponentAt(x, y) == corner && isSizeable())?Cursor.SE_RESIZE_CURSOR:-1;
    return cursor;
  }

  public bool catchInput (LwComponent c) {
    LwComponent tp = getTitlePanel();
    return c != exit && (c == tp || LwToolkit.isAncestorOf(tp, c) || c == corner);
  }

  public LwComponent getTitlePanel() {
    return (LwComponent)get(0);
  }

  public override LwLayout getDefaultLayout() {
    return new LwBorderLayout();
  }

  private const  short SIZABLE_BIT = 128;
  private const  short ONCLOSE_BIT = 256;

  private const  short MIN_SIZE = 40;
  private const  int   MOVE_ACTION = 1;
  private const  int   SIZE_ACTION = 2;
}



}
