/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.events {


using org.zaval.lw;
using org.zaval.port.cs;
using org.zaval.util.events;


public class LwEventManager
{
  public static LwEventManager manager;

  private LwMouseSupport       mouse;
  private LwMouseMotionSupport mouseMotion;
  private LwComponentSupport   component;
  private LwContainerSupport   container;
  private LwKeySupport         keyboard;
  private LwFocusSupport       focus;
  private LwWinSupport         window;

  public void addXXXListener (EventListener l)
  {
    if (l is LwComponentListener)  addComponentListener  ((LwComponentListener)l);
    if (l is LwContainerListener)  addContainerListener  ((LwContainerListener)l);
    if (l is LwMouseListener) 	   addMouseListener      ((LwMouseListener)l);
    if (l is LwMouseMotionListener)addMouseMotionListener((LwMouseMotionListener)l);
    if (l is LwKeyListener)        addKeyListener        ((LwKeyListener)l);
    if (l is LwFocusListener)	   addFocusListener      ((LwFocusListener)l);
    if (l is LwWinListener)	   addWinListener        ((LwWinListener)l);
  }

  public void removeXXXListener (EventListener l)
  {
    if (l is LwComponentListener)   removeComponentListener  ((LwComponentListener)l);
    if (l is LwContainerListener)   removeContainerListener  ((LwContainerListener)l);
    if (l is LwMouseListener)       removeMouseListener      ((LwMouseListener)l);
    if (l is LwMouseMotionListener) removeMouseMotionListener((LwMouseMotionListener)l);
    if (l is LwKeyListener)         removeKeyListener        ((LwKeyListener)l);
    if (l is LwFocusListener)       removeFocusListener      ((LwFocusListener)l);
    if (l is LwWinListener)         removeWinListener        ((LwWinListener)l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addComponentListener(LwComponentListener l) {
    if (component == null) component = new LwComponentSupport();
    component.addListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeComponentListener(LwComponentListener l) {
    if (component != null) component.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addContainerListener(LwContainerListener l) {
    if (container == null) container = new LwContainerSupport();
    container.addListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeContainerListener(LwContainerListener l) {
    if (container != null) container.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addMouseListener(LwMouseListener l) {
    if (mouse == null) mouse = new LwMouseSupport();
    mouse.addListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeMouseListener(LwMouseListener l) {
    if (mouse != null) mouse.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addMouseMotionListener(LwMouseMotionListener l) {
    if (mouseMotion == null) mouseMotion = new LwMouseMotionSupport();
    mouseMotion.addListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeMouseMotionListener(LwMouseMotionListener l) {
    if (mouseMotion != null) mouseMotion.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addFocusListener(LwFocusListener l) {
    if (focus == null) focus = new LwFocusSupport();
    focus.addListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeFocusListener(LwFocusListener l) {
    if (focus != null) focus.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addKeyListener(LwKeyListener l) {
    if (keyboard == null) keyboard = new LwKeySupport();
    keyboard.addListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeKeyListener(LwKeyListener l) {
    if (keyboard != null) keyboard.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void removeWinListener(LwWinListener l) {
    if (window != null) window.removeListener(l);
  }

[System.Runtime.CompilerServices.MethodImpl (System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
  public   void addWinListener(LwWinListener l) {
    if (window == null) window = new LwWinSupport();
    window.addListener(l);
  }

  public void perform(LwAWTEvent e)
  {
    LwComponent target = e.getLwComponent();
    switch (e.getUID())
    {
      case LwAWTEvent.MOTION_UID:
      {
        if (mouseMotion != null) mouseMotion.perform (e);
        if (target is LwMouseMotionListener)
          LwMouseMotionSupport.process((LwMouseMotionListener)target, (LwMouseMotionEvent)e);
        return;
      }
      case LwAWTEvent.MOUSE_UID:
      {
        if (mouse != null) mouse.perform (e);
        if (target is LwMouseListener)
          LwMouseSupport.process((LwMouseListener)target, (LwMouseEvent)e);
      } break;
      case LwAWTEvent.COMP_UID:
      {
        if (component != null) component.perform (e);
        if (target is LwComponentListener)
          LwComponentSupport.process((LwComponentListener)target, (LwComponentEvent)e);
      } break;
      case LwAWTEvent.CONT_UID:
      {
        if (container != null) container.perform (e);
        if (target is LwContainerListener)
          LwContainerSupport.process((LwContainerListener)target, (LwContainerEvent)e);
      } break;
      case LwAWTEvent.KEY_UID:
      {
        if (keyboard != null) keyboard.perform (e);
        if (target is LwKeyListener)
          LwKeySupport.process((LwKeyListener)target, (LwKeyEvent)e);
      } break;
      case LwAWTEvent.FOCUS_UID:
      {
        if (focus != null) focus.perform (e);
        if (target is LwFocusListener)
          LwFocusSupport.process((LwFocusListener)target, (LwFocusEvent)e);
      } break;
      case LwAWTEvent.WIN_UID:
      {
        if (window != null) window.perform (e);
        if (target is LwWinListener)
          LwWinSupport.process((LwWinListener)target, (LwWinEvent)e);
      } break;
    }

    for (target = target.getLwParent();target != null;target = target.getLwParent())
    {
      if (target is LwChildrenListener)
        ((LwChildrenListener)target).childPerformed(e);
    }
  }

  public virtual LwComponent getEventDestination(LwComponent target) {
    LwComponent composite = findComposite(target);
    return composite == null?target:composite;
  }

  private static LwComponent findComposite (LwComponent target) {
    return findComposite (target, target);
  }

  private static LwComponent findComposite (LwComponent target, LwComponent child)
  {
    if (target == null) return null;
    LwComponent parent = target.getLwParent();

    bool b = (parent is LwComposite);
    LwComponent res = findComposite (parent, b?parent:child);
    if (res != null) return res;

    if (b && ((LwComposite)parent).catchInput(child)) return parent;
    return null;
  }
}


}
