/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.events {


using org.zaval.lw;
using org.zaval.port.cs;


public class LwMouseEvent:LwAWTEvent
{
  public const  int MOUSE_CLICKED = 1;

  public const  int MOUSE_PRESSED  = 2;

  public const  int MOUSE_RELEASED = 4;

  public const  int MOUSE_ENTERED  = 8;

  public const  int MOUSE_EXITED   = 16;

  public int x, y, ax, ay, mask, clickCount;

  public LwMouseEvent(LwComponent target, int id, int ax, int ay, int mask, int clickCount):base(target, id) {
   
    reset(target, id, ax, ay, mask, clickCount);
  }

  public virtual void reset(LwComponent target, int id, int ax, int ay, int mask, int clickCount)
  {
    source = target;
    this.id = id;
    this.ax = ax;
    this.ay = ay;
    this.mask = mask;
    this.clickCount = clickCount;
    Point p = LwToolkit.getRelLocation(ax, ay, target);
    x = p.x;
    y = p.y;
  }


  public int getMask(){
    return mask;
  }

  public int getAbsX() {
    return ax;
  }

  public int getAbsY() {
    return ay;
  }

  public int getX() {
    return x;
  }

  public int getY() {
    return y;
  }

  public override int getUID() {
    return LwAWTEvent.MOUSE_UID;
  }

  public int getClickCount() {
    return clickCount;
  }
}


}
