/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.grid {


using org.zaval.data.events;
using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.lw;
using org.zaval.misc.events;
using org.zaval.misc;
using org.zaval.port.cs;
using org.zaval.util;



public class LwGrid:LwPanel,MatrixListener, ScrollObj, LwMouseListener, LwFocusListener, LwKeyListener, PosInfo, PosListener, LwLayout, LwGridMetrics, LwChildrenListener
{
    public static readonly  int TOP_CAPTION_EL  = 1;

    public static readonly  int EDITOR_EL       = 2;

    public static readonly  int LEFT_CAPTION_EL = 3;

    public const  short USE_PSMETRIC = 32;

    public const  short DRAW_HLINES = 64;

    public const  short DRAW_VLINES = 128;

    public const  short ENABLE_COLRESIZE = 256;

    public const  int DEF_COLWIDTH  = 80;

    public const  int DEF_ROWHEIGHT = 20;

    private const  short METRIC_VALID = 512;
    private const  short ROWVIS_VALID = 1024;
    private const  short COLVIS_VALID = 2048;

    private Dimension          psSize;
    private MatrixModel        data;
    public int[]            colWidths, rowHeights;
    public int              dx, dy;

    private int                editingRow = -1, editingCol = -1, netSize = 1;
    private Insets             cellInsets = new Insets (2, 2, 2, 2);
    private LwGridViewProvider provider;
    private Color              netColor = Color.gray, noneActSelColor = LwToolkit.darkBlue, actSelColor = Color.yellow;

    private CellsVisibility    visibility = new CellsVisibility();
    private ScrollMan          man;
    private PosController      controller;
    private LwComponent        topCaption, editor;
    private LwEditorProvider   editors;

    public LwGrid():this(new Matrix(5, 5)) {
     
    }

    public LwGrid(MatrixModel data)
    {
      setModel(data);
      setViewProvider(new LwDefViews());
      setPosController (new PosController());
      enableColResize(true);
      setNetMask ((short)(DRAW_HLINES | DRAW_VLINES));
    }

    public override bool canHaveFocus() {
      return true;
    }


    public virtual void setEditorProvider (LwEditorProvider p)
    {
      if (p != editors) {
        stopEditing(true);
        editors = p;
      }
    }

    public short getNetMask () {
      return (short)(bits & (DRAW_HLINES | DRAW_VLINES));
    }

    public void setNetMask (short mask)
    {
      if (mask != getNetMask())
      {
        bits = MathBox.getBits(bits, DRAW_HLINES, (mask & DRAW_HLINES) > 0);
        bits = MathBox.getBits(bits, DRAW_VLINES, (mask & DRAW_VLINES) > 0);
        repaint();
      }
    }

    public void enableColResize (bool b) {
      if (MathBox.checkBit(bits, ENABLE_COLRESIZE) != b)
        bits = MathBox.getBits(bits, ENABLE_COLRESIZE, b);
    }

    public void usePsMetric (bool b)
    {
      if (isUsePsMetric() != b)
      {
        bits = MathBox.getBits(bits, USE_PSMETRIC, b);
        if (b) enableColResize(false);
        iMetric();
        repaint();
      }
    }

    public bool isUsePsMetric() {
      return MathBox.checkBit(bits, USE_PSMETRIC);
    }

    public PosController getPosController() {
      return controller;
    }

    public void setPosController(PosController p)
    {
      if (controller != p)
      {
        if (controller != null) controller.removePosListener(this);
        controller = p;
        if (controller != null)
        {
          controller.addPosListener(this);
          controller.setPosInfo(this);
        }
        repaint();
      }
    }

    public MatrixModel getModel () {
      return data;
    }

    public virtual LwGridViewProvider getViewProvider() {
      return provider;
    }

    public virtual void setViewProvider(LwGridViewProvider p)
    {
      if (provider != p)
      {
        provider = p;
        iMetric();
        repaint();
      }
    }

    public virtual void setModel (MatrixModel d)
    {
      if (d != data)
      {
        if (data != null) data.removeMatrixListener(this);
        data = d;
        if (data != null) data.addMatrixListener(this);
        iMetric();
        repaint();
      }
    }

    public void setSelectColor (Color c, bool hasFocusVal)
    {
       Color old = getSelectColor(hasFocusVal);
       if (!c.Equals(old))
       {
         if (hasFocusVal)  actSelColor     = c;
         else              noneActSelColor = c;
         if (hasFocus() && hasFocusVal) repaint();
       }
    }

    public Color getSelectColor (bool hasFocus) {
      return hasFocus?actSelColor:noneActSelColor;
    }

    public Color getNetColor () {
      return netColor;
    }

    public Insets getCellInsets () {
      return new Insets (cellInsets.top, cellInsets.left, cellInsets.bottom, cellInsets.right);
    }

    public void setCellInsets (int t, int l, int b, int r)
    {
      Insets i = new Insets ((t<0)?cellInsets.top:t, (l<0)?cellInsets.left:l,
                             (b<0)?cellInsets.bottom:b, (r<0)?cellInsets.right:r);
      if (!i.Equals(cellInsets))
      {
        cellInsets = i;
        iMetric();
        repaint();
      }
    }

    public void setNetColor (Color c)
    {
      if (!c.Equals(netColor))
      {
        netColor = c;
        repaint();
      }
    }

    public void matrixResized(MatrixEvent e)
    {
      iMetric();
      if (controller != null && controller.getCurrentLine() >= getGridRows())
        controller.clearPos();
    }

    public void cellModified (MatrixEvent e) {
      if (isUsePsMetric()) iMetric();
    }

    public override void paint(Graphics g)
    {
      vVisibility();
      if (visibility.hasVisibleCells())
      {
        paintData  (g);
        paintNet (g);
      }
    }

    public override void paintOnTop(Graphics g) {
      vVisibility();
      if (visibility.hasVisibleCells()) paintMarker(g);
    }

    public override void validate () {
      vMetric();
      base.validate();
    }

    public override void invalidate()
    {
      base.invalidate();
      iColVisibility();
      iRowVisibility();
    }

    public virtual void setRowHeight (int row, int h)
    {
      if (!isUsePsMetric())
      {
        vMetric();
        if (rowHeight(row) != h)
        {
          stopEditing(false);
          int dy = h - rowHeight(row);
          rowHeights[row] = h;
          psSize.height += dy;
          updateCashedPs(-1, psSize.height);
          if (man != null) man.scrollObjResized (psSize.width, psSize.height);
          vrp();
        }
      }
    }

    public virtual void setColWidth (int col, int w)
    {
      if (!isUsePsMetric())
      {
        vMetric();
        if (colWidth(col) != w)
        {
          stopEditing(false);
          int dx = w - colWidth(col);
          colWidths[col] = w;
          psSize.width += dx;
          updateCashedPs(psSize.width, -1);
          if (man != null) man.scrollObjResized (psSize.width, psSize.height);
          vrp();
        }
      }
    }

    public override Point getOrigin () {
      return new Point (dx, dy);
    }

    public virtual Point getSOLocation() {
      return getOrigin ();
    }

    public virtual void setSOLocation(int x, int y)
    {
      if (x != dx || y != dy)
      {
        int offx = x - dx, offy = y - dy;
        dx = x;
        dy = y;
        if (offx != 0) iColVisibility();
        if (offy != 0) iRowVisibility();
        rVisibility(offx, offy);
        stopEditing(false);
        repaint();
      }
    }

    public virtual Dimension getSOSize() {
      return getPreferredSize();
    }

    public bool moveContent() {
      return true;
    }

    public void setScrollMan (ScrollMan m) {
      man = m;
    }

    public virtual void mouseClicked (LwMouseEvent e) {}
    public virtual void mouseEntered (LwMouseEvent e) {}
    public virtual void mouseExited  (LwMouseEvent e) {}
    public virtual void mouseReleased(LwMouseEvent e) {}

    public virtual void mousePressed (LwMouseEvent e)
    {
      if (visibility.hasVisibleCells())
      {
        stopEditing(true);
        if (LwToolkit.isActionMask(e.getMask()))
        {
          Point p = cellByLocation(e.getX(), e.getY());
          if (p != null)
          {
            if (controller != null)
            {
              int off = controller.getCurrentLine();
              if (off == p.x) calcOrigin(off, getRowY(off));
              else controller.setOffset (p.x);
            }

            if (editors != null && editors.shouldStartEdit(p.x, p.y, e))
              startEditing(p.x, p.y);
          }
        }
      }
    }

    public int getLines() {
      return getGridRows();
    }

    public int getLineSize(int line) {
      return 1;
    }

    public int getMaxOffset() {
      return getGridRows()-1;
    }

    public virtual void posChanged(PosEvent e)
    {
      int off = controller.getCurrentLine();

      if (off >= 0)
      {
        int y = getRowY(off);
        Point o = calcOrigin(off, y);
        int poff = e.getPrevOffset();
        if (poff >= 0)
        {
          int yy = getRowY(poff);
          repaint (0, System.Math.Min(yy, y) + dy, width, System.Math.Abs(yy - y) + rowHeight(System.Math.Max(off, poff)));
        }
        else repaint ();
      }
    }

    public virtual void keyPressed(LwKeyEvent e)
    {
      if (controller != null)
      {
        bool ctrl = (e.getMask() & LwConstants.CTRL_MASK) > 0;
        switch (e.getKeyCode())
        {
          case LwConstants.VK_LEFT     : controller.seek(-1); break;
          case LwConstants.VK_UP       : controller.seekLineTo(PosController.UP); break;
          case LwConstants.VK_RIGHT    : controller.seek(1); break;
          case LwConstants.VK_DOWN     : controller.seekLineTo(PosController.DOWN); break;
          case LwConstants.VK_PAGE_UP  : controller.seekLineTo(PosController.UP,   pageSize(-1)); break;
          case LwConstants.VK_PAGE_DOWN: controller.seekLineTo(PosController.DOWN, pageSize(1)); break;
          case LwConstants.VK_END      : if (ctrl) controller.setOffset(getLines()-1); break;
          case LwConstants.VK_HOME     : if (ctrl) controller.setOffset(0); break;
        }
      }
    }

    public virtual void keyReleased(LwKeyEvent e) {}
    public virtual void keyTyped   (LwKeyEvent e) {}
    public virtual void focusGained(LwFocusEvent e) {}
    public virtual void focusLost  (LwFocusEvent e) {}

    public virtual void layout(LayoutContainer target)
    {
      if (topCaption != null && topCaption.isVisible())
      {
        Insets i = getInsets();
        topCaption.setLocation (i.left, i.top);
        Dimension d = topCaption.getPreferredSize();
        topCaption.setSize(System.Math.Min(target.getWidth() - i.left - i.right, psSize.width), d.height);
      }

      if (editors != null && editor != null && editor.isVisible())
      {
        int w = getColWidth(editingCol), h = getRowHeight(editingRow);
        int x = getColX(editingCol), y = getRowY(editingRow);
        if (isUsePsMetric())
        {
          x += cellInsets.left;
          y += cellInsets.top;
          w -= (cellInsets.left + cellInsets.right);
          h -= (cellInsets.top  + cellInsets.bottom);
        }
        editor.setLocation(x + dx, y + dy);
        editor.setSize(w, h);
        LwFocusManager.manager.requestFocus((LwComponent)editor);
      }
    }

    public virtual void componentAdded(object id, Layoutable lw, int index)
    {
      if (id != null)
      {
        if (id.Equals(TOP_CAPTION_EL)) topCaption = (LwComponent)lw;
        else
        if (id.Equals(EDITOR_EL)) editor = (LwComponent)lw;
      }
    }

    public virtual void componentRemoved(Layoutable lw, int index)
    {
      if (lw == editor) editor = null;
      else
      if (lw == topCaption) topCaption = null;
    }

    public virtual Dimension calcPreferredSize(LayoutContainer target) {
      return new Dimension (psSize.width, psSize.height + getTopCaptionHeight());
    }

    public virtual int getGridRows() {
      return data.getRows();
    }

    public virtual int getGridCols() {
      return data.getCols();
    }

    public virtual int getRowHeight(int row) {
      vMetric();
      return rowHeight(row);
    }

    public virtual int getColWidth (int col) {
      vMetric();
      return colWidth(col);
    }

    public CellsVisibility getCellsVisibility() {
      vVisibility();
      return visibility;
    }

    public int getNetGap() {
      return netSize;
    }

    public virtual int getColX (int col)
    {
      vVisibility();
      int start = 0, d = 1;
      int x = getInsets().left + netSize;
      if (visibility.hasVisibleCells())
      {
        start = visibility.fc.x;
        x     = visibility.fc.y;
        d     = (col > visibility.fc.x)?1:-1;
      }
      for (int i=start; i != col; x += ((colWidth(i) + netSize)*d), i+=d);
      return x;
    }

    public virtual int getRowY (int row)
    {
      vVisibility();
      int start = 0, d = 1;
      int y = getInsets().top + getTopCaptionHeight() + netSize;
      if (visibility.hasVisibleCells())
      {
        start = visibility.fr.x;
        y     = visibility.fr.y;
        d     = (row > visibility.fr.x)?1:-1;
      }
      for (int i=start; i != row; y += ((rowHeight(i) + netSize)*d), i+=d);
      return y;
    }

    public void childPerformed(LwAWTEvent e) {
      if (e.getID() == LwKeyEvent.KEY_PRESSED &&
          ((LwKeyEvent)e).getKeyCode() == LwConstants.VK_ESCAPE) stopEditing(false);
    }

    public virtual void startEditing (int row, int col)
    {
      stopEditing (true);
      if (editors != null)
      {
        LwComponent editor = editors.getEditor(row, col, getDataToEdit(row, col));
        if (editor != null)
        {
          editingRow = row;
          editingCol = col;
          add(EDITOR_EL, editor);
          repaint();
        }
      }
    }

    public virtual void stopEditing (bool applyData)
    {
      if (editors != null && editingRow >= 0 && editingCol >= 0)
      {
        try {
          object v = editors.fetchEditedValue (editingRow, editingCol, (LwComponent)editor);
          if (applyData) setEditedData(editingRow, editingCol, v);
        }
        finally
        {
          editingRow = -1;
          editingCol = -1;
          int index = indexOf((LwComponent)editor);
          if (index >= 0) remove(index);
          repaint();
        }
      }
    }

    public LwComponent getTopCaption() {
      return topCaption;
    }

    public virtual int colWidth (int col) {
      return colWidths[col];
    }

    public virtual int rowHeight (int row) {
      return rowHeights[row];
    }

    public virtual object getDataToEdit (int row, int col) {
      return data.get(row, col);
    }

    public virtual void setEditedData (int row, int col, object value) {
      data.put (row, col, value);
    }

    public virtual bool isMetricValid() {
      return MathBox.checkBit(bits, METRIC_VALID);
    }

    public virtual object dataToPaint (int row, int col) {
      return data.get(row, col);
    }

    public void vMetric()
    {
      if (!isMetricValid())
      {
        if (isUsePsMetric()) rPsMetric();
        else                 rCustomMetric();
        psSize = rPs();
        bits = MathBox.getBits(bits, METRIC_VALID, true);
      }
    }

    public void vVisibility() {
      vMetric();
      if (getVisiblePart() != null) rVisibility(0, 0);
    }

    public Dimension rPs()
    {
      int cols = getGridCols();
      int rows = getGridRows();
      int psWidth  = netSize * (cols + 1);
      int psHeight = netSize * (rows + 1);
      for (int i=0; i<cols; i++) psWidth  += colWidth(i);
      for (int i=0; i<rows; i++) psHeight += rowHeight(i);
      return new Dimension (psWidth, psHeight);
    }

    public virtual void paintNet (Graphics g)
    {
      int topX = visibility.fc.y - netSize;
      int topY = visibility.fr.y - netSize;
      int botX = visibility.lc.y + colWidth (visibility.lc.x);
      int botY = visibility.lr.y + rowHeight(visibility.lr.x);

      g.setColor(netColor);
      if (MathBox.checkBit(bits, DRAW_HLINES))
      {
        int y = topY, i = visibility.fr.x;
        for (;i <= visibility.lr.x; i++)
        {
          g.drawLine(topX, y, botX, y);
          y += rowHeight(i) + netSize;
        }
        g.drawLine(topX, y, botX, y);
      }

      if (MathBox.checkBit(bits, DRAW_VLINES))
      {
        int x = topX, i = visibility.fc.x;
        for (;i <= visibility.lc.x; i++)
        {
          g.drawLine(x, topY, x, botY);
          x += colWidth(i) + netSize;
        }
        g.drawLine(x, topY, x, botY);
      }
    }

    public virtual void paintData (Graphics g)
    {
      int y    = visibility.fr.y + cellInsets.top;
      int addW = cellInsets.left + cellInsets.right;
      int addH = cellInsets.top  + cellInsets.bottom;
      Rectangle r = g.getClipBounds();

      for (int i=visibility.fr.x; i<=visibility.lr.x && y < r.y + r.height; i++)
      {
        if (y + rowHeight(i) > r.y)
        {
          int x = visibility.fc.y + cellInsets.left;
          for (int j=visibility.fc.x; j<=visibility.lc.x; j++)
          {
             LwView v = provider.getView(i, j, dataToPaint(i, j));
             if (v != null && !(i == editingRow && j == editingCol))
             {
               Color bg = provider.getCellColor(i, j);
               if (bg != null)
               {
                 g.setColor(bg);
                 g.fillRect(x - cellInsets.left, y - cellInsets.top, colWidth(j), rowHeight(i));
               }

               int w = colWidth (j) - addW;
               int h = rowHeight(i) - addH;
               g.clipRect(x, y, w, h);
               if (isUsePsMetric() || v.getType() == LwView.STRETCH) v.paint(g, x, y, w, h, this);
               else
               {
                 Dimension ps = v.getPreferredSize();
                 Point p = Alignment.getLocation (ps,
                                                  provider.getXAlignment(i, j),
                                                  provider.getYAlignment(i, j),
                                                  w, h);
                 v.paint(g, x + p.x, y + p.y, ps.width, ps.height, this);
               }
               g.setClip (r);
             }
             x += (colWidth(j) + netSize);
          }
        }
        y += (rowHeight(i) + netSize);
      }
    }

    public virtual void paintMarker (Graphics g)
    {
      if (controller != null && controller.getOffset() >= 0)
      {
        //???
        g.clipRect(-dx, getTopCaptionHeight() - dy, width, height);

        int offset = controller.getOffset();
        int y = getRowY(offset), h = rowHeight(offset);
        int x = visibility.fc.y;

        for (int i=visibility.fc.x; i<=visibility.lc.x; i++)
        {
          if (i != editingCol || editingRow != offset)
          {
            Color bg = provider.getCellColor(offset, i);
            if (bg == null) bg = getBackground();
            LwToolkit.drawMarker(g, x, y, colWidth(i), h, bg, hasFocus()?actSelColor:noneActSelColor);
          }
          x += colWidth(i) + netSize;
        }
      }
    }

    public override LwLayout getDefaultLayout() {
      return this;
    }

    public virtual Point cellByLocation(int x, int y)
    {
      vVisibility();
      int ry1 = visibility.fr.y + dy;
      int ry2 = visibility.lr.y + rowHeight(visibility.lr.x) + dy;
      int rx1 = visibility.fc.y + dx;
      int rx2 = visibility.lc.y + colWidth(visibility.lc.x) + dx;

      int row = -1, col = -1;

      if (y > ry1 && y < ry2)
      {
         for (int i = visibility.fr.x; i<=visibility.lr.x; ry1 += rowHeight(i) + netSize, i++)
           if (y > ry1 && y < ry1 + rowHeight(i))
           {
             row = i;
             break;
           }
      }

      if (x > rx1 && x < rx2)
      {
         for (int i = visibility.fc.x; i<=visibility.lc.x; rx1 += colWidth(i) + netSize, i++)
           if (x > rx1 && x < rx1 + colWidth(i))
           {
             col = i;
             break;
           }
      }
      return (col >= 0 && row >=0)?new Point(row, col):null;
    }

    public virtual void rPsMetric()
    {
      colWidths  = new int[getGridCols()];
      rowHeights = new int[getGridRows()];

      int addW = cellInsets.left + cellInsets.right;
      int addH = cellInsets.top  + cellInsets.bottom;
      for (int i=0; i<colWidths.Length ;i++)
      {
        for (int j=0; j<rowHeights.Length; j++)
        {
          LwView v = provider.getView(j, i, data.get(j, i));
          if (v != null)
          {
            Dimension ps = v.getPreferredSize();
            ps.width  += addW;
            ps.height += addH;
            if (ps.width  > colWidths[i] ) colWidths [i] = ps.width;
            if (ps.height > rowHeights[j]) rowHeights[j] = ps.height;
          }
          else
          {
            if (DEF_COLWIDTH  > colWidths [i]) colWidths [i] = DEF_COLWIDTH;
            if (DEF_ROWHEIGHT > rowHeights[j]) rowHeights[j] = DEF_ROWHEIGHT;
          }
        }
      }
    }

    public virtual void rCustomMetric()
    {
      int start = 0;
      if (colWidths != null)
      {
        start = colWidths.Length;
        if (colWidths.Length != getGridCols())
        {
          int[] na = new int[getGridCols()];
          System.Array.Copy(colWidths, 0, na, 0, System.Math.Min(colWidths.Length, na.Length));
          colWidths = na;
        }
      }
      else colWidths = new int[getGridCols()];
      for (; start<colWidths.Length; start++) colWidths[start] = DEF_COLWIDTH;

      start = 0;
      if (rowHeights != null)
      {
        start = rowHeights.Length;
        if (rowHeights.Length != getGridRows())
        {
          int[] na = new int[getGridRows()];
          System.Array.Copy(rowHeights, 0, na, 0, System.Math.Min(rowHeights.Length, na.Length));
          rowHeights = na;
        }
      }
      else rowHeights = new int[getGridRows()];
      for (;start<rowHeights.Length; start++) rowHeights[start] = DEF_ROWHEIGHT;
    }

    public void iMetric()  {
      bits = MathBox.getBits(bits, METRIC_VALID, false);
      invalidate();
    }

    public virtual int getTopCaptionHeight() {
      return (topCaption != null && topCaption.isVisible())?topCaption.getPreferredSize().height:0;
      //return (topCaption != null && topCaption.isVisible())?topCaption.getHeight():0;
    }

    public virtual int pageSize(int d)
    {
      int off = controller.getOffset();
      if (off >= 0)
      {
         Rectangle visibleArea = getVisiblePart();
         visibleArea.height -= getTopCaptionHeight();

         int sum = 0, poff = off;
         for (;off >=0 && off < getGridRows() && sum < visibleArea.height; sum += rowHeight(off) + netSize, off+=d);
         return System.Math.Abs(poff - off);
      }
      return 0;
    }

    public virtual void iColVisibility() {
      bits = MathBox.getBits(bits, COLVIS_VALID, false);
    }

    public virtual void iRowVisibility() {
      bits = MathBox.getBits(bits, ROWVIS_VALID, false);
    }

    private Point colVisibility(Point p, int d, bool b)
    {
      Rectangle visibleArea = getVisiblePart();
      int       cols        = getGridCols();
      if (cols == 0 || p == null) return null;

      int x   = p.y;
      int col = p.x;
      for (;col < cols && col >=0; col+=d)
      {
        if (x + dx < (visibleArea.x + visibleArea.width) && (x + colWidth(col) + dx) > visibleArea.x)
        {
          if (b) return new Point (col, x);
        }
        else if (!b) return colVisibility(new Point(col, x), (d > 0?-1:1), true);

        if (d < 0) {
          if (col > 0) x -= (colWidth(col - 1) + netSize);
        }
        else if (col < cols - 1) x += (colWidth(col) + netSize);
      }

      if (!b)
        return (d > 0)?new Point(col - 1, x):
                       new Point(0, getInsets().left + netSize);
      return null;
    }

    private Point rowVisibility(Point p, int d, bool b)
    {
      Rectangle visibleArea = getVisiblePart();
      int       rows        = getGridRows();
      if (rows == 0 || p == null) return null;

      int y   = p.y;
      int row = p.x;
      for (;row < rows && row >= 0; row+=d)
      {
        if (y + dy < (visibleArea.y + visibleArea.height) && (y + rowHeight(row) + dy) > visibleArea.y)
        {
          if (b) return new Point (row, y);
        }
        else if (!b) return rowVisibility(new Point(row, y), (d > 0?-1:1), true);

        if (d < 0)  {
          if (row > 0) y -= (rowHeight(row - 1) + netSize);
        }
        else if (row < rows - 1) y += (rowHeight(row) + netSize);
      }

      if (!b)
        return (d > 0)?new Point(row - 1, y):
                       new Point(0, getInsets().top + getTopCaptionHeight() + netSize);
      return null;
    }

    private void rVisibility (int offx, int offy)
    {
      bool b = visibility.hasVisibleCells();
      if (!MathBox.checkBit(bits, COLVIS_VALID))
      {
        if (offx > 0 && b)
        {
          visibility.lc = colVisibility(visibility.lc, -1, true);
          visibility.fc = colVisibility(visibility.lc,-1, false);
        }
        else
        if (offx < 0 && b)
        {
          visibility.fc = colVisibility(visibility.fc, 1, true);
          visibility.lc = colVisibility(visibility.fc, 1, false);
        }
        else
        {
          visibility.fc = new Point(0, getInsets().left + netSize);
          visibility.fc = colVisibility(visibility.fc, 1, true);
          visibility.lc = colVisibility(visibility.fc, 1, false);
        }
        bits = MathBox.getBits(bits, COLVIS_VALID, true);
      }

      if (!MathBox.checkBit(bits, ROWVIS_VALID))
      {
        if (offy > 0 && b)
        {
          visibility.lr = rowVisibility(visibility.lr,  -1, true);
          visibility.fr = rowVisibility(visibility.lr , -1, false);
        }
        else
        if (offy < 0 && b)
        {
          visibility.fr = rowVisibility(visibility.fr, 1, true);
          visibility.lr = rowVisibility(visibility.fr, 1, false);
        }
        else
        {
          visibility.fr = new Point(0, getInsets().top + getTopCaptionHeight() + netSize);
          visibility.fr = rowVisibility(visibility.fr, 1, true);
          visibility.lr = rowVisibility(visibility.fr, 1, false);
        }
        bits = MathBox.getBits(bits, ROWVIS_VALID, true);
      }
    }

    private Point calcOrigin(int off, int y)
    {
      Insets i = getInsets();
      i.top += getTopCaptionHeight();
      Point o = LwToolkit.calcOrigin(getColX(0) - netSize, y - netSize, psSize.width, rowHeight(off) + 2*netSize, dx, dy, this, i);
      if (man != null) man.scrollObjMoved(o.x, o.y);
      else             setSOLocation(o.x, o.y);
      return o;
    }
 }



}
