/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.grid {


using org.zaval.data.events;
using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.lw.tree;
using org.zaval.lw;
using org.zaval.misc.events;
using org.zaval.misc;
using org.zaval.port.cs;
using org.zaval.util;



public class LwTreeGrid:LwGrid,LwGridViewProvider, MatrixModel, TreeListener, LwActionListener, LwComposite
{
  private LwTree              tree;
  private LwGridViewProvider  provider;
  private Hashtable           data;
  private int                 gridCols = 1;
  private Vector              itemByRow = new Vector(), listeners;
  private int                 extraRowHeight = 2;

  public LwTreeGrid ():this(new Tree(new Item ("root"))) {
   
  }

  public LwTreeGrid (TreeModel model)
  {
    tree = new LwTree(model);
    tree.enableSelection (false);
    tree.showLines (false);
    tree.addActionListener(this);
    tree.setOpaque(false);
    add (tree);
    setCellInsets(0,2,0,2);
    base.setViewProvider(this);
    setViewProvider(new LwDefViews());
    base.setModel (this);
  }

  public override void setModel(MatrixModel m) {
   // throw new org.zaval.port.cs.RuntimeException();
  }

  public bool catchInput(LwComponent child)  {
    return LwToolkit.isAncestorOf(tree, child);
  }

  public void setExtraRowHeight (int h)
  {
    if (extraRowHeight != h)
    {
      extraRowHeight = h;
      iMetric();
      repaint();
    }
  }

  public override LwGridViewProvider getViewProvider() {
    return provider;
  }

  public virtual void setTreeViewProvider(LwViewProvider p)
  {
    if (tree.getViewProvider() != p)
    {
      tree.setViewProvider(p);
      iMetric();
    }
  }

  public void setTreeElView(int id, LwView v) {
    tree.setView(id, v);
    iMetric();
  }

  public override void setViewProvider(LwGridViewProvider p)
  {
    if (provider != p)
    {
      provider = p;
      iMetric();
      repaint();
    }
  }

  public override void mousePressed (LwMouseEvent e)
  {
    if (e.getX() >= tree.getX() && e.getX() < tree.getX() + tree.getWidth())
    {
      LwMouseEvent ee = new LwMouseEvent (tree, e.getID(), e.getAbsX(), e.getAbsY(), e.getMask(), 1);
      tree.mousePressed(ee);
    }
    base.mousePressed (e);
  }

  public override void keyPressed (LwKeyEvent e)
  {
    PosController controller = getPosController();
    if (controller != null && controller.getCurrentLine() >=0)
    {
      char ch = e.getKeyChar();
      if (ch == '+')
      {
        Item item = getItemByRow(controller.getCurrentLine(), true);
        if (!tree.isOpen(item)) tree.toggle(item);
        return;
      }
      else
      if (ch == '-')
      {
        Item item = getItemByRow(controller.getCurrentLine(), true);
        if (tree.isOpen(item)) tree.toggle(item);
        return;
      }
    }
    base.keyPressed(e);
  }

  public override void layout(LayoutContainer target)
  {
    base.layout(target);
    if (tree != null && tree.isVisible())
    {
      Insets    ins = getInsets();
      Insets    ci  = getCellInsets();
      Dimension ps  = tree.getPreferredSize();
      int       gap = getNetGap();
      Point     off = getOrigin();
      tree.setSize(getColWidth(0) - ci.left - ci.right, ps.height);
      tree.setLocation (ins.left + gap + ci.left + off.x, ins.top + gap + ci.top + getExtraHeight() + getTopCaptionHeight() + off.y);
    }
  }

  public LwView getView(int row, int col, object o)
  {
    if (col > 0)
    {
      Item item = getItemByRow(row, true);
      object[] a = (object[])data.get(item);
      o = (a == null || a.Length < col ?null:a[col - 1]);
      return provider.getView(row, col, o);
    }
    return null;
  }

  public int getXAlignment(int row, int col) {
    return provider.getXAlignment(row, col);
  }

  public int getYAlignment(int row, int col) {
    return provider.getYAlignment(row, col);
  }

  public Color getCellColor (int row, int col) {
    return provider.getCellColor(row, col);
  }

  public virtual void setTreeModel(TreeModel m)
  {
    TreeModel data = tree.getModel();
    if (m != data)
    {
      if (data != null) data.removeTreeListener(this);
      data = m;
      tree.setModel(data);
      if (data != null) data.addTreeListener(this);
      if (this.data != null) this.data.clear();
      iTreeIndex();
      iMetric();
    }
  }

  public TreeModel getTreeModel() {
    return tree.getModel();
  }

  public int getRows () {
    return getTreeModel().getItemsCount();
  }

  public int getCols () {
    return gridCols;
  }

  public override int getGridRows ()  {
    vTreeIndex();
    return itemByRow.size();
  }

  public override int getGridCols () {
    return getCols();
  }

  public object get(int row, int col)
  {
    if (row >= getRows() || col >= getCols()) throw new System.IndexOutOfRangeException();

    if (col != 0 && data != null)
    {
      Item item = getItemByRow (row, false);
      object[] a = (object[])data.get(item);
      return a == null || a.Length < col ?null:a[col - 1];
    }
    return null;
  }

  public void actionPerformed (LwActionEvent e)
  {
    stopEditing(true);
    iMetric();
    iTreeIndex();
    PosController c = getPosController();
    if (c != null)
    {
      if (c.getCurrentLine() >= getGridRows()) c.clearPos();
      else
      {
        c.invalidate();
        c.validate();
      }
    }
    repaint();
  }

  public void put(int row, int col, object o) {
    if (col == 0) throw new org.zaval.port.cs.IllegalArgumentException();
    put (getItemByRow(row, false), row, col, o);
  }

  public void put(Item item, int row, int col, object o)
  {
    if (col == 0) throw new org.zaval.port.cs.IllegalArgumentException();
    if (data == null) data = new Hashtable();
    object[] cells = (object[])data.get(item);
    int      len   = (cells == null)?0:cells.Length;
    if (len < col)
    {
      object[] na = new object[col];
      if (cells != null) System.Array.Copy(cells, 0, na, 0, cells.Length);
      cells = na;
      if (cells.Length + 1 > gridCols) gridCols = cells.Length + 1;
      perform (new MatrixEvent(this, getRows(), len));
    }
    object prev = cells[col-1];
    cells[col-1] = o;
    data.put(item, cells);
    perform (new MatrixEvent(this, row, col, prev));
  }

  public void addMatrixListener (MatrixListener m) {
    if (listeners == null) listeners = new Vector();
    if (!listeners.contains(m)) listeners.addElement(m);
  }

  public void removeMatrixListener(MatrixListener l) {
    if (listeners != null) listeners.removeElement(l);
  }

  public void itemInserted(TreeEvent e)
  {
    if (tree.isOpen(getTreeModel().getRoot())) {
      iTreeIndex();
      iMetric();
    }
    perform (new MatrixEvent (this, getRows() - 1, getCols()));
  }

  public void itemRemoved (TreeEvent e)
  {
    iTreeIndex();
    iMetric();

    if (data != null)
    {
      data.remove (e.getItem ());
      perform (new MatrixEvent (this, getRows() + 1, getCols()));
    }
  }

  public void itemModified(TreeEvent e) {
    iMetric();
  }

  public override void setSOLocation (int dx, int dy)
  {
    if (dx != this.dx || dy != this.dy)
    {
      tree.invalidate();
      base.setSOLocation(dx, dy);
    }
  }

  public override void setRowHeight (int row, int h)  {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public int getModelRow(int gridRow) {
    return getDataRow(getItemByRow(gridRow, true));
  }

  public Item getSelectedItem()
  {
    PosController c = getPosController ();
    if (c != null && c.getCurrentLine() >= 0) return getItemByRow(c.getCurrentLine(), true);
    return null;
  }

  public override void startEditing (int row, int col) {
    if (col != 0) base.startEditing(row, col);
  }

  public Item getItemByRow (int row) {
    return getItemByRow(row, false);
  }

  public void perform(MatrixEvent e)
  {
    if (listeners != null)
    {
      for (int i=0; i<listeners.size(); i++)
      {
        MatrixListener l = (MatrixListener)listeners.elementAt(i);
        if (e.getID() == MatrixEvent.MATRIX_RESIZED) l.matrixResized(e);
        else l.cellModified(e);
      }
    }
  }

  public override void rCustomMetric()
  {
    Insets ins = getCellInsets();
    tree.setGaps(-1, ins.top + ins.bottom + getNetGap() + extraRowHeight*2);

    vTreeIndex();
    int start = 0;
    if (colWidths != null)
    {
      start = colWidths.Length;
      if (colWidths.Length != getGridCols())
      {
        int[] na = new int[getGridCols()];
        System.Array.Copy(colWidths, 0, na, 0, System.Math.Min(colWidths.Length, na.Length));
        colWidths = na;
      }
    }
    else colWidths = new int[getGridCols()];
    for (;start<colWidths.Length; start++) colWidths[start] = DEF_COLWIDTH;
  }

  public override void rPsMetric()
  {
    Insets ins = getCellInsets();
    tree.setGaps(-1, ins.top + ins.bottom + getNetGap());
    vTreeIndex();
    colWidths = new int[getGridCols()];
    Insets cellInsets = getCellInsets();
    int addW = cellInsets.left + cellInsets.right;
    for (int i=1; i<colWidths.Length ;i++)
    {
      for (int j=0; j<getGridRows(); j++)
      {
        LwView v = getView(j, i, dataToPaint(j, i));
        if (v != null)
        {
          Dimension ps = v.getPreferredSize();
          ps.width  += addW;
          if (ps.width  > colWidths[i] ) colWidths [i] = ps.width;
        }
        else if (DEF_COLWIDTH  > colWidths [i]) colWidths [i] = DEF_COLWIDTH;
      }
    }
    colWidths[0] = tree.getPreferredSize().width + addW;
  }

  public override object dataToPaint (int row, int col)
  {
    if (col > 0)
    {
      Item i = getItemByRow(row, true);
      object[] a = (object[])data.get(i);
      return a == null || a.Length < col ?null:a[col - 1];
    }
    return null;
  }

  public override object getDataToEdit (int row, int col) {
    return getModel().get(getDataRow(getItemByRow(row, true)), col);
  }

  public override void setEditedData (int row, int col, object value) {
    getModel().put (getDataRow(getItemByRow(row, true)), col, value);
  }

  public override int rowHeight(int r) {
    Insets cellInsets = getCellInsets();
    return tree.getItemMetrics((Item)itemByRow.elementAt(r)).getHeight() + cellInsets.top + cellInsets.bottom + 2*getExtraHeight();
  }

  private Item getItemByRow (int row, bool onlyVisible) {
    vTreeIndex();
    return onlyVisible?(Item)itemByRow.elementAt(row):
                        getItemByRow(getTreeModel().getRoot(), row, new int[] { 0 } );
  }

  private Item getItemByRow (Item root, int row, int[] c)
  {
    if (row == c[0]) return root;
    c[0]++;
    TreeModel data = getTreeModel();
    for (int i=0; i<data.getChildrenCount(root); i++)
    {
      Item res = getItemByRow(data.getChildAt(root, i), row, c);
      if (res != null) return res;
    }
    return null;
  }

   private int getDataRow (Item item) {
     int[] c = new int[1];
     return getDataRow(getTreeModel().getRoot(), item, c)?c[0]:-1;
   }

   private bool getDataRow (Item root, Item item, int[] c)
   {
     if (root == item) return true;
     c[0]++;
     TreeModel data = getTreeModel ();
     int size = data.getChildrenCount(root);
     for (int i=0; i<size; i++) if (getDataRow(data.getChildAt(root, i), item, c)) return true;
     return false;
   }

  private void iTreeIndex() {
    itemByRow.removeAllElements();
  }

  private void vTreeIndex() {
    if (itemByRow.size() == 0) rTreeIndex(getTreeModel().getRoot());
  }

  private void rTreeIndex(Item root)
  {
    itemByRow.addElement(root);
    if (tree.isOpen(root))
    {
      TreeModel data = getTreeModel ();
      int       size = data.getChildrenCount(root);
      for (int i=0; i<size; i++)
        rTreeIndex(data.getChildAt(root, i));
    }
  }

  private int getExtraHeight() {
    return isUsePsMetric()?0:extraRowHeight;
  }
}



}
