/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.mask {


using org.zaval.port.cs;

public class BasicMaskValidator:MaskValidator
{
  public const  int LETTER_TYPE  = 1;

  public const  int NUMERIC_TYPE = 2;

  private static char LETTER_TAG   = 'a';
  private static char CLETTER_TAG  = 'A';
  private static char NUMERIC_TAG  = 'n';
  private static char HEX_TAG      = 'x';
  private static char SIGN_TAG     = '#';

  public virtual bool isHandledTag (char tag) {
    return tag == LETTER_TAG || tag == CLETTER_TAG || tag == NUMERIC_TAG || tag == SIGN_TAG || tag == HEX_TAG;
  }

  public virtual char getBlankChar(char tag)
  {
    if (tag == LETTER_TAG || tag == CLETTER_TAG) return '_';
    else
    if (tag == NUMERIC_TAG || tag == HEX_TAG) return '0';
    else
    if (tag == SIGN_TAG) return '+';
    return tag;
  }

  public virtual int getTypeByTag(char tag)
  {
    if (tag == LETTER_TAG || tag == CLETTER_TAG) return LETTER_TYPE;
    else
    if (tag == NUMERIC_TAG || tag == HEX_TAG || tag == SIGN_TAG) return NUMERIC_TYPE;
    return -1;
  }

  public virtual bool isValidValue (MaskElement e, string value)
  {
    switch (e.getType()) {
      case LETTER_TYPE : return isLetterType (e, value);
      case NUMERIC_TYPE: return isNumericType(e, value);
    }
    return false;
  }

  public virtual string completeValue (MaskElement e, string newValue)
  {
    char[] buf = newValue.ToCharArray();
    for (int i=0; i<buf.Length; i++)
    {
      char tag = e.getTagAt(i);
      if (tag == CLETTER_TAG || tag == HEX_TAG) buf[i] = System.Char.ToUpper(buf[i]);
      else
      if (tag == SIGN_TAG && buf[i] == getBlankChar(tag)) buf[i] = '+';
    }
    return new string (buf);
  }

  public virtual object getValueByElement (MaskElement e) {
    return e.getValue ();
  }

  private bool isLetterType(MaskElement e, string value)
  {
    char[] buf = value.ToCharArray();
    for (int i=0; i<buf.Length; i++)
    {
      char tag = e.getTagAt(i);
      if  (buf[i] != getBlankChar(tag) && !System.Char.IsLetter(buf[i])) return false;
    }
    return true;
  }

  private bool isNumericType(MaskElement e, string value)
  {
    char[] buf = value.ToCharArray();
    for (int i=0; i<buf.Length; i++)
    {
      char tag = e.getTagAt(i);
      if ((buf[i] != getBlankChar(tag)) &&
          ((tag == SIGN_TAG && (buf[i] != '+' && buf[i] != '-')||
           (tag == NUMERIC_TAG && !System.Char.IsDigit(buf[i]))||
       (tag == HEX_TAG &&  System.Uri.IsHexDigit(buf[i])))  ))


        return false;
    }
    return true;
  }
}





}
