/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.mask {


using org.zaval.port.cs;


public class DateMaskValidator:MaskValidator
{
  public const  int DATE_TYPE = 1;

  private static char DAY_TAG    = 'd';
  private static char YEAR_TAG   = 'y';
  private static char MONTH_TAG  = 'm';
  private static char LMONTH_TAG = 'M';

  private string[] months = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

  public DateMaskValidator () {
  }

  public void setShortMonths(string[] months) {
    this.months = months;
  }

  public char getBlankChar(char tagName) {
    return '_';
  }

  public int getTypeByTag(char tag) {
    return DATE_TYPE;
  }

  public bool isHandledTag(char tagName) {
    return tagName == DAY_TAG || tagName == YEAR_TAG || tagName == MONTH_TAG || tagName == LMONTH_TAG;
  }

  public bool isValidValue (MaskElement e, string value)
  {
    for (int i=0; i<value.Length; i++)
      if (!isHandledTag(e.getTagAt(i)) && value[i] != e.getTagAt(i)) return false;

    string day    = MaskElement.getValueByTag(e, 0, DAY_TAG, value);
    string month1 = MaskElement.getValueByTag(e, 0, MONTH_TAG, value);
    string month2 = MaskElement.getValueByTag(e, 0, LMONTH_TAG, value);
    string year   = MaskElement.getValueByTag(e, 0, YEAR_TAG, value);
    try
    {
      int iDay = -1, iMonth = -1, iYear = -1;
      if ((day    != null && ((iDay=System.Convert.ToInt32(day))   > 31   || iDay   <= 0))||
          (month1 != null && ((iMonth=System.Convert.ToInt32(month1))> 12   || iMonth <= 0))||
          (year   != null && ((iYear=System.Convert.ToInt32(year))  > 9999 || iYear  <= 0))||
          (month2 != null && getMonthByName(month2) <= 0))
        return false;
    }
    catch (System.FormatException) { return false; }
    return true;
  }

  public string completeValue(MaskElement e, string value)
  {
    string tags = e.getTags();
    string m1 = MaskElement.getValueByTag(e, 0, MONTH_TAG, value);
    string m2 = MaskElement.getValueByTag(e, 0, LMONTH_TAG, value);

    if (m1 != null)
    {
      char   blank = getBlankChar(MONTH_TAG);
      char[] buf   = m1.ToCharArray();
      if (buf[0] == blank) buf[0] = '0';
      if (buf[1] == blank) buf[1] = (buf[0] == '0')?'1':'0';
      m1 = new string(buf);
      value = MaskUtil.setValueByTag (MONTH_TAG, value, tags, m1);
    }

    if (m2 != null)
    {
      string s = null;
      if (m1 != null)
      {
        try {
          int im1 = System.Convert.ToInt32(MaskUtil.trim(m1, getBlankChar(MONTH_TAG)));
          if (im1 > 0 && im1 < 13) s = months[im1 - 1];
        }
        catch (System.FormatException) {}
      }
      if (s == null) s = getAppropriateMonth(m2, getBlankChar(LMONTH_TAG));
      if (s != null) value = MaskUtil.setValueByTag (LMONTH_TAG, value, tags, s);
    }

    string day = MaskElement.getValueByTag(e, 0, DAY_TAG, value);
    if (day != null)
    {
      char   blank = getBlankChar(DAY_TAG);
      char[] buf = day.ToCharArray();
      if (buf[0] == blank) buf[0] = '0';
      if (buf[1] == blank) buf[1] = (buf[0] == '0')?'1':'0';
      value = MaskUtil.setValueByTag (DAY_TAG, value, tags, new string(buf));
    }

    string year = MaskElement.getValueByTag(e, 0, YEAR_TAG, value);
    if (year != null)
    {
      char   blank = getBlankChar(YEAR_TAG);
      char[] buf   = year.ToCharArray();
      if (buf.Length == 2)
      {
        if (buf[0] == blank) buf[0] = '9';
        if (buf[1] == blank) buf[1] = '1';
      }
      else
      if (buf.Length == 4)
      {
        if (buf[0] == blank) buf[0] = '2';
        for (int i=1; i<buf.Length; i++) if (buf[i] == blank) buf[i] = '0';
      }
      value = MaskUtil.setValueByTag (YEAR_TAG, value, tags, new string(buf));
    }
    return value;
  }

  private int getMonthByName(string name) {
    for (int i=0; i<months.Length; i++) if (UtilBox.equalsIgnoreCase(name,months[i])) return i + 1;
    return -1;
  }

  private string getAppropriateMonth (string value, char blank)
  {
    char[]   buf = value.ToCharArray();
    Vector   res = new Vector();
    for (int i=0; i<months.Length; i++) res.addElement (months[i].ToCharArray());

    for (int i=0; i<buf.Length && buf[i] != blank; i++)
    {
      int size = res.size();
      Vector prev = (Vector)res.Clone();
      for (int j = size-1; j >= 0; j--)
      {
        char[] m = (char[])res.elementAt(j);
        if (System.Char.ToUpper(m[i]) != System.Char.ToUpper(buf[i]))
          res.removeElementAt(j);
      }

      if (res.size() == 0)
      {
        res = prev;
        break;
      }
    }
    return res.size()>0?new string((char[])res.elementAt(0)):null;
  }
}




}
