/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.mask {


using org.zaval.lw;
using org.zaval.port.cs;



public class LwMaskTextField:LwTextField
{
  public LwMaskTextField():this("", "") {
   

  }

  public LwMaskTextField(string text, string mask):this(new MaskedText(mask)) {
   
    setText(text);
  }

  public LwMaskTextField(MaskedText t):base(t) {
   
    setPosController (new MaskPosController(t));
  }

  public void setValidator (MaskValidator v) {
   ((MaskedText)getTextModel()).setValidator(v);
    repaint();
  }

  public MaskValidator getValidator () {
    return ((MaskedText)getTextModel()).getValidator();
  }


  public string getMask () {
    return ((MaskedText)getTextModel()).getMask();
  }

  public void setMask (string mask)
  {
    MaskedText mt = (MaskedText)getTextModel();
    string     m  = mt.getMask();
    if (m == null || !m.Equals(mask))
    {
      mt.setMask (mask);
      repaint();
    }
  }

  public override void write(int pos, string s)
  {
    MaskedText mt = (MaskedText)getTextModel();
    pos = mt.findHandledTag(pos, 1);
    getPosController().setOffset(pos);
    base.write(pos, s);
  }
}



}
