/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.tree {


using org.zaval.data.events;
using org.zaval.data;
using org.zaval.lw.events;
using org.zaval.lw;
using org.zaval.misc;
using org.zaval.port.cs;
using org.zaval.util;



public class LwTree:LwCanvas,TreeListener, LwViewProvider, LwMouseListener, LwFocusListener, LwKeyListener , ScrollObj
{
   public const  int LEAST_VIEW  = 1;

   public const  int OPENED_VIEW = 2;

   public const  int CLOSED_VIEW = 3;

   public const  int TOGGLE_OFF_VIEW = 4;

   public const  int TOGGLE_ON_VIEW  = 5;

   /* To speed up */
   private static readonly  Dimension ZERO = new Dimension();

   private TreeModel          data;
   private LwActionSupport    actions;
   private LwActionSupport    selection;
   private bool isOpenVal;

   private LwViewProvider  provider;
   private LwView          openedImg, closedImg, leastImg, toggleOnView, toggleOffView;
   private Hashtable       nodes;
   private int             gapx = 5, gapy = 2, dx, dy;
   private Item            selected, firstVisible;
   private Rectangle       visibleArea;
   private Dimension       max,openedImgSize, closedImgSize, leastImgSize, toggleOnSize, toggleOffSize;
   private ScrollMan       man;
   private LwTextRender    defaultRender = new LwTextRender("");
   private Color           selectColor1 = LwToolkit.darkBlue, selectColor2 = Color.yellow, linesColor = Color.gray;

   public LwTree(TreeModel d):this(d, true) {
    
   }

   public LwTree(TreeModel d, bool b)
   {
     isOpenVal = b;
     setModel(d);
     setViewProvider(this);
     setView(TOGGLE_ON_VIEW,  LwManager.getView("toggle.on"));
     setView(TOGGLE_OFF_VIEW, LwManager.getView("toggle.off"));
     setView(LEAST_VIEW,      LwManager.getView("tree.least"));
     setView(OPENED_VIEW,     LwManager.getView("tree.open"));
     setView(CLOSED_VIEW,     LwManager.getView("tree.close"));
     enableSelection(true);
     showLines(true);
   }

   public override bool canHaveFocus() {
     return true;
   }

   public void enableSelection(bool b)
   {
     if (isSelectionEnabled() != b)
     {
       if (!b && selected != null) select(null);
       bits = MathBox.getBits(bits, ENSEL_BIT, b);
       repaint();
     }
   }

   public bool isSelectionEnabled() {
     return MathBox.checkBit(bits, ENSEL_BIT);
   }

   public void showLines(bool b)
   {
     if (isLinesShown() != b)
     {
       bits = MathBox.getBits(bits, ENLINES_BIT, b);
       repaint();
     }
   }

   public bool isLinesShown() {
     return MathBox.checkBit(bits, ENLINES_BIT);
   }

   public Color getLinesColor() {
     return linesColor;
   }

   public void setLinesColor(Color c)
   {
     if (!c.Equals(linesColor))
     {
       linesColor = c;
       repaint();
     }
   }

   public void setGaps (int gx, int gy)
   {
     if ((gx >= 0 && gx != gapx)||
         (gy >= 0 && gy != gapy)  )
     {
       gapx = gx<0?gapx:gx;
       gapy = gy<0?gapy:gy;
       invalidateMetrics();
       repaint();
     }
   }

   public TreeModel getModel() {
     return data;
   }

   public void setViewProvider (LwViewProvider p)
   {
     if (p == null) p = this;
     if (provider != p)
     {
       provider = p;
       invalidateData();
       repaint();
     }
   }

   public LwViewProvider getViewProvider () {
     return provider;
   }

   public void setView(int id, LwView v)
   {
     switch (id)
     {
       case LEAST_VIEW     : if (v != leastImg)  { leastImg  = v; leastImgSize  = viewPS(v); } break;
       case OPENED_VIEW    : if (v != openedImg) { openedImg = v; openedImgSize = viewPS(v); } break;
       case CLOSED_VIEW    : if (v != closedImg) { closedImg = v; closedImgSize = viewPS(v); } break;
       case TOGGLE_OFF_VIEW: if (v != toggleOffView) { toggleOffView = v; toggleOffSize = viewPS(v); } break;
       case TOGGLE_ON_VIEW : if (v != toggleOnView)  { toggleOnView = v; toggleOnSize = viewPS(v); } break;
       default: throw new org.zaval.port.cs.IllegalArgumentException();
     }
     invalidateMetrics();
     repaint();
   }

   public void setModel(TreeModel d)
   {
     if (data != d)
     {
       if (data != null) data.removeTreeListener(this);
       data = d;
       if (data != null) data.addTreeListener(this);
       invalidateData();
       repaint();
     }
   }

   public override void paint (Graphics g)
   {
     if (data != null)
     {
       vVisibility();
       if (MathBox.checkBit(bits, VALVIS_BIT) && firstVisible != null)
         paintTree (g, firstVisible);
     }
   }

   public override void recalc()
   {
     if (max == null)
     {
       Insets i = getInsets();
       max = new Dimension (i.left, i.top);
       recalc (i.left, i.top, null, data.getRoot());
       max.width  -= i.left;
       max.height -= i.top;
     }
   }

   public void addSelectionListener(LwActionListener l) {
     if (selection == null) selection = new LwActionSupport();
     selection.addListener(l);
   }

   public void removeSelectionListener(LwActionListener l) {
     if (selection != null) selection.removeListener(l);
   }

   public LwView getView(Drawable d, object obj) {
     defaultRender.getTextModel().setText(obj.ToString());
     return defaultRender;
   }

   public void focusGained  (LwFocusEvent e) {}
   public void focusLost    (LwFocusEvent e) {}
   public void mouseClicked (LwMouseEvent e) {}
   public void mouseEntered (LwMouseEvent e) {}
   public void mouseExited  (LwMouseEvent e) {}
   public void mouseReleased(LwMouseEvent e) {}
   public void keyReleased  (LwKeyEvent e) {}
   public void keyTyped     (LwKeyEvent e) {}

   public void keyPressed (LwKeyEvent e)
   {
     if (selected != null)
     {
       switch (e.getKeyChar())
       {
         case '+' : if (!isOpen(selected)) toggle(selected); return;
         case '-' : if (isOpen(selected)) toggle(selected); return;
       }
     }

     Item newSelection = null;
     switch (e.getKeyCode())
     {
       case LwConstants.VK_DOWN :
       case LwConstants.VK_RIGHT: newSelection = findNext (selected); break;

       case LwConstants.VK_UP   :
       case LwConstants.VK_LEFT : newSelection = findPrev (selected); break;

       case LwConstants.VK_HOME : if ((e.getMask() & LwConstants.CTRL_MASK) > 0) select(data.getRoot());
                               break;
       case LwConstants.VK_END  : if ((e.getMask() & LwConstants.CTRL_MASK) > 0) select(findLast(data.getRoot()));
                               break;
       case LwConstants.VK_PAGE_DOWN : if (selected != null) select(nextPage(selected, 1));
                                    break;
       case LwConstants.VK_PAGE_UP   : if (selected != null) select(nextPage(selected, -1));
                                    break;
     }
     if (newSelection != null) select(newSelection);
   }

   public void mousePressed(LwMouseEvent e)
   {
     if (LwToolkit.isActionMask(e.getMask()))
     {
       int x = e.getX() - dx, y = e.getY() - dy;

       Item root = getItemAt(firstVisible, x, y);
       if (root != null)
       {
         Rectangle r = getToggleBounds(root);
         if (r.contains(x, y)) toggle(root);
         else if (x > r.x + r.width) select(root);
       }
     }
   }

   public void setSelectionColor (Color c, bool isHasFocus)
   {
      Color rc = getSelectionColor(isHasFocus);
      if (rc != c || (rc == null || !rc.Equals(c)))
      {
        if (isHasFocus) selectColor1 = c;
        else            selectColor2 = c;
        repaint();
      }
   }

   public Color getSelectionColor (bool isHasFocus) {
     return isHasFocus?selectColor1:selectColor2;
   }

   public void toggle(Item item)
   {
     validate();
     ItemMetrics node = getIM (item);
     node.isOpen = !node.isOpen;
     if (actions != null)  actions.perform (new LwActionEvent (this, item));
     invalidateMetrics();
     if (!node.isOpen && selected != null && isParentFor(item, selected)) select(item);
     else repaint();
   }

   public void select(Item item)
   {
     if (isSelectionEnabled())
     {
       if (item != selected)
       {
         selected = item;
         if (selected != null)
         {
           validate();
           Rectangle r = getViewBounds(selected);

           Point o = LwToolkit.calcOrigin(r.x, r.y, r.width, r.height, this);
           if (man != null) man.scrollObjMoved(o.x, o.y);
           else             setSOLocation(o.x, o.y);

           if (selection != null)
             selection.perform (new LwActionEvent(this, selected));
         }
         repaint();
       }
     }
   }

   public Item getSelectedItem() {
     return selected;
   }

   public void addActionListener(LwActionListener l) {
     if (actions == null) actions = new LwActionSupport();
     actions.addListener(l);
   }

   public void removeActionListener(LwActionListener l) {
     if (actions != null) actions.removeListener(l);
   }

   public void itemInserted(TreeEvent e) {
     invalidateMetrics();
     repaint();
   }

   public void itemRemoved (TreeEvent e)
   {
     if (nodes != null)
     {
       nodes.remove(e.getItem());
       invalidateMetrics();
       repaint();
     }
   }

   public void itemModified(TreeEvent e)
   {
     if (nodes != null)
     {
       ItemMetrics node = getIM(e.getItem());
       if (node != null) node.viewWidth = -1;
       invalidateMetrics();
       repaint();
     }
   }

   public override void invalidate () {
     bits = MathBox.getBits(bits, VALVIS_BIT, false);
     base.invalidate();
   }

   public override Point getOrigin() {
     return new Point(dx, dy);
   }

   public Point getSOLocation () {
     return getOrigin();
   }

   public void setSOLocation (int x, int y)
   {
     if (x != dx || y != dy)
     {
       dx = x;
       int prevDy = dy;
       dy = y;

       if (isVisibilityValid())
         firstVisible = (y < prevDy)?nextVisible(firstVisible):prevInVisible(firstVisible);
       repaint();
     }
   }

   public Dimension getSOSize() {
     return getPreferredSize();
   }

   public void setScrollMan (ScrollMan m) {
     man = m;
   }

   public bool moveContent() {
     return true;
   }

   public bool isOpen(Item i) {
     validate();
     return isOpen_(i);
   }

   public ItemMetrics getItemMetrics (Item i) {
     validate();
     return getIM (i);
   }

   public override Dimension calcPreferredSize() {
     return max == null?base.calcPreferredSize():new Dimension (max.width, max.height);
   }

   private int recalc(int x, int y, Item parent, Item root)
   {
     ItemMetrics node = getIM(root);
     if (node == null)
     {
       node = new ItemMetrics(isOpenVal);
       if (nodes == null) nodes = new Hashtable();
       nodes.put(root, node);
     }

     if (node.viewWidth < 0)
     {
       LwView nodeView = provider.getView(this, root);
       Dimension viewSize = nodeView.getPreferredSize();
       node.viewWidth  = viewSize.width == 0?5:viewSize.width;
       node.viewHeight = viewSize.height;
     }

     Dimension imageSize  = getImageSize (root);
     Dimension toggleSize = getToggleSize(root);
     if (parent != null)
     {
       Rectangle pImg = getImageBounds(parent);
       x = pImg.x + (pImg.width - toggleSize.width)/2;
     }

     node.x      = x;
     node.y      = y;
     node.width  = toggleSize.width + imageSize.width + node.viewWidth +
                   (toggleSize.width>0?gapx:0) + (imageSize.width>0?gapx:0);
     node.height = System.Math.Max(System.Math.Max(toggleSize.height, imageSize.height), node.viewHeight);

     if (node.x + node.width > max.width ) max.width  = node.x + node.width;
     if (node.y + node.height> max.height) max.height = node.y + node.height;

     x  = node.x + toggleSize.width + (toggleSize.width>0?gapx:0);
     y += (node.height + gapy);

     //!!!
     // Take care that it is necessary to use method isOpen(Item), but the method
     // implementation works correctly if the field isOpenVal is used. The feature allows
     // speed up performance for the method.
     //!!!
     if (node.isOpen)
     {
       int count = data.getChildrenCount(root);
       for (int i=0; i<count; i++) y = recalc(x, y, root, data.getChildAt(root, i));
     }
     return y;
   }

   private bool isOpen_(Item i) {
     return (i == null || (data.hasChildren(i) && getIM(i).isOpen && isOpen_(data.getParent(i))));
   }

   private ItemMetrics getIM(Item i) {
     return (nodes == null)?null:(ItemMetrics)nodes.get(i);
   }

   private Item getItemAt (Item root, int x, int y)
   {
     if (isVisibilityValid() && y + dy >= visibleArea.y && y + dy < visibleArea.y + visibleArea.height)
     {
       Item found = getItemAtInBranch(root, x, y);
       if (found != null) return found;

       Item parent = data.getParent(root);
       while(parent != null)
       {
         int index = data.getChildIndex(root);
         int count = data.getChildrenCount(parent);
         for (int i=index + 1; i<count; i++)
         {
           found = getItemAtInBranch (data.getChildAt(parent, i), x, y);
           if (found != null) return found;
         }
         root   = parent;
         parent = data.getParent(root);
       }
     }
     return null;
   }

   private Item getItemAtInBranch (Item root, int x, int y)
   {
     if (root != null)
     {
       ItemMetrics node = getIM(root);
       if (x >= node.x && y >= node.y && x < node.x + node.width && y < node.y + node.height + gapy)
         return root;
       if (isOpen_(root))
       {
         for (int i=0; i<data.getChildrenCount(root); i++)
         {
           Item res = getItemAtInBranch(data.getChildAt(root, i), x, y);
           if (res != null) return res;
         }
       }
     }
     return null;
   }

   private LwView getImageView (Item i)
   {
     if (data.hasChildren(i))
     {
       ItemMetrics node = getIM(i);
       return node.isOpen?openedImg:closedImg;
     }
     return leastImg;
   }

   private Dimension getImageSize (Item i)
   {
     if (data.hasChildren(i))
     {
       ItemMetrics node = getIM(i);
       return node.isOpen?openedImgSize:closedImgSize;
     }
     return leastImgSize;
   }

   private Rectangle getImageBounds (Item root)
   {
     ItemMetrics node = getIM(root);
     Dimension id = getImageSize(root);
     Dimension td = getToggleSize(root);
     return new Rectangle(node.x + td.width + (td.width>0?gapx:0), node.y + (node.height - id.height)/2, id.width, id.height);
   }

   private Rectangle getToggleBounds (Item root)
   {
     ItemMetrics node = getIM(root);
     Dimension d = getToggleSize(root);
     return new Rectangle(node.x, node.y + (node.height - d.height)/2, d.width, d.height);
   }

   private Rectangle getViewBounds(Item root)
   {
     ItemMetrics metrics = getIM(root);
     Rectangle toggle = getToggleBounds(root);
     Rectangle image  = getImageBounds(root);
     return new Rectangle(image.x + image.width + (image.width > 0 || toggle.width > 0 ?gapx:0),
                          metrics.y + (metrics.height - metrics.viewHeight)/2,
                          metrics.viewWidth, metrics.viewHeight);
   }

   private LwView getToggleView (Item i)
   {
     if (data.hasChildren(i))
     {
       ItemMetrics node = getIM(i);
       return node.isOpen?toggleOnView:toggleOffView;
     }
     return null;
   }

   private Dimension getToggleSize (Item i)  {
     return isOpen_(i)?toggleOnSize:toggleOffSize;
   }

   private bool isAbove (Item i){
     ItemMetrics node = getIM(i);
     return node.y + node.height + dy < visibleArea.y;
   }

   private bool isVisible (Item i)  {
     ItemMetrics node = getIM(i);
     return visibleArea.intersects(new Rectangle(node.x + dx, node.y + dy , node.width, node.height));
   }

   private bool isParentFor (Item parent, Item item)  {
     while ((item = data.getParent(item))!= null && parent != item);
     return (item == parent && parent != null);
   }

   private Item findNext(Item item)
   {
     if (item == null) return null;
     if (data.hasChildren(item) && isOpen_(item)) return data.getChildAt(item, 0);

     Item parent = null;
     while ((parent = data.getParent(item)) != null)
     {
       int index = data.getChildIndex(item);
       int count = data.getChildrenCount(parent);
       if (index + 1 < count) return  data.getChildAt(parent, index + 1);
       item = parent;
     }
     return null;
   }

   private Item findPrev(Item item)
   {
     if (item == null) return null;
     Item parent = data.getParent(item);
     if (parent != null)
     {
       int index = data.getChildIndex(item);
       int count = data.getChildrenCount(parent);
       if (index - 1 >= 0) return findLast(data.getChildAt(parent, index - 1));
       else                return parent;
     }
     return null;
   }

   private Item findLast(Item item)
   {
     if (item == null) return null;
     int count = data.getChildrenCount(item);
     if (count == 0 || !isOpen_(item)) return item;
     else
     if (count > 0) return findLast(data.getChildAt(item, count - 1));
     return null;
   }

   private Item prevInVisible(Item item)
   {
     Item parent = null;
     if (item != null)
     while ((parent = data.getParent(item)) != null)
     {
       if (isAbove(item)) return item;
       int index = data.getChildIndex(item);
       for (int i=index-1; i>=0; i--)
       {
         Item res = nextInVisibleInBranch(data.getChildAt(parent, i));
         if (res != null) return res;
       }
       item = parent;
     }
     return data.getRoot();
   }

   private Item nextVisible(Item item)
   {
     Item res = nextVisibleInBranch(item);
     if (res != null) return res;

     Item parent = data.getParent(item);
     while (parent != null)
     {
       int count = data.getChildrenCount(parent);
       for (int i=data.getChildIndex(item) + 1; i<count; i++)
       {
         res = nextVisibleInBranch(data.getChildAt(parent, i));
         if (res != null) return res;
       }
       item   = parent;
       parent = data.getParent(item);
     }
     return null;
   }

   private Item nextInVisibleInBranch (Item item)
   {
     if (item != null)
     {
       if (isAbove(item)) return item;
       if (isOpen_(item))
       {
          for (int i=data.getChildrenCount(item)-1; i>=0; i--)
          {
            Item res = nextInVisibleInBranch(data.getChildAt(item, i));
            if (res != null) return res;
          }
       }
     }
     return null;
   }

   private Item nextVisibleInBranch (Item item)
   {
     if (item != null)
     {
       if (isVisible(item)) return item;
       if (isOpen_(item))
       {
          for (int i=0; i < data.getChildrenCount(item); i++)
          {
            Item res = nextVisibleInBranch(data.getChildAt(item, i));
            if (res != null) return res;
          }
       }
     }
     return null;
   }

   private void paintTree (Graphics g, Item item)
   {
     paintBranch(g, item);
     Item parent = null;
     while ((parent = data.getParent(item)) != null)
     {
       paintChild (g, parent, data.getChildIndex(item)+1);
       item = parent;
     }
   }

   private bool paintBranch(Graphics g, Item root)
   {
     if (root == null) return false;

     ItemMetrics node = getIM(root);
     if (isVisible(root))
     {
       Rectangle toggle     = getToggleBounds(root);
       LwView    toggleView = getToggleView(root);
       if (toggleView != null) toggleView.paint(g, toggle.x, toggle.y, this);

       Rectangle image = getImageBounds(root);
       if (image.width > 0) getImageView (root).paint(g, image.x, image.y, this);

       int vx = image.x + image.width + (image.width > 0 || toggle.width > 0 ?gapx:0);
       int vy = node.y + (node.height - node.viewHeight)/2;
       LwView nodeView = provider.getView(this, root);
       nodeView.paint(g, vx, vy, this);

       if (isLinesShown())
       {
         g.setColor (linesColor);
         LwToolkit.drawDotHLine(g, toggle.x + (toggleView==null?toggle.width/2 + 1:toggle.width) + 1, image.x - 1, toggle.y + toggle.height/2);
       }

       if (selected == root)
       {
         Color selectColor = getSelectionColor(hasFocus());
         if (selectColor != null)
           LwToolkit.drawMarker(g, vx, vy, node.viewWidth, node.viewHeight, getBackground(), selectColor);
       }
     }
     else
     {
       if (node.y + dy > visibleArea.y + visibleArea.height||
           node.x + dx > visibleArea.x + visibleArea.width   ) return false;
     }
     return paintChild (g, root, 0);
   }

   private int y_(Item item, bool isStart)
   {
     Rectangle r = getToggleBounds(item);
     if (data.hasChildren(item)) return isStart?r.y + r.height:r.y - 1;
     else                        return r.y + r.height/2;
   }

   private bool paintChild (Graphics g, Item root, int index)
   {
     if (isOpen_(root))
     {
       Rectangle tb = getImageBounds(root);
       //???
       Rectangle br = getToggleBounds(data.getChildAt(root, 0));
       int x = br.x + br.width/2, count = data.getChildrenCount(root);
       if (index < count)
       {
         int y = (index > 0)?y_(data.getChildAt(root, index-1), true):tb.y + tb.height;
         for (int i=index; i<count; i++)
         {
           Item child = data.getChildAt(root, i);
           if (isLinesShown())
           {
             g.setColor(linesColor);
             LwToolkit.drawDotVLine(g, y, y_(child, false), x);
             y = y_(child, true);
           }

           if (!paintBranch(g, child))
           {
             if (isLinesShown() && i + 1 != count)
             {
               g.setColor(linesColor);
               LwToolkit.drawDotVLine(g, y, height - dy, x);
             }
             return false;
           }
         }
       }

       if (isLinesShown() && index == count && count > 0)
       {
         g.setColor(linesColor);
         LwToolkit.drawDotVLine(g, y_(root, true), y_(data.getChildAt(root, index-1), false), x);
       }
     }
     return true;
   }

   private Item nextPage(Item item, int dir)
   {
     int sum = 0;
     Item prev = item;
     while (item != null && sum < visibleArea.height)
     {
       ItemMetrics node = getIM(item);
       sum += (node.height + gapy);
       prev = item;
       item = dir < 0?findPrev(item):findNext(item);
     }
     return prev;
   }

   private void invalidateData() {
     if (nodes != null) nodes.clear();
     invalidateMetrics();
   }

   private bool isVisibilityValid () {
     return MathBox.checkBit(bits, VALVIS_BIT);
   }

   private void vVisibility ()
   {
     //!!!
     Rectangle nva = getVisiblePart();
     if (nva == null) return;

     if (!isVisibilityValid() ||
         (visibleArea != null && !nva.Equals(visibleArea)))
     {
       visibleArea = nva;
       if (visibleArea != null)
       {
         firstVisible = null;
         if (-dy > max.height/2)
           firstVisible = prevInVisible(findLast(data.getRoot()));
         else
           firstVisible = nextVisible(data.getRoot());
         bits = MathBox.getBits(bits, VALVIS_BIT, true);
       }
     }
   }

   private Dimension viewPS(LwView v) {
     return (v == null)?new Dimension():v.getPreferredSize();
   }

   private void invalidateMetrics()  {
     max = null;
     invalidate();
   }

   private const  short ENSEL_BIT   = 32;
   private const  short ENLINES_BIT = 64;
   private const  short VALVIS_BIT  = 128;
}



}
