/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

namespace org.zaval.port.cs
{
  public class Color
  {
    public static readonly  Color gray      = new Color(128, 128, 128);
    public static readonly  Color darkGray  = new Color(64, 64, 64);
    public static readonly  Color lightGray = new Color(192, 192, 192);
    public static readonly  Color black     = new Color(0,0,0);
    public static readonly  Color white     = new Color(255, 255, 255);
    public static readonly  Color red       = new Color(255, 0, 0);
    public static readonly  Color blue      = new Color(0, 0, 255);
    public static readonly  Color yellow    = new Color(255, 255, 0);
    public static readonly  Color pink      = new Color(255, 175, 175);
    public static readonly  Color orange    = new Color(255, 200, 0);
    public static readonly  Color green     = new Color(0, 255, 0);
    public static readonly  Color magenta   = new Color(255, 0, 255);
    public static readonly  Color cyan      = new Color(0, 255, 255);

    private int color;

    public Color(int r, int g, int b): this ((r<<16 | g << 8 | b))  {
    }

    public Color(int c) {
      color = c & 0xFFFFFF;
    }

    public int getRed() {
      return 0xFF & (color >> 16);
    }

    public int getGreen() {
      return 0xFF & (color >> 8);
    }

    public int getBlue() {
      return 0xFF & color;
    }

    public int getRGB() {
      return color;
    }

    public override bool Equals(object o) {
      if (o is Color) return color == ((Color)o).getRGB();
      return base.Equals(o);
    }

    public override int GetHashCode() {
      return base.GetHashCode();
    }

    public System.Drawing.Color toNativeColor () {
      return System.Drawing.Color.FromArgb(getRed(), getGreen(),getBlue());
    }
  }
}
