/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

namespace org.zaval.port.cs {

  public class Cursor
  {
    public const int HAND_CURSOR      = 1;
    public const int W_RESIZE_CURSOR  = 2;
    public const int N_RESIZE_CURSOR  = 3;
    public const int S_RESIZE_CURSOR  = 4;
    public const int E_RESIZE_CURSOR  = 5;
    public const int SE_RESIZE_CURSOR = 6;
    public const int SW_RESIZE_CURSOR = 7;
    public const int NE_RESIZE_CURSOR = 8;
    public const int NW_RESIZE_CURSOR = 9;
    public const int TEXT_CURSOR      = 10;
    public const int WAIT_CURSOR      = 11;
    public const int MOVE_CURSOR      = 12;
    public const int CROSSHAIR_CURSOR = 13;
    public const int DEFAULT_CURSOR   = 14;

    private int cursor;

    public Cursor():this(DEFAULT_CURSOR) {
    }

    public Cursor(int c) {
      cursor = c;
    }

    public int getType () {
      return cursor;
    }

    public static Cursor getDefaultCursor () {
      return new Cursor (Cursor.DEFAULT_CURSOR);
    }

    public static System.Windows.Forms.Cursor getNativeCursor(Cursor c)
    {
       switch (c.getType())
       {
         case HAND_CURSOR     : return System.Windows.Forms.Cursors.Hand;
         case E_RESIZE_CURSOR :
         case W_RESIZE_CURSOR : return System.Windows.Forms.Cursors.SizeWE;
         case S_RESIZE_CURSOR :
         case N_RESIZE_CURSOR : return System.Windows.Forms.Cursors.SizeNS;
         case NW_RESIZE_CURSOR:
         case SE_RESIZE_CURSOR: return System.Windows.Forms.Cursors.SizeNWSE;
         case NE_RESIZE_CURSOR:
         case SW_RESIZE_CURSOR: return System.Windows.Forms.Cursors.SizeNESW;
         case TEXT_CURSOR     : return System.Windows.Forms.Cursors.IBeam;
         case WAIT_CURSOR     : return System.Windows.Forms.Cursors.WaitCursor;
         case MOVE_CURSOR     : return System.Windows.Forms.Cursors.SizeAll;
         case DEFAULT_CURSOR  : return System.Windows.Forms.Cursors.Default;
         case CROSSHAIR_CURSOR: return System.Windows.Forms.Cursors.Cross;
         default: throw new IllegalArgumentException();
       }
    }
  }
}
