/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

namespace org.zaval.port.cs
{

  public class Font
  {
    public const int BOLD   = 1;
    public const int ITALIC = 2;
    public const int PLAIN  = 4;

    private System.Drawing.Font target;
    private System.Drawing.FontStyle style;

    private static Hashtable map = new Hashtable();

    public Font(string name, int s, int size)
    {
      style = System.Drawing.FontStyle.Regular;
      if ((s & PLAIN)  > 0) style |= System.Drawing.FontStyle.Regular;
      if ((s & BOLD )  > 0) style |= System.Drawing.FontStyle.Bold;
      if ((s & ITALIC) > 0) style |= System.Drawing.FontStyle.Italic;
      name = (string)map.get(name);
      if (name == null) name = (string)map.get("default");
      target = new System.Drawing.Font(name, size, style, System.Drawing.GraphicsUnit.Pixel);
    }

    public System.Drawing.Font getSrc() {
      return target;
    }

    public System.Drawing.FontStyle getFontStyle() {
      return style;
    }

    public override bool Equals (object o) {
      return o is Font && target.Equals(((Font)o).getSrc());
    }

    public override int GetHashCode() {
      return target.GetHashCode();
    }

    static Font ()
    {
      //???
      map.put ("default", "Arial");
      map.put ("dialog", "Arial");
      map.put ("helvetica", "Arial");
      map.put ("sansserif", "Arial");
    }
  }
}
