/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

namespace org.zaval.port.cs {

  public class FontMetrics
  {
    private Font font;
    private int ascent, descent;

    public FontMetrics(Font f)
    {
      font = f;
      ascent  = (int)font.getSrc().FontFamily.GetCellAscent(font.getFontStyle());
      descent = (int)font.getSrc().FontFamily.GetCellDescent(font.getFontStyle());
      ascent  = (int)(font.getSrc().Size * ascent) / font.getSrc().FontFamily.GetEmHeight(font.getSrc().Style);
      descent = (int)(font.getSrc().Size * descent)/ font.getSrc().FontFamily.GetEmHeight(font.getSrc().Style);
    }

    public Font getFont() {
      return font;
    }

    public int getHeight() {
      return font.getSrc().Height;
    }

    public int getDescent() {
      return descent;
    }

    public int getAscent() {
      return ascent;
    }

    public int stringWidth(string s) {
      return (int)(stringWidthF(s).Width);
    }

    public int charsWidth(char[] buf, int off, int len) {
      return (int)(charsWidthF(buf, off, len).Width);
    }

    public System.Drawing.RectangleF charsWidthF(char[] buf, int off, int len)
    {
      if (len == 0 || buf.Length == 0) return new System.Drawing.RectangleF(0, 0, 0, 0);
      string s = new string(buf);
      System.Drawing.CharacterRange[] ranges = { new System.Drawing.CharacterRange(off, len) };
      format.SetMeasurableCharacterRanges(ranges);
      System.Drawing.Region[] r = fmg.MeasureCharacterRanges(s, font.getSrc(),
                                  formatRect,
                                  format);
      return (r[r.Length - 1]).GetBounds(fmg);
    }

    public System.Drawing.RectangleF stringWidthF(string s)
    {
      if (s.Length == 0) return new System.Drawing.RectangleF(0, 0, 0, 0);

      System.Drawing.CharacterRange[] ranges = { new System.Drawing.CharacterRange(0, s.Length) };
      format.SetMeasurableCharacterRanges(ranges);
      System.Drawing.Region[] r = fmg.MeasureCharacterRanges(s, font.getSrc(),
                                  formatRect,
                                  format);
      return (r[r.Length - 1]).GetBounds(fmg);
    }

    public static System.Drawing.StringFormat getStringFormat() {
      return format;
    }

    private static System.Drawing.Graphics     fmg;
    private static System.Drawing.StringFormat format;
    private static System.Drawing.RectangleF   formatRect = new System.Drawing.RectangleF(0, 0, 10000, 10000);

    static FontMetrics()
    {
      fmg    = System.Drawing.Graphics.FromImage(new System.Drawing.Bitmap(1, 1));
      format = System.Drawing.StringFormat.GenericTypographic;
      format.FormatFlags = System.Drawing.StringFormatFlags.NoWrap | System.Drawing.StringFormatFlags.MeasureTrailingSpaces;
    }
  }
}
