/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

namespace  org.zaval.port.cs {

  public class Graphics
  {
    private System.Drawing.Graphics target;
    private System.Drawing.Pen pen, dpen;
    private System.Drawing.Brush brush;
    private Font font;
    private Color color;
    private int fontAscent;

    public Graphics(System.Drawing.Graphics g)  {
      target = g;
    }

    public Graphics(System.Windows.Forms.PaintEventArgs a) {
      target = a.Graphics;
      target.SetClip (a.ClipRectangle);
    }

    public Font getFont () {
      return font;
    }

    public FontMetrics getFontMetrics () {
      return new FontMetrics(font);
    }

    public FontMetrics getFontMetrics (Font f) {
      return new FontMetrics(f);
    }

    public Color getColor () {
      return color;
    }

    public void setColor (Color c)
    {
      if (color == null || !c.Equals(color))
      {
        color = c;
        System.Drawing.Color nc = color.toNativeColor();
        if (pen   != null) pen.Dispose();
        if (dpen  != null) dpen.Dispose();
        if (brush != null) brush.Dispose();
        pen   = new System.Drawing.Pen(nc);
        brush = new System.Drawing.SolidBrush(nc);
        dpen  = new System.Drawing.Pen(nc);
        dpen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
      }
    }

    public void setFont (Font f) {
      font = f;
      fontAscent = getFontMetrics(f).getAscent();
    }

    public void drawLine (int x1, int y1, int x2, int y2) {
      if (x1 == x2 && y1 == y2) target.DrawLine(dpen, x1, y1, x2+1, y2);
      else                      target.DrawLine(pen, x1, y1, x2, y2);
    }

    public void drawRect (int x, int y, int w, int h) {
      target.DrawRectangle(pen, x, y, w, h);
    }

    public bool drawImage (Image img, int x, int y, object observer) {
      target.DrawImage(img.getSrc(), x, y);
      return true;
    }

    public bool drawImage (Image img, int x, int y, int w, int h, object observer) {
      target.DrawImage(img.getSrc(), x, y, w, h);
      return true;
    }

    public bool drawImage(Image img, int dx1, int dy1, int dx2, int dy2,
                                     int sx1, int sy1, int sx2, int sy2, object observer)
    {
      System.Drawing.Rectangle r = new System.Drawing.Rectangle();
      r.X      = dx1;
      r.Y      = dy1;
      r.Width  = dx2 - dx1;
      r.Height = dy2 - dy1;
      target.DrawImage(img.getSrc(), r, sx1, sy1, sx2 - sx1, sy2 - sy1, System.Drawing.GraphicsUnit.Pixel);
      return true;
    }

    public void drawArc(int x, int y, int w, int h, int startAngle, int arcAngle) {
      target.DrawArc(pen, x, y, w, h, startAngle, arcAngle);
    }

    public void drawOval(int x, int y, int w, int h) {
      target.DrawEllipse(pen, x, y, w, h);
    }

    public void drawString(string str, int x, int y) {
      target.DrawString(str, font.getSrc(), brush, x, y - fontAscent, FontMetrics.getStringFormat());
    }

    public void fillRect(int x, int y, int w, int h) {
      target.FillRectangle(brush, x, y, w, h);
    }

    public void clearRect(int x, int y, int w, int h) {
      //???
       target.FillRectangle(brush, new System.Drawing.Rectangle(x, y, w, h));
    }

    public void fillArc(int x, int y, int w, int h, int startAngle, int arcAngle) {
      target.FillPie(brush, x, y, w, h, startAngle, arcAngle);
    }

    public void setClip(int x, int y, int w, int h) {
      target.SetClip(new System.Drawing.Rectangle(x, y, w, h));
    }

    public void setClip(Rectangle r) {
      target.SetClip(new System.Drawing.Rectangle(r.x, r.y, r.width, r.height));
    }

    public void clipRect(int x, int y, int w, int h) {
      target.IntersectClip(new System.Drawing.Rectangle(x, y, w, h));
    }

    public Rectangle getClipBounds ()  {
      return new Rectangle((int)target.ClipBounds.X, (int)target.ClipBounds.Y,
                           (int)target.ClipBounds.Width, (int)target.ClipBounds.Height);
    }

    public void translate (int dx, int dy) {
      target.TranslateTransform (dx, dy);
    }

    public void dispose() {
      //target.Dispose();
    }

    public System.Drawing.Graphics getSrc () {
      return target;
    }
  }
}
