/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {

using org.zaval.lw.events;
using org.zaval.port.cs;

public class LwFrame: System.Windows.Forms.Form
{
  private LwDesktop root;

  public LwFrame()
  {
    root = LwToolkit.createDesktop();
    Controls.Add((System.Windows.Forms.UserControl)root);
  }

  public void setTitle (string t) {
    Text = t;
  }

  public void setSize(int w, int h) {
    Width  = w;
    Height = h;
  }


  public void setLocation (int x, int y) {
    Left = x;
    Top  = y;
  }

  public void setBounds (int x, int y, int w, int h)
  {
    Width  = w;
    Height = h;
    Left   = x;
    Top    = y;
  }

  public Point getLocation () {
    return new Point (Left, Top);
  }

  public void setVisible (bool b) {
     Visible = b;
     System.Windows.Forms.Application.Run(this);
  }

  public Dimension getSize () {
    return new Dimension (Width, Height);
  }

  public Rectangle getBounds () {
    return new Rectangle (Left, Top, Width, Height);
  }

  public LwContainer getRoot () {
    return root.getRootLayer();
  }

  // focusGained
  protected override void OnGotFocus(System.EventArgs a) {
    base.OnGotFocus(a);
    ((System.Windows.Forms.UserControl)root).Focus();
  }

  protected override void InitLayout()
  {
    root.setLocation(0, 0);
    root.setSize(Width, Height);
  }

  // validate
  protected override void OnLayout (System.Windows.Forms.LayoutEventArgs a) {
    root.setLocation(ClientRectangle.X, ClientRectangle.Y);
    root.setSize(ClientRectangle.Width, ClientRectangle.Height);
    root.validate();
  }


  protected override void OnClosed(System.EventArgs a)
  {
    //???
    System.Windows.Forms.Application.Exit();
    org.zaval.util.Timer.getTimer(false).interrupt();
  }

}
}
