/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {

using org.zaval.lw.events;
using org.zaval.lw;
using org.zaval.port.cs;
using org.zaval.util;
using System.Reflection;

public class LwManager
{
  internal static Hashtable staticObjects;
  internal static string version;

  public static object getStaticObject(string key) {
    return staticObjects.get(key);
  }

  public static LwView getView(string key) {
    return (LwView)staticObjects.get(key);
  }

  public static LwLayout getLwLayout(string key) {
    return (LwLayout)staticObjects.get(key);
  }

  public static void loadObjects(string key, Properties props) {
    loadObjects(key, props, staticObjects);
  }

  public static string getVersion () {
    return version;
  }

  public static void loadObjects(string key, Properties props, Hashtable res)
  {
      string val = props.getProperty(key);
      if (val != null)
      {
        char[]   seps   = {','};
        string[] values = val.Split(seps);

        for (int i=0; i < values.Length; i++)
        {
          object m         = null;
          string token     = values[i];
          string className = props.getProperty(key + "." + token + ".cl").Replace("event.", "events.");
          string keyName   = props.getProperty(key + "." + token + ".key");

          if (className.IndexOf('@') == 0)
          {
            m = staticObjects == null?res.get(className.Substring(1)):getStaticObject(className.Substring(1));
          }
          else
          {
            string constructorArg = props.getProperty(key + "." + token + ".arg");
            string typeName = className.IndexOf('.')>0?className:(PACKAGE_NAME + className);
    	    if (constructorArg == null)
    	    {
    	      m = assem.CreateInstance(typeName, false);
    	    }
            else
            {
              string[] args  = constructorArg.Split(seps);
              object[] argsValues = new object[args.Length];
              for (int j = 0; j < args.Length; j++)
              {
                args[j] = args[j].Trim();
                if (args[j].IndexOf("\"")==0)
                {
                  argsValues[j] = args[j].Substring(1, args[j].Length - 2);
	        }
                else
                if (args[j].Equals("true") || args[j].Equals("false"))
                {
                  argsValues[j] = args[j];
                }
                else
                if (args[j].IndexOf('@') == 0)
                {
                  argsValues[j] = staticObjects == null?res.get(args[j].Substring(1)):getStaticObject(args[j].Substring(1));
                }
                else
                {
                  argsValues[j] = System.Convert.ToInt32(args[j]);
                }
              }

              if (className.Equals("java.lang.String"))
                 m = argsValues[0];
              else
                 m = assem.CreateInstance(typeName, false, BindingFlags.CreateInstance, null, argsValues, null, null);
            }
          }

          res.put(keyName == null?token:keyName, m);
        }
     }
  }

  private static string   PACKAGE_NAME = "org.zaval.lw.";
  private static Assembly assem        = Assembly.GetCallingAssembly();

  static LwManager()
  {
    try {
      staticObjects = new Hashtable();
      Properties props = LwToolkit.getProperties("rs/lw.properties");
      if (props == null) throw new org.zaval.port.cs.RuntimeException("Properties file not found.");
      version = props.getProperty("lwvcl.ver");

      Properties extProps = LwToolkit.getProperties("rs/lwext.properties");

      LwEventManager.manager = (LwEventManager)assem.CreateInstance(PACKAGE_NAME + props.getProperty("man.event.cl").Replace("event.", "events."), false);
      staticObjects.put ("event", LwEventManager.manager);

      loadObjects("man", props, staticObjects);
      if (extProps != null) loadObjects("man", extProps, staticObjects);

      for (Enumeration en = staticObjects.elements();en.hasMoreElements();)
      {
        object manager = en.nextElement();
        System.Type field = manager.GetType();
        field.InvokeMember("manager",  System.Reflection.BindingFlags.FlattenHierarchy |
                                       System.Reflection.BindingFlags.Public |
                                       System.Reflection.BindingFlags.Static |
                                       System.Reflection.BindingFlags.SetField, null, manager,
                                       new object[] {manager});
        if (manager is EventListener) LwEventManager.manager.addXXXListener((EventListener)manager);
      }

      loadObjects("obj", props, staticObjects);
      if (extProps != null) loadObjects("obj", extProps, staticObjects);
    }
    catch (System.Exception e) {
      System.Console.WriteLine(e);
    }
  }
}

}
