/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {

using org.zaval.lw;
using org.zaval.port.cs;

public class LwPaintManImpl:LwPaintManager
{
  private bool isBuffered;

  public LwPaintManImpl():this(false) {
  }

  public LwPaintManImpl(bool isBuffered) {
    this.isBuffered  = isBuffered;
  }

  public bool useDoubleBuffer() {
    return isBuffered;
  }

  public override void updateComponent (Graphics g, LwComponent c)
  {
     LwViewMan skins = c.getViewMan(false);
     Dimension d = c.getSize();
     if (skins != null && skins.getBg() != null) paintSkin(g, skins.getBg(), c);
     else
     {
       g.setColor(c.getBackground());
       g.fillRect(0, 0, d.width, d.height);
     }
     c.update(g);
  }

  public override void paintComponent(Graphics g, LwComponent c)
  {
    LwViewMan skins = c.getViewMan(false);
    LwView    view  = null;
    if (skins != null)
    {
      paintSkin(g, skins.getBorder(), c);
      view = skins.getView();
      if (view != null && view.getType() != LwView.ORIGINAL)
      {
        paintSkin(g, view, c);
        view = null;
      }
    }

    Insets insets = c.getInsets();
    if (insets.left + insets.right + insets.top + insets.bottom > 0)
    {
      //
      //!!! This is MSIE clipRect() method equivalent.
      //
      Rectangle clip = g.getClipBounds();
      int x1 = System.Math.Max(clip.x, insets.left);
      int y1 = System.Math.Max(clip.y, insets.top);
      g.setClip(x1, y1, System.Math.Min(clip.x + clip.width,  insets.left + c.getWidth() - insets.left - insets.right) - x1,
                        System.Math.Min(clip.y + clip.height, insets.top + c.getHeight() - insets.top - insets.bottom) - y1);
    }

    Point p = c.getOrigin();
    if (p != null) g.translate(p.x, p.y);
    if (view != null) paintSkin(g, view, c);
    c.paint(g);
    if (p != null) g.translate(-p.x, -p.y);
  }

  public override void paintOnTop (Graphics g, LwContainer c)
  {
    Point p = c.getOrigin();
    if (p != null) g.translate(p.x, p.y);
    c.paintOnTop(g);
    if (p != null) g.translate(-p.x, -p.y);
  }

  private static void paintSkin(Graphics g, LwView s, Drawable c)
  {
    if (s != null)
    {
      switch (s.getType())
      {
        case LwView.STRETCH :
        {
          s.paint(g, 0, 0, c.getWidth(), c.getHeight(), c);
        } break;
        case LwView.MOSAIC  :
        {
          Dimension ps = s.getPreferredSize();
          if (ps.width > 0 && ps.height > 0)
          {
            int dx = c.getWidth()/ps.width + (c.getWidth()%ps.width>0?1:0);
            int dy = c.getHeight()/ps.height + (c.getHeight()%ps.height>0?1:0);
            int xx = 0, yy;
            for (int i=0; i<dx; i++)
            {
               yy = 0;
               for (int j=0; j<dy; j++)
               {
                 s.paint(g, xx, yy, ps.width, ps.height, c);
                 yy += ps.height;
               }
               xx += ps.width;
            }
          }
        } break;
        case LwView.ORIGINAL:
        {
          Insets i= c.getInsets();
          s.paint(g, i.left, i.top, c);
        } break;
      }
    }
  }

}

}
