/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.lw.events;
using org.zaval.lw;
using org.zaval.port.cs;
using org.zaval.util.events;
using org.zaval.util;

public class LwRoot:System.Windows.Forms.UserControl, LwDesktop, LwLayout
{
  public const  int CURSOR_PROPERTY = 1;

  private LwPanel proxy;
  private LwLayer root;
  private Hashtable layers = new Hashtable ();
  private LwComponent draggOwner, moveOwner, pressOwner, last;
  private int keyValue, keyMask, pressX, pressY;
  private bool isValidVal;
  private Cursor cursor;

  public LwRoot()
  {
    proxy = new LwPanel();
    proxy.setLwLayout(this);
    proxy.setLwParent(this);
    root    = new LwLayer("root");
    proxy.add (root);
    proxy.add (new LwWinLayer());

    // Cursor
    setProperty(CURSOR_PROPERTY, new Cursor());

    SetStyle(System.Windows.Forms.ControlStyles.DoubleBuffer |
             System.Windows.Forms.ControlStyles.UserPaint |
             System.Windows.Forms.ControlStyles.AllPaintingInWmPaint,
             true);
    UpdateStyles();
  }


  public int getWidth() {
    return proxy.getWidth();
  }

  public int getHeight() {
    return proxy.getHeight();
  }

  public int getX() {
    return proxy.getX();
  }

  public int getY() {
    return proxy.getY();
  }

  public void setLwLayout (LwLayout l) {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public LwLayout getLwLayout () {
    return proxy.getLwLayout();
  }

  public LwComponent getLwParent() {
    return null;
  }

  public void setLwParent(LwComponent p) {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public LwViewMan getViewMan (bool b) {
    return null;
  }

  public void setViewMan (LwViewMan v) {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public Rectangle getVisiblePart() {
    return isVisible()?new Rectangle(0, 0, getWidth(), getHeight()):null;
  }

  public LwComponent getLwComponentAt(int x, int y)
  {
    LwLayer targetLayer = fal();
    if (targetLayer != null)
    {
/*???      if (last != null && LwToolkit.isAncestorOf(targetLayer, last))
      {
        Point p = LwToolkit.getAbsLocation(last);
        if (x >= p.x && y >= p.y && x < p.x + last.getWidth() && y < p.y + last.getHeight())
        {
          last = LwEventManager.manager.getEventDestination(last.getLwComponentAt(x - p.x, y - p.y));
          if (last != null) return last;
        }
      }*/

      last = LwEventManager.manager.getEventDestination(targetLayer.getLwComponentAt(x, y));
      return last;
    }
    return null;
  }

  public bool isOpaque () {
    return true;
  }

  public Layoutable get(int index) {
    return proxy.get(index);
  }

  public int count() {
    return proxy.count();
  }

  public void add(LwComponent c) {
    proxy.add(c);
  }

  public void add(object s, LwComponent c) {
    proxy.add(s, c);
  }

  public void insert(int i, object s, LwComponent d) {
    proxy.insert(i, s, d);
  }

  public void remove(int i) {
    if (get(i) == root) throw new org.zaval.port.cs.RuntimeException();
    proxy.remove(i);
  }

  public void removeAll() {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public int indexOf(LwComponent c) {
    return proxy.indexOf(c);
  }

  public void validate()
  {
    if (!isValidVal)
    {
      proxy.validate();
      isValidVal = true;
    }
  }

  public void setEnabled (bool b) {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public Color getBackground () {
    return proxy.getBackground();
  }

  public void setBackground (Color c) {
    proxy.setBackground(c);
  }

  public Dimension getPreferredSize() {
    return proxy.getPreferredSize();
  }

  public Insets getInsets() {
    return proxy.getInsets();
  }

  public void setOpaque (bool b) {
    throw new org.zaval.port.cs.RuntimeException();
  }

  public Point getOrigin () {
    return null;
  }

  public Point getLayoutOffset() {
    return new Point();
  }

  public LwLayer getRootLayer () {
    return root;
  }

  public LwLayer getLayer (object id) {
    return (LwLayer)layers.get(id);
  }

  public object[] getLayersIDs ()
  {
    object[] ids = new object[layers.size()];
    Enumeration en  = layers.keys();
    for (int i=0; i < ids.Length; i++) ids[i] = en.nextElement();
    return ids;
  }

  public void update(Graphics g) {
    paint (g);
  }

  public void paint(Graphics g) {
    if (Parent != null && Parent.Visible) org.zaval.lw.LwPaintManager.manager.rootPaint(g, proxy);
  }


  public void paintOnTop(Graphics g) {
  }

  // update
  protected override void OnPaintBackground (System.Windows.Forms.PaintEventArgs a) {
  }

  // paint
  protected override void OnPaint (System.Windows.Forms.PaintEventArgs a) {
    base.OnPaint(a);
    paint(new Graphics(a));
  }

  // componentHidden/componentShown
  protected override void OnVisibleChanged(System.EventArgs a) {
    base.OnVisibleChanged(a);
    proxy.setVisible(Visible);
  }

  // reshape
  protected override void OnLocationChanged(System.EventArgs a) {
    base.OnLocationChanged(a);
    proxy.setLocation(Left, Top);
  }

  // reshape
  protected override void OnSizeChanged(System.EventArgs a) {
    base.OnSizeChanged(a);
    proxy.setSize(Width, Height);
  }

  // focusGained
  protected override void OnGotFocus(System.EventArgs a)
  {
    base.OnGotFocus(a);
    LwLayer activeLayer = fal();
    if (activeLayer != null) activeLayer.setupFocus();
  }

  // focusLost
  protected override void OnLostFocus(System.EventArgs a) {
    base.OnLostFocus(a);
    LwLayer activeLayer = fal();
    if (activeLayer != null) activeLayer.releaseFocus();
  }

  // mouseEntered
  protected override void OnMouseEnter(System.EventArgs a)
  {
    base.OnMouseEnter(a);

    if (draggOwner == null)
    {
      System.Drawing.Point p = PointToClient(new System.Drawing.Point (System.Windows.Forms.Cursor.Position.X,
                                                                       System.Windows.Forms.Cursor.Position.Y));

      LwComponent d = getLwComponentAt(p.X, p.Y);
      if (isEventable(d)){
        moveOwner = d;
        perform (new LwMouseEvent(d, LwMouseEvent.MOUSE_ENTERED, p.X, p.Y, 0, 0));
      }
    }
  }

  // mouseExited
  protected override void OnMouseLeave(System.EventArgs a)
  {
    base.OnMouseLeave(a);
    if (moveOwner != null && draggOwner == null)
    {
      System.Drawing.Point p = PointToClient(new System.Drawing.Point (System.Windows.Forms.Cursor.Position.X,
                                                                       System.Windows.Forms.Cursor.Position.Y));
      LwComponent prev = moveOwner;
      moveOwner = null;
      perform (new LwMouseEvent(prev, LwMouseEvent.MOUSE_EXITED, p.X, p.Y, 0, 0));
    }
  }

  // mousePressed
  protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
  {
    base.OnMouseDown(e);

    pressX = e.X;
    pressY = e.Y;

    int mask = formMouseMask(e);
    for (int i=proxy.count()-1; i>=0; i--)
    {
      LwLayer l = (LwLayer)proxy.get(i);
      l.mousePressed(pressX, pressY, mask);
      if (l.isActive()) break;
    }

    LwComponent d = getLwComponentAt(pressX, pressY);
    if (isEventable(d))
    {
      pressOwner = d;
      fme(d, LwMouseEvent.MOUSE_PRESSED, e);
    }
  }

  // mouseReleased
  protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
  {
    base.OnMouseUp(e);

    bool drag = (draggOwner != null);
    if (drag)
    {
      fmme(draggOwner, LwMouseMotionEvent.MOUSE_ENDDRAGGED, e.X, e.Y, formMouseMask(e));
      draggOwner = null;
    }

    LwComponent po = pressOwner;
    if (pressOwner != null)
    {
      fme(pressOwner, LwMouseEvent.MOUSE_RELEASED, e);
      if (!drag) fme(pressOwner, LwMouseEvent.MOUSE_CLICKED, e);
      pressOwner = null;
    }

    if (drag || (po != null && po != moveOwner))
    {
      LwComponent nd = getLwComponentAt(e.X, e.Y);
      if (nd != moveOwner)
      {
        if (moveOwner != null) fme(moveOwner, LwMouseEvent.MOUSE_EXITED, e);
        if (isEventable(nd))
        {
          moveOwner = nd;
          fme(nd, LwMouseEvent.MOUSE_ENTERED, e);
        }
      }
    }
  }

  protected override bool IsInputKey (System.Windows.Forms.Keys k) {
    return true;
  }

  protected override bool IsInputChar (char ch) {
    return true;
  }

  protected override void OnKeyPress(System.Windows.Forms.KeyPressEventArgs e)
  {
    base.OnKeyPress(e);
    LwComponent focusOwner = LwFocusManager.manager.getFocusOwner();
    if (focusOwner != null) fke(focusOwner, LwKeyEvent.KEY_TYPED, keyValue, e.KeyChar, keyMask);
  }

  protected override void OnKeyDown(System.Windows.Forms.KeyEventArgs e)
  {
    base.OnKeyDown(e);

    keyMask = formKeyMask(e);
    for (int i=proxy.count()-1; i>=0; i--)
    {
      LwLayer l = (LwLayer)proxy.get(i);
      l.keyPressed(e.KeyValue, keyMask);
      if (l.isActive()) break;
    }

    LwComponent focusOwner = LwFocusManager.manager.getFocusOwner();
    if (focusOwner != null)
    {
      keyValue = e.KeyValue;
      fke(focusOwner, LwKeyEvent.KEY_PRESSED, keyValue, '?', keyMask);
    }
  }

  protected override void OnKeyUp(System.Windows.Forms.KeyEventArgs e)
  {
    base.OnKeyUp(e);

    LwComponent focusOwner = LwFocusManager.manager.getFocusOwner();
    if (focusOwner != null) fke(focusOwner, LwKeyEvent.KEY_RELEASED, keyValue, '?', keyMask);
  }

  // mouseMoved
  protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs e)
  {
    base.OnMouseMove(e);

    if (pressOwner != null)
    {
      OnMouseDrag(e);
      return;
    }

    LwComponent d = getLwComponentAt(e.X, e.Y);
    if (moveOwner != null)
    {
       if (d != moveOwner)
       {
         LwComponent old = moveOwner;
         moveOwner = null;
         fme(old, LwMouseEvent.MOUSE_EXITED, e);

         if (isEventable(d))
         {
           moveOwner = d;
           fme(moveOwner, LwMouseEvent.MOUSE_ENTERED, e);
         }
       }
       else
       {
         if (isEventable(d))
           fmme(d, LwMouseMotionEvent.MOUSE_MOVED, e.X, e.Y, formMouseMask(e));
       }
    }
    else
    if (isEventable(d))
    {
      moveOwner = d;
      fme(d, LwMouseEvent.MOUSE_ENTERED, e);
    }
  }

  protected virtual void OnMouseDrag(System.Windows.Forms.MouseEventArgs e)
  {
    int m = formMouseMask(e);
    m = m | LwConstants.BUTTON1_MASK;

    if (draggOwner == null)
    {
      LwComponent d = (moveOwner == null)?getLwComponentAt(pressX, pressY):moveOwner;
      if (isEventable(d))
      {
        draggOwner = d;
        fmme(draggOwner, LwMouseMotionEvent.MOUSE_STARTDRAGGED, pressX, pressY, m);
        if (pressX != e.X || pressY != e.Y)
          fmme(draggOwner, LwMouseMotionEvent.MOUSE_DRAGGED, e.X, e.Y, m);
      }
    }
    else fmme(draggOwner, LwMouseMotionEvent.MOUSE_DRAGGED, e.X, e.Y, m);
  }

  public Dimension calcPreferredSize(LayoutContainer target) {
    return new Dimension ();
  }

  public void layout(LayoutContainer target)
  {
    int w = getWidth(), h = getHeight();
    for (int i=0;i<target.count();i++)
    {
      Layoutable l = target.get(i);
      l.setSize(w, h);
    }
  }

  public void componentAdded(object id, Layoutable lw, int index)
  {
    LwLayer l = (LwLayer)lw;
    if (layers.get(l.getID()) != null)
      throw new RuntimeException();

    layers.put (l.getID(), lw);
  }

  public void componentRemoved(Layoutable lw, int index) {
    layers.remove (((LwLayer)lw).getID());
  }

  public bool canHaveFocus () {
    return false;
  }

  //
  //  Start undef section
  //
  //
  //

  public void repaint() {
    Invalidate();
  }

  public void repaint(int x, int y, int w, int h)
  {
    System.Drawing.Rectangle r = new System.Drawing.Rectangle();
    r.X      = x;
    r.Y      = y;
    r.Width  = w;
    r.Height = h;
    Invalidate(r);
  }

  public void invalidate() {
    isValidVal = false;
  }

  public bool isValid() {
    return isValidVal;
  }

  public Image createImage(int w, int h) {
    System.Drawing.Image img = new System.Drawing.Bitmap(w, h);
    return new Image (img);
  }

  public Graphics getGraphics() {
    return new Graphics(CreateGraphics());
  }

  public void setVisible (bool b) {
    Visible = b;
  }

  public bool isEnabled() {
    return Enabled;
  }

  public Point getLocation() {
    return new Point(Left, Top);
  }

  public void setLocation(int x, int y) {
    Left = x;
    Top  = y;
  }

  public void setSize(int w, int h) {
    Width  = w;
    Height = h;
  }

  public Dimension getSize() {
    return new Dimension(Width, Height);
  }

  public Rectangle getBounds() {
    return new Rectangle(Left, Top, Width, Height);
  }

  public bool isVisible() {
    return Visible;
  }

  public void requestFocus() {
    Focus();
  }

  //
  //
  //  end undef section
  //
  //


  public object getProperty (int id) {
    if (id == CURSOR_PROPERTY) return cursor;
    else throw new org.zaval.port.cs.IllegalArgumentException();
  }

  public void setProperty (int id, object value)
  {
    if (id == CURSOR_PROPERTY)
    {
      cursor = (Cursor)value;
      Cursor = org.zaval.port.cs.Cursor.getNativeCursor(cursor);
    }
    else throw new org.zaval.port.cs.IllegalArgumentException();
  }

  private void perform (LwAWTEvent e) {
     LwEventManager.manager.perform (e);
  }

  private int formMouseMask(System.Windows.Forms.MouseEventArgs e)
  {
    int mask = 0;
    if (e.Button == System.Windows.Forms.MouseButtons.Left) mask = LwConstants.BUTTON1_MASK;
    else
    if (e.Button == System.Windows.Forms.MouseButtons.Right) mask = LwConstants.BUTTON3_MASK;
    return mask;
  }

  private int formKeyMask(System.Windows.Forms.KeyEventArgs e)
  {
    int mask = 0;
    if (e.Alt)     mask = mask | LwConstants.ALT_MASK;
    if (e.Shift)   mask = mask | LwConstants.SHIFT_MASK;
    if (e.Control) mask = mask | LwConstants.CTRL_MASK;
    return mask;
  }

  /*[find active layer]*/
  private LwLayer fal()
  {
    for (int i=proxy.count()-1; i>=0; i--)
    {
      LwLayer targetLayer = (LwLayer)proxy.get(i);
      if (targetLayer.isActive()) return targetLayer;
    }
    return null;
  }

  private void fmme (LwComponent target, int id, int ax, int ay, int m) {
    MME_STUB.reset(target, id, ax, ay, m, 0);
    LwEventManager.manager.perform (MME_STUB);
  }

  /*[fire key event]*/
  private void  fke(LwComponent target, int id, int keyCode, char keyChar, int keyMask)  {
    KE_STUB.reset(target, id, keyValue, keyChar, keyMask);
    LwEventManager.manager.perform (KE_STUB);
  }

  /*[fire mouse event]*/
  private void fme(LwComponent target, int id, System.Windows.Forms.MouseEventArgs e) {
    ME_STUB.reset(target, id, e.X, e.Y, formMouseMask(e), e.Clicks);
    LwEventManager.manager.perform(ME_STUB);
  }

  private static bool isEventable(LwComponent c) {
    return c != null && c.isEnabled();
  }

  private LwKeyEvent         KE_STUB  = new LwKeyEvent        (null, LwKeyEvent.KEY_PRESSED, 0, 'x', 0);
  private LwMouseEvent       ME_STUB  = new LwMouseEvent      (null, LwMouseEvent.MOUSE_PRESSED, 0, 0, 0, 1);
  private LwMouseMotionEvent MME_STUB = new LwMouseMotionEvent(null, LwMouseMotionEvent.MOUSE_MOVED, 0, 0, 0);
}

}
