/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {

using org.zaval.lw;
using org.zaval.port.cs;
using System.IO;

public class LwToolkit
{
  public const int HORIZONTAL = 1;

  public const int VERTICAL = 2;

  public const int NONE = 0;

  public static readonly  Color darkBlue = new Color(0, 0, 140);

  public static readonly  Font FONT = new Font("dialog", Font.PLAIN, 12);

  public static readonly  Font SFONT = new Font("dialog", Font.PLAIN, 10);

  public static readonly  Font BFONT = new Font("dialog", Font.BOLD, 12);

  public static readonly  FontMetrics FONT_METRICS = new FontMetrics(FONT);

  public static readonly  Color BACK_COLOR = new Color(204, 204, 204);

  public static readonly  Color FG_COLOR = Color.black;

  public static readonly  Dimension PS_SIZE = new Dimension();

  private static readonly  string  resources = "rs/";

  public static LwDesktop createDesktop() {
    return new org.zaval.lw.LwRoot();
  }

  public static LwDesktop getDesktop(LwComponent c) {
    for (;c != null && !(c is LwDesktop);c = c.getLwParent());
    return c==null?null:(LwDesktop)c;
  }

  public static Properties getProperties(string fname)
  {
    StreamReader stream = null;
    try {
      stream = File.OpenText(fname);
      string line = null;
      Properties p = new Properties();
      while ((line = stream.ReadLine()) != null)
      {
        line = line.Trim();
        int index = line.IndexOf('#');
        if (index >= 0) line = line.Substring(0, index);
        line = line.Trim();
        if (line.Length > 0)
        {
          index = line.IndexOf('=');
          if (index > 0)
          {
            p.put (line.Substring(0, index),
                   line.Substring(index + 1));
          }
        }
      }
      return p;
    }
    catch (IOException) {
      return null;
    }
    finally {
      if (stream != null) stream.Close();
    }
  }

  public static Image getRsImage(string name){
    return getImage(resources + name);
  }

  public static Image getImage(string name) {
    return new Image(System.Drawing.Image.FromFile (name));
  }

  public static FontMetrics getFontMetrics(Font f) {
     return f == null?FONT_METRICS:new FontMetrics(f);
  }

  public static void drawDotHLine(Graphics gr, int x1, int x2, int y1)
  {
    System.Drawing.Graphics g = gr.getSrc();
    System.Drawing.Pen      p = new System.Drawing.Pen(gr.getColor().toNativeColor());
    p.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
    g.DrawLine (p, x1, y1, x2, y1);
    p.Dispose();
  }

  public static void drawDotVLine(Graphics gr, int y1, int y2, int x1)
  {
    System.Drawing.Graphics g = gr.getSrc();
    System.Drawing.Pen      p = new System.Drawing.Pen(gr.getColor().toNativeColor());
    p.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
    g.DrawLine (p, x1, y1, x1, y2);
    p.Dispose();
  }

  public static void drawDotRect(Graphics gr, int x, int y, int w, int h)
  {
    System.Drawing.Graphics g = gr.getSrc();
    System.Drawing.Pen      p = new System.Drawing.Pen(gr.getColor().toNativeColor());
    p.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
    g.DrawRectangle (p, x, y, w, h);
    p.Dispose();
  }

  public static LwComponent getDirectChild(LwComponent parent, LwComponent child) {
    for (;child != null && child.getLwParent() != parent; child = child.getLwParent());
    return child;
  }

  public static int getDirectCompAt(int x, int y, LwContainer p)
  {
    for (int i=0; i<p.count(); i++)
    {
      LwComponent c = (LwComponent)p.get(i);
      if (c.isVisible())
      {
        Rectangle b = c.getBounds();
        if (b.contains(x, y)) return i;
      }
    }
    return -1;
  }

  public static bool isAncestorOf(LwComponent p, LwComponent c) {
    while (c != null && c != p) c = c.getLwParent();
    return c != null;
  }

  public static Point getAbsLocation(int x, int y, LwComponent c)
  {
    LwComponent p = null;
    while ((p = c.getLwParent()) != null)
    {
      x += c.getX();
      y += c.getY();
      c = p;
    }
    return new Point (x, y);
  }

  public static Point getAbsLocation(LwComponent c) {
    return getAbsLocation(0, 0, c);
  }

  public static Point getScreenLocation(LwComponent c)
  {
    Point p = getAbsLocation(c);
    System.Windows.Forms.UserControl uc = (System.Windows.Forms.UserControl)getDesktop(c);
    System.Drawing.Point r = uc.PointToScreen(new System.Drawing.Point(p.x, p.y));
    return new Point (r.X, r.Y);
  }

  public static Point getRelLocation(int x, int y, LwComponent c) {
    return getRelLocation(x, y, getDesktop(c), c);
  }

  public static Point getRelLocation(int x, int y, LwComponent target, LwComponent c)
  {
    while (c != target)
    {
      x -= c.getX();
      y -= c.getY();
      c = c.getLwParent();
    }
    return new Point(x, y);
  }

  public static Dimension getScreenSize () {
    return new Dimension(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height);
  }

  public static Graphics getGraphics(LwComponent c) {
    return getGraphics(c, 0, 0, c.getWidth(), c.getHeight());
  }

  public static Graphics getGraphics(LwComponent c, int x, int y, int w, int h)
  {
    LwDesktop nc = getDesktop(c);
    if (nc == null) return null;
    Rectangle vp = c.getVisiblePart();
    if (vp == null) return null;
    Graphics  gr = nc.getGraphics();
    if (gr == null) return null;
    Point     l  = getAbsLocation(c);
    gr.clipRect(vp.x + l.x + x, vp.y + l.y + y, vp.width , vp.height);
    gr.translate(l.x, l.y);
    return gr;
  }

  public static Dimension getMaxPreferredSize(LayoutContainer target)
  {
     int maxWidth = 0, maxHeight = 0;
     for (int i=0; i<target.count(); i++)
     {
       Layoutable l = target.get(i);
       if (l.isVisible())
       {
         Dimension ps = l.getPreferredSize();
         if (ps.width  > maxWidth ) maxWidth = ps.width;
         if (ps.height > maxHeight) maxHeight = ps.height;
       }
     }
     return new Dimension(maxWidth, maxHeight);
  }

  public static Point calcOrigin (int x, int y, int w, int h, LwComponent target) {
    Point origin = target.getOrigin();
    return calcOrigin (x, y, w, h, origin==null?0:origin.x, origin==null?0:origin.y, target);
  }

  public static Point calcOrigin (int x, int y, int w, int h, int px, int py, LwComponent target) {
    return calcOrigin (x, y, w, h, px, py, target, target.getInsets());
  }

  public static Point calcOrigin (int x, int y, int w, int h, int px, int py, LwComponent target, Insets i)
  {
     int dw = target.getWidth(), dh = target.getHeight();
     if (dw > 0 && dh > 0)
     {
       if (dw - i.left - i.right >= w)
       {
         int xx = x + px;
         if (xx < i.left) px += (i.left - xx);
         else
         {
           xx += w;
           if (xx > dw - i.right) px -= (xx - dw + i.right);
         }
       }

       if (dh - i.top - i.bottom >= h)
       {
         int yy = y + py;
         if (yy < i.top) py += (i.top - yy);
         else
         {
           yy += h;
           if (yy > dh - i.bottom) py -= (yy - dh + i.bottom);
         }
       }
       return new Point(px, py);
     }
     else return new Point();
  }


  public static void drawMarker(Graphics g, int x, int y, int w, int h, Color bg, Color fc)
  {
    System.Drawing.Color color = System.Drawing.Color.FromArgb(130, fc.getRed(), fc.getGreen(), fc.getBlue());
    System.Drawing.Graphics gg = g.getSrc();
    gg.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceOver;
    System.Drawing.Brush brush = new System.Drawing.SolidBrush(color);
    gg.FillRectangle(brush, x, y, w, h);
  }

  public static void drawXORRect(LwComponent target, int x, int y, int w, int h)
  {
    Point p = getScreenLocation (target);
    int x1 = x + p.x, y1 = y + p.y;
    int x2 = x1 + w - 1, y2 = y1 + h - 1;
    System.Drawing.Color nc = System.Drawing.Color.Black;
    System.Windows.Forms.ControlPaint.DrawReversibleLine(new System.Drawing.Point (x1, y1),
                                                         new System.Drawing.Point (x2, y1), nc);
    System.Windows.Forms.ControlPaint.DrawReversibleLine(new System.Drawing.Point (x1, y1),
                                                         new System.Drawing.Point (x1, y2), nc);
    System.Windows.Forms.ControlPaint.DrawReversibleLine(new System.Drawing.Point (x2, y1),
                                                         new System.Drawing.Point (x2, y2), nc);
    System.Windows.Forms.ControlPaint.DrawReversibleLine(new System.Drawing.Point (x1, y2),
                                                         new System.Drawing.Point (x2, y2), nc);
  }

  public static void drawXORLine(LwComponent target, int x1, int y1, int x2, int y2)
  {
    Point p = getScreenLocation (target);
    System.Windows.Forms.ControlPaint.DrawReversibleLine(new System.Drawing.Point (p.x + x1, p.y + y1),
                                                         new System.Drawing.Point (p.x + x2, p.y + y2),
                                                         System.Drawing.Color.Black);
  }

  public static void fillXORRect(LwComponent target, int x, int y, int w, int h)
  {
    Point p = getScreenLocation (target);
    System.Windows.Forms.ControlPaint.FillReversibleRectangle(new System.Drawing.Rectangle(p.x + x, p.y + y, w, h),
                                                              System.Drawing.Color.Black);
  }

  public static bool isActionMask(int mask) {
    return mask == 0 || ((mask & org.zaval.port.cs.LwConstants.BUTTON1_MASK) >  0&&
                         (mask & org.zaval.port.cs.LwConstants.BUTTON3_MASK) == 0  );
  }
}

}
