/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

namespace org.zaval.port.cs {

  using System;

  public class Rectangle
  {
    public int x, y, width, height;

    public Rectangle():this(0, 0, 0, 0) {
    }

    public Rectangle(int x, int y, int w, int h)
    {
      this.x = x;
      this.y = y;
      width = w;
      height = h;
    }

    public bool contains(int xx, int yy) {
      return (xx >= x) && ((xx - x) < width) && (yy >= y) && ((yy-y) < height);
    }

    public Rectangle intersection(Rectangle r)
    {
      int x1 = Math.Max(x, r.x);
      int x2 = Math.Min(x + width, r.x + r.width);
      int y1 = Math.Max(y, r.y);
      int y2 = Math.Min(y + height, r.y + r.height);
      return new Rectangle(x1, y1, x2 - x1, y2 - y1);
    }

    public bool intersects(Rectangle r) {
      return !((r.x + r.width <= x) ||
               (r.y + r.height <= y) ||
               (r.x >= x + width) ||
               (r.y >= y + height));
    }

    public override bool Equals(object o) {
      return o is Rectangle && ((Rectangle)o).x == x && ((Rectangle)o).y == y &&
                               ((Rectangle)o).width == width && ((Rectangle)o).height == height;
    }

    public override int GetHashCode() {
      return base.GetHashCode();
    }

    public override string ToString() {
      return "[" + x + ","+  y + "," + width + "," + height + "]";
    }
  }
}
