/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.util {

using org.zaval.port.cs;

public class MathBox
{
  public static Insets max (Insets a, Insets b)
  {
    if (a == null && b == null) return null;
    if (a == null) return b;
    else
    if (b == null) return a;
    else
    return new Insets(System.Math.Max(a.top,   b.top),
                      System.Math.Max(a.left,  b.left),
                      System.Math.Max(a.bottom,b.bottom),
                      System.Math.Max(a.right, b.right));
  }

  public static Dimension max (Dimension a, Dimension b)
  {
    if (a == null && b == null) return null;
    else
    if (a == null) return b;
    else
    if (b == null) return a;
    else
    return new Dimension (System.Math.Max(a.width, b.width), System.Math.Max(a.height, b.height));
  }

  public static Point index2point(int offset, int cols)
  {
     Point p = new Point(-1, -1);
     if (offset < 0 || cols == 0) return p;
     p.x = (offset / cols);
     p.y = (offset % cols);
     return p;
  }

  public static int indexByPoint(int row, int col, int cols) {
    return (cols <= 0)?-1:((row*cols) + col);
  }

  public static bool checkBit(short bits, short mask) {
    return (bits & mask) > 0;
  }

  public static short getBits(short bits, short mask, bool b) {
    return (short) (b?(bits | mask):(bits & ~mask));
  }

  public static Rectangle intersection(int x1, int y1, int w1, int h1,
                                       int x2, int y2, int w2, int h2)
  {
    int xx1 = System.Math.Max(x1, x2);
    int xx2 = System.Math.Min(x1 + w1, x2 + w2);
    int yy1 = System.Math.Max(y1, y2);
    int yy2 = System.Math.Min(y1 + h1, y2 + h2);
    return new Rectangle(xx1, yy1, xx2 - xx1, yy2 - yy1);
  }
}

}
